dojo.provide("dojox.widget.AnalogGauge2.ReallySimpleStyle");

dojo.require("dojo.fx");
dojo.require("dojo.fx.easing");
dojo.require("dojox.widget.AnalogGauge2._Style");

dojo.declare("dojox.widget.AnalogGauge2.ReallySimpleStyle", [dojox.widget.AnalogGauge2._Style], {
    // backgroundColor: String or Object
    //          Dojo color for the background of the gauge.
    // foregroundColor: String or Object
    //          Dojo color for the foreground of the gauge.
    backgroundColor: "black",
    foregroundColor: "white",
    
    getDrawOrder: function(){
        return ["drawBackground", "drawTicks", "drawNeedle"];
    },
    
    getDrawSemantics: function(){
        return {
            background: "drawBackground",
            ticks: "drawTicks",
            needle: "drawNeedle",
            foreground: "drawForeground",
            updateValue: "drawNeedle"
        };
    },
    
    drawBackground: function(settings){
        // summary:
        //          Draws the background of the ReallySimpleStyle-d gauge.
        // description:
        //          Draws a semi-circle with the specified backgroundColor. 
        //          That's it...
        // settings:
        //          The standard style settings object. See 
        //          dojox.widget.AnalogGauge2._Style for more information.
        // returns:
        //          A dojox.gfx.Group containing the Background for the gauge.
        var parentGroup = settings.group;
        var resultGroup = parentGroup.createGroup();
        
        var path = resultGroup.createPath();
        path.moveTo(0, 0).lineTo(-50,0).arcTo(50, 50, 0, 0, 1, 50, 0).lineTo(0,0);
        path.setFill(this.backgroundColor);
        path.setStroke(this.backgroundColor);
        
        return resultGroup;
    },
    
    drawTicks: function(settings){
        // summary: 
        //          Draws the ticks of the ReallySimpleStyle-d gauge. 
        // description: 
        //          Draws the correct number of small and large 
        //          "foregroundColor" lines to represent the major and minor 
        //          ticks on the gauge. 
        // settings: 
        //          The standard style settings object. See  
        //          dojox.widget.AnalogGauge2._Style for more information. 
        // returns: 
        //          A dojox.gfx.Group containing the ticks for the gauge. 
        var parentGroup = settings.group;
        var resultGroup = parentGroup.createGroup();
        
        var minValue = settings.minValue;
        var maxValue = settings.maxValue;
        var majorTickSeparation = settings.majorTickSeparation;
        var minorTickSeparation = settings.minorTickSeparation;
        
        var range = maxValue - minValue;
        var minorDelta = minorTickSeparation / range * 180;
        var majorDelta = majorTickSeparation / range * 180;
        
        var r_outer = 50-2; // 2% margin;
        var r_minor_inner = 50-3; // total 3% length;
        var r_major_inner = 50-5; // total 5% length;
        
        function drawTickMarks(separation, delta, r_inner, color){
            // summary:
            //          Draws tick graphics around a semi-circular gauge given a 
            //          set of parameters.
            // description: 
            //          Draws "color"-colored lines as ticks around a 180 degree 
            //          arc.  It first calculates a polar coordinate, then 
            //          transforms that coordinate into rectangular space.
            // separation: Integer
            //          The separation between the ticks, in the units the gauge 
            //          is displaying.
            // delta: Number
            //          Distance, in degrees, between each tick.  This is pre-
            //          calculated in the code above this function's declaration
            // r_inner: Integer
            //          The inner radius of the tick line (outer radius is pre-
            //          defined outside of this function definition.
            // color: String
            //          Color of the tick marks...
            for(var i = minValue, j = 0; i <= maxValue; i = i + separation, j++){
                // Add 180 degrees here, so we "rotate" the angles by 180 
                //  degrees and the ticks show up on the top half of the circle.
                var theta = (delta*j + 180) * Math.PI / 180;
                var line = resultGroup.createLine({
                    x1: r_outer * Math.cos(theta), y1: r_outer * Math.sin(theta), 
                    x2: r_inner * Math.cos(theta), y2: r_inner * Math.sin(theta)
                });
                line.setStroke(color);
            }
        }
        
        // Draw minor ticks
        drawTickMarks(minorTickSeparation, minorDelta, r_minor_inner, this.foregroundColor);
        // Draw major ticks
        drawTickMarks(majorTickSeparation, majorDelta, r_major_inner, this.foregroundColor);
        
        return resultGroup;
    },
    
    _needleGroup: null,
    _animatableNeedlePart: null,
    drawNeedle: function(settings){
        // summary: 
        //          Draws and animates the gauge's needle.
        // description:
        //          If this._needleGroup is null, we haven't ever drawn a needle 
        //          before.. so we need to do that.  After the initial needle is 
        //          drawn (or if this._needleGroup was not null), we must 
        //          animate the needle from the startValue to the endValue in 
        //          the settings argument.
        // settings: 
        //          The standard style settings object. See  
        //          dojox.widget.AnalogGauge2._Style for more information. 
        // returns: 
        //          A dojox.gfx.Group containing the needle for the gauge. 
        if(this._needleGroup == null || this._animatableNeedlePart == null){
            var parentGroup = settings.group;
            this._needleGroup = parentGroup.createGroup();
            
            var nubbin = this._needleGroup.createCircle({cx: 0, cy: 0, r: 2.5}).setFill(this.foregroundColor);
            var stick = this._needleGroup.createPath();
            stick.moveTo(0,0)
                .lineTo(0,-2.5)
                .lineTo(-40,0)
                .lineTo(0,2.5)
                .lineTo(0,0)
                .setFill(this.foregroundColor)
                .setStroke(null);
            this._animatableNeedlePart = stick;
        }
        
        var minValue = settings.minValue;
        var maxValue = settings.maxValue;
        var startValue = settings.startValue;
        var endValue = settings.endValue;
        
        var range = maxValue - minValue;
        var startRadians = (startValue - minValue) / range * Math.PI;
        var endRadians = (endValue - minValue) / range * Math.PI;
        
        // set initial position
        this._animatableNeedlePart.setTransform(dojox.gfx.matrix.rotate(startRadians));
        
        // Animate it!
        new dojo._Animation({
            curve: [0, 1], 
            duration: 1000, 
            easing: dojo.fx.easing.quadratic,
            onAnimate: dojo.hitch(this, function(step){
                var percent = step;
                var currentRadians = percent * (endRadians - startRadians) + startRadians;
                if(this._animatableNeedlePart != null){
                    this._animatableNeedlePart.setTransform(dojox.gfx.matrix.rotate(currentRadians));
                }
            })
        }).play();
        
        return this._needleGroup;
    },
    
    drawForeground: function(settings){
        // summary:
        //          This is left empty because our style does not need a 
        //          foreground. It is really really simple..
        var parentGroup = settings.group;
        var resultGroup = parentGroup.createGroup();
        
        return resultGroup;
    },
    
    reset: function(){
        // summary:
        //          Deletes any locally-stored shape objects when the gauge 
        //          needs to be reset.
        delete this._needleGroup;
        delete this._animatableNeedlePart;
        
        this._needleGroup = null;
        this._animatableNeedlePart = null;
    }
});