dojo.provide("dojox.widget.AnalogGauge2._Style");
dojo.require("dijit._Widget");

dojo.declare("dojox.widget.AnalogGauge2._Style", [dijit._Widget], {
	 dir: "ltr",
    // summary:
    //          Defines an Abstract Class to describe all valid Style classes 
    //          for the AnalogGauge.
    // description:
    //          Developers can extend this class in order to make their own 
    //          wildly-varying styles for gauges.
    //          
    //          In a break from tradition, this acts as a kind of meta-interface 
    //          which allows developers to override two meta functions 
    //          'getDrawOrder' and 'getDrawSemantics' in order to define their 
    //          own interface.  We also require developers to implement a third 
    //          function, "reset", which will destroy any dojox.gfx objects and 
    //          reset the theme to the beginning when the gauge needs to be 
    //          completely redrawn.
    //
    //          getDrawOrder() should return an array of function names in the 
    //          order that the AnalogGauge class should call them
    //
    //          getDrawSemantics() should return an object containing either 
    //          Strings or arrays of Strings - which denote function names and 
    //          the order in which to call them.  The object **must** have the 
    //          following fields: (and therefore at least 4 drawing functions, 
    //          for the 4 required parts listed..) 
    //          {
    //              background: String,
    //              needle: String,
    //              ticks: String,
    //              foreground: String,
    //              updateValue: String or Array, 
    //              attentionUp: String or Array,   // OPTIONAL
    //              attentionDown: String or Array, // OPTIONAL (required if 
    //                                                  attentionUp exists)
    //              textDisplay: String, // OPTIONAL
    //              trend: String, // OPTIONAL
    //              ceiling: String // OPTIONAL
    //          }
    //          
    //          Each function called-out by getDrawOrder and getDrawSemantics 
    //          should have a similar signature to the following (in terms of 
    //          the arguments they accept):
    //          
    //          myDrawingFunc: function({
    //              group: Object
    //                      The dojox.gfx Group object used by the gauge.  All 
    //                      result groups returned by these functions should be 
    //                      created from this group.
    //              minValue: int
    //                      Minimum displayable value on the gauge.
    //              maxValue: int
    //                      Maximum displayable value on the gauge.
    //              startValue: int
    //                      The previous value for the gauge needle. (used with 
    //                      the endValue for animation of a needle) 
    //              endValue: int
    //                      The current value for the gauge needle.
    //              startTrendValue: int
    //                      The previous value for the trend needle.
    //              endTrendValue: int
    //                      The current value for the trend needle.
    //              startCeilingValue: int
    //                      The previous value for the ceiling needle.
    //              endCeilingValue: int
    //                      The current value for the ceiling needle.
    //              units: String
    //                      Unit string (like ft. or mi. or cubic meters per hr)
    //              thresholds: Array
    //                      Array of dojox.widget.AnalogGaugeThreshold 
    //                      objects, in order of increasing "start" 
    //                      parameter. Styles can use these to do cool stuff 
    //                      by rendering themselves differently based on the 
    //                      location of the needle relative to the 
    //                      thresholds.
    //              minorTickSeparation: int
    //                      Configured distance between minor ticks. If non-
    //                      positive, styles shouldn't render ticks.
    //              majorTickSeparation: int
    //                      Configured distance between major ticks. If non-
    //                      positive, styles shouldn't render ticks.
    //              width: int
    //                      Width (in pixels) of the gauge widget.
    //              height: int
    //                      Height (in pixels) of the gauge widget.
    //          })
    //              
    //          Those functions must also each return a dojox.gfx.Group object, 
    //          so that the Gauge itself may perform necessary transformations 
    //          on the collection of parts generated by a style.
    //
    //          Also, because gauges will be able to perform transformations on 
    //          the groups returned by these functions, it's important to 
    //          remember to draw all shapes with the following in mind:
    //
    //          1. All sizes should be in the range 0-100, where they denote not 
    //              pixels, but percentages.  The gauge can scale itself, so the 
    //              shapes that make it up should be able to be scaled as well. 
    //              These percentages should be interpreted as percentages of the 
    //              gauge's smallest dimension (width or height).
    //          2. All drawing should be centered around the origin. This way the 
    //              gauge can translate itself wherever it needs to (useful in 
    //              the cases where a gauge object is drawn within a larger 
    //              dojox.gfx scope.  
    //
    //          For example, if your gauge needs to have a circular shape with 
    //          diameter equal to the smallest dimension, it should be drawn like 
    //          this:
    //
    //              myGroup.drawCircle({cx: 0, cy: 0, r: 50});
    
    getDrawOrder: function(){
        // summary: 
        //          Returns an array containing a list of functions on the style 
        //          object which should be called and in which order, to render 
        //          the style correctly.
        // returns: Array
        //          List of names, which called in the order as defined, will 
        //          correctly render the gauge in the defining style.
        throw new Error("_Style.getDrawOrder() not implemented yet");
    },
    
    getDrawSemantics: function(){
        // summary:
        //          Returns an object containing semantic descriptions paired with 
        //          function names or arrays of function names which fulfill those 
        //          descriptions.
        // returns: Object
        //          See the summary..
        throw new Error("_Style.getDrawSemantics() not implemented yet");
    },

    adjust: function(opt){
        // summary:
        //          This function is used to adjust any of the attributes of the 
        //          gauge. It could be abstracted further to provide resize 
        //          capability, for example.
        // opt: Object
        //          Object containing key/value pairs to mix into the gauge.
        //
		// *note: You really do not have to implement this as the default is to mixin
        dojo.mixin(this, opt);
    },

	reset: function(){
        // summary:
        //          Resets the style so that the gauge can be re-drawn from 
        //          scratch.  For example, implementations should delete all 
        //          dojox.gfx objects they are storing locally in their reset 
        //          function.
        throw new Error("_Style.reset() not implemented yet");
    },
    
    getThresholdFromValue: function(value, thresholds){
        // summary:
        //          Returns the threshold from the sorted array that the value 
        //          resides within.
        // value: Number
        //          Value to find the threshold for.
        // thresholds: Array
        //          Sorted array of thresholds, sorted by their "start" 
        //          attribute.
        // returns:
        //          The threshold from the array where the value resides, or 
        //          thresholds[0] if can't find it anywhere...
        
        if (thresholds.length==0) 
              return {name: "Normal", status: "Normal", start: 0};

		for(var i = 0, len = thresholds.length; i < len; i++){
            if(i == len-1){
                if(value < thresholds[i].start){
                    return thresholds[0];
                } else {
                    return thresholds[i];
                }
            }
            if(value >= thresholds[i].start && value < thresholds[i+1].start){
                return thresholds[i];
            }
        }

		return thresholds[0];
    }

});
