/**
 * This class is responsible for providing the mechanisms to make direct AJAX 
 * calls to a NavModel API server.  It can be thought of as a simpler stepping 
 * stone towards the RAWNavModelStore and the iWidgetNavModelStore.  
 *
 * You can use this class to in conjunction with Diplomat on your personal 
 * computer for testing your UIs against a live NavModel API.
 *
 * @author Jason Feinstein (jwfeinst@us.ibm.com)
 */
(function(dojo){
    dojo.provide("ibm.tivoli.data.AJAXNavModelStore");
    dojo.require("ibm.tivoli.data._NavModelStore");
    dojo.require("ibm.tivoli.data.Utilities");
    
    dojo.declare("ibm.tivoli.data.AJAXNavModelStore",[ibm.tivoli.data._NavModelStore],{
        /**
         * Called when an instance of this 'class' is created, by dojo. Knows 
         * how to accept a single argument, an object, structured like this:
         * 
         * {
         *        TODO: need a description here.
         * }
         */
        constructor: function(kwArgs){
            var settings = {
                baseUrl: "",
                provider: "",
                dataset: "",
                datasource: "",
                parameters: {},
                defaultProperties: ['all'],
                version: 0.1
            };
            dojo.mixin(settings,kwArgs);
            dojo.mixin(this, settings);
        },
        
        /**
         * Gets the implementation-specific settings for the implementing NavModelStore.
         */
        _getNavModelSettings: function(){
            var settings = {};
            settings.baseUrl = this.baseUrl;
            settings.provider = this.provider;
            settings.dataset = this.dataset;
            settings.datasource = this.datasource;
            settings.parameters = this.parameters;
            settings.defaultProperties = this.defaultProperties;
            settings.version = this.version;

            // we have nothing here, because we're the 'root' data store.
            return settings;
        },

        /**
         * Perform the function nessecary to asynchronously get data for a fetch.  
         * We don't need to do anything with the data but make sure it's in the 
         * NavModel API format.
         * 
         * NOTE:
         *   If itemTypes is larger than one element, conditions will be ignored.
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean
         *   parameters: {
         *      (param_) paramName : paramValue
         *   }
         *   start: int,
         *   count: int,
         *   sort: [
         *       { attribute: "property name", descending: true or false },
         *       { attribute: "property name", descending: true or false }
         *   ]
         *   query: {
         *       navModel: {
         *           parameters: {} // more parameters, they'll get mixed in with the others above,
         *           itemTypes: ['link','task','item'], // defaults to ['link','task','item','childDataset']
         *           origin: boolean, // defaults to false, if true, itemTypes is ignored and we only obtain 'item's.
         *           additionalContent: {
         *               'key' : 'value',
         *               'key2' : 'value2'
         *               ...
         *           }, // defaults to {} (empty object, no key/value pairs)
         *           search: {
         *               term: 'search term',
         *               attributes: ["property name", "property name 2", ...]
         *           }
         *       },
         *       id1 : value1,  // assumes we're trying for equality
         *       id2 : {
         *          value: value2,
         *          op: operator 
         *       },
         *       id3 : {
         *          value: value3,
         *          op: "BETWEEN",      // if op == "BETWEEN", look for value2, 
         *                              //   so we can use the BETWEEN operator 
         *                              //   for a range.
         *          value2: value4 
         *       },
         *       id4 : {
         *          value: "value2",    // if value is a string, look for 
         *                              //   sensitive boolean 
         *          op: operator,
         *          sensitive: true|false 
         *       },
         *       id5 : {
         *          value: [array],     // if it's an array, we're looking for 
         *                              //   the "IN" operator.
         *          op: "IN"
         *       }
         *   },
         *   queryOptions: {
         *       ignoreCase: false
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getDataForFetch: function(kwArgs){
            var settings = {
                onLoad: function(){
                    console.log("OnLoad Called: ", Array.prototype.slice.call(arguments));
                },
                onError: function(){
                    console.log("OnError Called: ", Array.prototype.slice.call(arguments));
                },
                scope: null,
                synchronous: false,
                parameters: {},
                start: 0,
                count: 50,
                sort: [],
                query: {
                    navModel: {
                        parameters: this.parameters,
                        itemTypes: ['item','task','link','childDataset'],
                        origin: false,
                        additionalContent: {}
                    }
                },
                properties: this.defaultProperties
            };
            dojo.advancedMixin(settings, kwArgs);
            dojo.advancedMixin(settings.parameters, settings.query.navModel.parameters);
            
            if(settings.query == null){
                settings.query = {};
                settings.query.navModel = {
                    parameters: {},
                    itemTypes: ['item','task','link','childDataset'],
                    origin: false
                };
            }

            var content = null;
            var url = null;
            try {
                content = this._buildContent(settings);
                url = this._buildFetchURL(settings);
            } catch (e){
                settings.onError.call(settings.scope, e);
                throw e;
            }
            
            return dojo.xhrGet({
                url: this._buildFetchURL(settings),
                handleAs: "json",
                sync: settings.synchronous,
                content: content,
                load: function(result, ioArgs){
                    /*
                     * Do other stuff...
                     
                    var resultData = result;
                    
                    if(settings.properties != null){
                        var items = resultData.items;
                        if(items != null){
                            var items_length = items.length;
                            for(var i = 0; i < items_length; i++){
                                items[i].__loaded = true;
                            }
                        }
                    }
                    */
                    
                    /*
                     * Call the callback.
                     */
                    settings.onLoad.call(settings.scope, result, ioArgs);
                },
                error: function(error, ioArgs){
                    settings.onError.call(settings.scope, error, ioArgs);
                }
            });
        },
        
        _buildFetchURL: function(settings){
            if(settings.query != null && settings.query.navModel != null){
                var itemTypes = settings.query.navModel.itemTypes;
                if(itemTypes != null){
                    if(itemTypes[0] === "item" && settings.query.navModel.origin === true){
                        return this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/originItems";
                    } else if(itemTypes.length == 1 && itemTypes[0] != "childDataset"){
                        return this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/"+itemTypes[0]+"s";
                    } 
                } 
            }
            return this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset;            
        },
        
        /**
         * Builds the "content" object to use for the dojo.xhr part of a fetch.
         *
         * @param settings
         *     The whole, sanitized, settings object from this.getDataForFetch.
         *
         * @return
         *     Content object ready for use in a call to one of dojo's XHR 
         *     methods.
         */
        _buildContent: function(settings){
            var result = {
                version: settings.version,
                start: settings.start
            };
            if(settings.count !== null){
                result.count = settings.count;
            }
            
            /**
             * If we have a multi-type selection, get the stuff straight.
             */
            if(settings.query != null && settings.query.navModel != null){
                if(settings.query.navModel.itemTypes != null){
                    if(settings.query.navModel.itemTypes.length > 1){
                        var types = settings.query.navModel.itemTypes;
                        for(var i = 0; i < types.length; i++){
                            types[i] = types[i] == "task" ? "toolbarTask" : types[i];
                            result[types[i]+"s"] = true;
                        }
                        if(settings.linkProperties != null){
                            result.linkProperties = settings.linkProperties.join(",");
                        }
                        if(settings.itemProperties != null){
                            result.itemProperties = settings.itemProperties.join(",");
                        }
                    } else {
                        if(settings.properties != null && settings.properties !== ""){
                            result.properties = settings.properties.join(",");
                        }
                    }
                } else {
                    if(settings.properties != null && settings.properties !== ""){
                        result.properties = settings.properties.join(",");
                    }
                }
                
                var additionalContent = settings.query.navModel.additionalContent;
                if(additionalContent != null){
                    for(var i in additionalContent){
                        if(additionalContent.hasOwnProperty(i)){
                            result[i] = additionalContent[i];
                        }
                    }
                }
            } else {
                if(settings.properties != null && settings.properties !== ""){
                    result.properties = settings.properties.join(",");
                }
            }
            
            /*
             * build sort and condition props.
             */
            var sorting = this._buildSorting(settings);
            var conditions = this._buildConditions(settings);
            if(sorting !== ""){
                result.sort = sorting;
            }
            if(conditions !== ""){
                result.condition = conditions;
            }
            
            /*
             * Build the search stuff...
             */
            console.log("Getting ready to create search terms");
            if(settings.query != null && settings.query.navModel != null){
                if(settings.query.navModel.search != null){
                    console.log("Creating search terms...");
                    result.search = settings.query.navModel.search.term;
                    result.searchProperties = settings.query.navModel.search.attributes.join(",");
                }
            }
            
            
            /*
             * Gather the parameters....
             */
            var baseParams = dojo.clone(settings.parameters);
            var queryParams = [];
            if(settings.query!= null && settings.query.navModel != null){
                settings.query.navModel.parameters;
            }
            dojo.mixin(baseParams,queryParams);
            for(var i in baseParams){
                if(baseParams.hasOwnProperty(i)){
                    result["param_"+i] = baseParams[i];
                }
            }
            
            
            return result;
        },
        
        /**
         * Builds a sorting string (to be used in the "content" part of a 
         * dojo.xhr call.  
         *
         * @param settings
         *      Settings object from the getDataForFetch and getRelatedItems 
         *      methods.
         *      
         *      The "sort" parameter of the settings object must conform to
         *        {
         *            attribute: attribute,    // attribute id,
         *            descending: true|false   // Optional.  Default is false.
         *        }
         *      
         * @return
         *      We're looking for origin nodes or more than one type of element,
         *      we don't allow conditions, so we'll just return "".
         *      
         *      Otherwise, we'll build up the condition string as defined in the 
         *      API spec and given the way the settings object is designed.
         */
        _buildSorting: function(settings){
            var sortArray = [];
            if(settings.sort == null){
                return "";
            }
            if(dojo.isArray(settings.sort)){
                sortArray = settings.sort;
            } else if (dojo.isObjectObject(settings.sort)){
                sortArray.push(settings.sort);
            }
            var sortConditions = [];
            
            var sortArray_length = sortArray.length;
            for(var i=0; i < sortArray_length; i++){
                var element = sortArray[i];
                if(!dojo.isObjectObject(element)){
                    throw new Error("Invalid Sorting Specified.");
                }
                element.descending = element.descending != null ? element.descending : false;
                sortConditions.push((element.descending ? "-" : "+")+element.attribute);
            }
            
            return sortConditions.join(",");
        },
        
        /**
         * Builds a condition string (to be used in the "content" part of a 
         * dojo.xhr call.  
         *
         * @param settings
         *      Settings object from the getDataForFetch and getRelatedItems 
         *      methods.  
         *      
         * @return
         *      We're looking for origin nodes or more than one type of element,
         *      we don't allow conditions, so we'll just return "".
         *      
         *      Otherwise, we'll build up the condition string as defined in the 
         *      API spec and given the way the settings object is designed.
         */
        _buildConditions: function(settings){
            if(settings.query == null){
                return "";
            }
            if(settings.query.navModel.origin === true || settings.query.navModel.itemTypes.length > 1){
                return "";
            }
            
            /*
             * Iterate through the query.
             */
            var query = settings.query;
            var conditions = [];
            for(var i in query){
                if(query.hasOwnProperty(i) && i != "navModel"){
                    var condition = query[i];
                    
                    /*
                     * If it's not an Object, it's a literal equality condition, 
                     * so just push it on and get on with our merry way.
                     */
                    if(!dojo.isObjectObject(condition)){
                        if(dojo.isString(condition)){
                            var splitCondition = /^(\*?)([^*]*)(\*?)$/.exec(condition);
                            if(splitCondition[1] !== "" && splitCondition[3] !== ""){
                                conditions.push(i + " CONTAINS '" + splitCondition[2] +"'");
                            } else if(splitCondition[1] === "*"){
                                conditions.push(i + " ENDS '" + splitCondition[2] +"'");
                            } else if(splitCondition[3] === "*"){
                                conditions.push(i + " STARTS '" + splitCondition[2] +"'");
                            } else {
                                conditions.push(i + " = '" + condition + "'");
                            }
                        } else {
                            conditions.push(i + " = " + condition);
                        }
                        continue;
                    }
                    
                    /*
                     * If it was an object, then check the value's type...
                     */
                    var value = condition.value; 
                    var operator = condition.op;
                    var conditionPieces = [];
                    conditionPieces.push(i);
                    conditionPieces.push(operator);
                    conditionPieces.push(value);
                    
                    if(dojo.isString(value)){
                        /*
                         * If it's a string, we should check for the 'sensitive' 
                         * attribute, also - pop the value back off, and push on 
                         * a version surrounded by single quotes.
                         */
                        conditionPieces.pop();
                        conditionPieces.push("'"+value+"'");
                         
                        var sensitive = settings.queryOptions.ignoreCase ? "" : "SENSITIVE";
                        sensitive = condition.sensitive != null ? (condition.sensitive ? "SENSITIVE" : "") : sensitive;
                        if(sensitive !== ""){
                            conditionPieces.push(sensitive);
                        }
                    } else if (dojo.isArray(value)){
                        /*
                         * If it's an array, we know we're doing an "IN" 
                         * operation.
                         */
                        var options = [];
                        
                        // pop the value off, we need to re-process it.
                        conditionPieces.pop();
                        if(dojo.isString(value[0])){
                            conditionPieces.push("('"+value.join("', '")+"')");
                        } else {
                            conditionPieces.push("("+value.join(", ")+")");
                        }
                    } else {
                        /*
                         * It's likely a number if it wasn't an array or a 
                         * string.
                         */
                        if(operator === "BETWEEN"){
                            conditionPieces.push("AND");
                            conditionPieces.push(condition.value2);
                        } else if (condition.op2 && condition.value2) {
                            conditionPieces.push("AND");
                            conditionPieces.push(condition.op2);
                            conditionPieces.push(condition.value2);
                        }				
                    }
                    
                    conditions.push(conditionPieces.join(" "));
                }
            }
            return conditions.join(" AND ");
        },
        
        /**
         * Perform the function nessecary to asynchronously get data for a fetch 
         * of related items.
         *  
         * We don't need to do anything with the data but make sure it's in the 
         * NavModel API format.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   sourceItem: item, // item to get the related items for.
         *
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   start: int,
         *   count: int,
         *   sort: [
         *       { attribute: "property name", descending: true or false },
         *       { attribute: "property name", descending: true or false }
         *   ]
         *   query: {
         *       navModel: {
         *           parameters: {} // more parameters, they'll get mixed in with the others above,
         *           origin: boolean, // defaults to false, if true, itemTypes is ignored and we only obtain 'item's.
         *           additionalContent: {
         *               'key' : 'value',
         *               'key2' : 'value2'
         *               ...
         *           } // defaults to {} (empty object, no key/value pairs)
         *       },
         *       id1 : value1,  // assumes we're trying for equality
         *       id2 : {
         *          value: value2,
         *          op: operator 
         *       },
         *       id3 : {
         *          value: value3,
         *          op: "BETWEEN",      // if op == "BETWEEN", look for value2, 
         *                              //   so we can use the BETWEEN operator 
         *                              //   for a range.
         *          value2: value4 
         *       },
         *       id4 : {
         *          value: "value2",    // if value is a string, look for 
         *                              //   sensitive boolean 
         *          op: operator,
         *          sensitive: true|false 
         *       },
         *       id5 : {
         *          value: [array],     // if it's an array, we're looking for 
         *                              //   the "IN" operator.
         *          op: "IN"
         *       }
         *   },
         *   queryOptions: {
         *       ignoreCase: false
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getRelatedItems: function(kwArgs){
            var settings = {
                onLoad: function(){
                    console.log("OnLoad Called: ", Array.prototype.slice.call(arguments));
                },
                onError: function(){
                    console.log("OnError Called: ", Array.prototype.slice.call(arguments));
                },
                scope: null,
                synchronous: false,
                
                sourceItem: null,
                
                parameters: {},
                start: 0,
                count: 50,
                sort: [],
                query: {
                    navModel: {
                        parameters: this.parameters,
                        itemTypes: ['item'],
                        origin: false,
                        additionalContent: {}
                    }
                },
                queryOptions: {
                    ignoreCase: false
                },
                properties: this.defaultProperties
            };
            dojo.advancedMixin(settings, kwArgs);
            dojo.advancedMixin(settings.parameters, settings.query.navModel.parameters);
            
            if(settings.sourceItem == null){
                throw new Error("No Source Item Specified!");
            }
            if(!this._isItemItem(settings.sourceItem)){
                throw new Error("Specified sourceItem");
            }
            
            
            var content = null;
            var url = null;
            try {
                content = this._buildContent(settings);
                url = this._buildRelatedItemsURL(settings);
            } catch (e){
                settings.onError.call(settings.scope, e);
                throw e;
            }
            
            console.log("Content and URL: ",content,url);
            
            return dojo.xhrGet({
                url: url,
                handleAs: "json",
                sync: settings.synchronous,
                content: content,
                load: function(result, ioArgs){
                    /*
                     * Do other stuff...
                     */
                    var resultData = result;
                    
                    /*
                     * Call the callback.
                     */
                    settings.onLoad.call(settings.scope, resultData, ioArgs);
                },
                error: function(error, ioArgs){
                    settings.onError.call(settings.scope, error, ioArgs);
                }
            });
        },
        
        _buildRelatedItemsURL: function(settings){
            if(settings.sourceItem == null){
                return this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/originItems";
            } else {
                return this.baseUrl + settings.sourceItem.relatedItemsUri;
            }
            /*
            if(settings.query != null && settings.query.navModel != null){
                var itemTypes = settings.query.navModel.itemTypes;
                if(itemTypes[0] === "item" && settings.query.navModel.origin === true){
                    return this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/originItems";
                } else if(itemTypes.length == 1){
                    return this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset + "/"+itemTypes[0]+"s";
                } 
            }
            return this.baseUrl + "/providers/"+this.provider+(this.datasource !== "" ?  ("/datasources/" + this.datasource) : "") + "/datasets/"+ this.dataset; 
            */
        },
        
        /**
         * Responsible for getting an individual element by it's ID.
         */
        _getIndividualElementById: function(kwArgs){
            var settings = {
                onLoad: function(result, args){
                    // nothing.
                    console.dir(result);
                },
                onError: function(error, args){
                    console.dir(error);
                },
                scope: null,
                synchronous: false,
                item: null,
                parameters: {},
                properties: ['all']
            };
            dojo.mixin(settings, kwArgs);
            dojo.mixin(settings.parameters, this.parameters);
            
            var content = {
                version: settings.version,
                properties: (settings.properties.join(","))
            };
            for(var i in settings.parameters){
                if(settings.parameters.hasOwnProperty(i)){
                    if(/^param_/.exec(i) == null){
                        content["param_"+i] = settings.parameters[i];
                    } else {
                        content[i] = settings.parameters[i];
                    }
                }
            }
        
            var responseItem = null;
            
            dojo.xhrGet({
                url: this._buildGetElementURL(settings),
                load: function(response, ioArgs){
                    dojo.mixin(ioArgs,settings);
                    settings.onLoad.call(settings.scope, response, ioArgs);
                    responseItem = response;
                }, 
                error: function(error, ioArgs){
                    dojo.mixin(ioArgs,settings);
                    settings.onError.call(settings.scope, error, ioArgs);
                },
                handleAs: "json",
                sync: settings.synchronous,
                content: content
            });
            return responseItem;
        },
        
        _buildGetElementURL: function(settings){
            return this.baseUrl + settings.item.uri;
        },
        
        /**
         * Retrieve just one item, by its id.  This should be able to work both 
         * asynchronously and synchronously.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   item: itemItem, // item to get the related items for.
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getItemById: function(kwArgs){
            if(kwArgs.item == null){
                throw new Error("No item specified in the arguments for AJAXNavModelStore.getItemById()");
            }
            if(!this.isItem(kwArgs.item)){
                kwArgs.item = {
                    uri: "/providers/"+this.provider
                            +((this.datasource != null && this.datasource != "") ? "/datasources/"+this.datasoruce : "")
                            +"/datasets/"+this.dataset
                            +"/items/"+kwArgs.item
                };
            }
            
            return this._getIndividualElementById(kwArgs);
        },
        
        /**
         * Retrieve just one task, by its id.  This should be able to work both 
         * asynchronously and synchronously.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   link: linkItem, // item to get the related items for.
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getLinkById: function(kwArgs){
            if(kwArgs.item == null){
                throw new Error("No link specified in the arguments for AJAXNavModelStore.getLinkById()");
            }
            if(!this.isItem(kwArgs.item)){
                kwArgs.item = {
                    uri: "/providers/"+this.provider
                            +((this.datasource != null && this.datasource != "") ? "/datasources/"+this.datasoruce : "")
                            +"/datasets/"+this.dataset
                            +"/links/"+kwArgs.item
                };
            }
            
            return this._getIndividualElementById(kwArgs);
        },
        
        /**
         * Retrieve just one task, by its id.  This should be able to work both 
         * asynchronously and synchronously.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   task: taskItem, // item to get the related items for.
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getTaskById: function(kwArgs){
            if(kwArgs.item == null){
                throw new Error("No task specified in the arguments for AJAXNavModelStore.getTaskById()");
            }
            if(!this.isItem(kwArgs.item)){
                kwArgs.item = {
                    uri: "/providers/"+this.provider
                            +((this.datasource != null && this.datasource != "") ? "/datasources/"+this.datasoruce : "")
                            +"/datasets/"+this.dataset
                            +"/tasks/"+kwArgs.item
                };
            }
            
            return this._getIndividualElementById(kwArgs);
        },
        
        /**
         * Build a new instance of an implementation of _NavModelStore, based on 
         * a child dataset of the dataset described by this data store.
         *
         * @param item
         *      Must be a valid Child Dataset "item" object.  If not, an 
         *      exception will be thrown.
         * @return 
         *      New instance of a _NavModelStore implementation (the same 
         *      implementation as the calling object)
         */
        createDataStoreFromChildDataset: function(item){
            if(!this._isChildDatasetItem(item)){
                throw new Error("Specified item is not a valid child dataset.");
            }
            
            var id = item.id;
            var uriRegex = /\/providers\/([a-zA-Z0-9\._-]+)(\/datasources\/([a-zA-Z0-9\._-]+))?\/datasets\/([a-zA-Z0-9\._-]+)/;
            var match = uriRegex.exec(item.uri);
            var provider = match[1] === undefined ? "" : match[1];
            var datasource = match[3] === undefined ? "" : match[3];
            var dataset = match[4] === undefined ? "" : match[4];
            
            return new (eval(this.declaredClass))({
                dataset: id,
                baseUrl: this.baseUrl,
                datasource: datasource,
                provider: provider,
                defaultProperties: this.defaultProperties,
                parameters: this.parameters
            });
        }
    });
})(dojo);
