/**
 * This file contains the definition of the NavModelStore Abstract class.  It 
 * is not meant to be used outright, but instead as an extention point for 
 * different kinds of access mechanisms.
 *
 * For example, the ibm.tivoli.data.AJAXNavModelStore implementation is meant 
 * for retrieving information from a NavModel Rest service using simple AJAX.  
 * Alternatively, the ibm.tivoli.data.WidgetNavModelStore implementation first 
 * performs iWidget URL sanitization before any AJAX calls are made.  The idea 
 * could even be extended further to offer a flat-file based simulation of a 
 * NavModel (good for use with testing).
 * 
 * The following functions should be implemented in an extension of this class,
 * (see the code below for information about the functions themselves):
 *     getDataForFetch(kwArgs)
 *     getRelatedItems(kwArgs)
 *     getItemById(kwArgs)
 *     getLinkById(kwArgs)
 *     getTaskById(kwArgs)
 *
 * @author Jason Feinstein (jwfeinst@us.ibm.com)
 */
(function(dojo){
    dojo.provide("ibm.tivoli.data._NavModelStore");
    
    /*
     * APIs we plan to implement.
     */
    dojo.require("dojo.data.api.Identity");
    dojo.require("dojo.data.api.Notification");
    dojo.require("dojo.data.api.Write");
    dojo.require("dojo.data.api.Read");
    dojo.require("dojo.data.api.Request");
    dojo.require("ibm.tivoli.data.api.Relationship");
    dojo.require("ibm.tivoli.data.Utilities");
    
    /*
     * Other imports
     */
    dojo.require("dojo.data.api.Request");
    
    /*
     * Local scope variables, to use for setting "static" variables to (see the 
     * end of the file).
     */
    var navModelStoreLog = "ibm.tivoli.data._NavModelStore.log";
    
    function isNullish(val){
        return (val == null);
    }
    
    /**
     * Lazy-loading data store for NavModel REST Service. 
     */
    dojo.declare("ibm.tivoli.data._NavModelStore", [], {
        cache: {
            items: {},
            links: {},
            tasks: {},
            childDatasets: {}
        },
        
        
        /*********
         *    Functions to implement/override in your extensions.
         *********/
         
        /**
         * Gets the implementation-specific settings for the implementing NavModelStore.
         */
        _getNavModelSettings: function(){
            // we have nothing here, because we're the 'root' data store.
            return {};
        },
        
        /**
         * Perform the function nessecary to asynchronously get data for a fetch.  
         * We don't need to do anything with the data but make sure it's in the 
         * NavModel API format.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean
         *   parameters: {
         *      (param_) paramName : paramValue
         *   }
         *   start: int,
         *   count: int,
         *   sort: [
         *       { property: "property name", direction: "asc" or "desc" },
         *       { property: "property name", direction: "asc" or "desc" }
         *   ]
         *   query: {
         *       parameters: {} // more parameters, they'll get mixed in with the others above,
         *       
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getDataForFetch: function(kwArgs){
            throw new Error("getDataForFetch has not yet been implemented.");
        },
        
        /**
         * Perform the function nessecary to asynchronously get data for a fetch 
         * of related items.
         *  
         * We don't need to do anything with the data but make sure it's in the 
         * NavModel API format.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   sourceItem: item, // item to get the related items for.
         *
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   start: int,
         *   count: int,
         *   sort: [
         *       { property: "property name", direction: "asc" or "desc" },
         *       { property: "property name", direction: "asc" or "desc" }
         *   ]
         *   query: {
         *       parameters: {} // more parameters, they'll get mixed in with the others above,
         *       
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getRelatedItems: function(kwArgs){
            throw new Error("getRelatedItems has not yet been implemented.");
        },
        
        /**
         * Retrieve just one item, by its id.  This should be able to work both 
         * asynchronously and synchronously.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   item: itemItem, // item to get the related items for.
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getItemById: function(kwArgs){
            throw new Error("getItemById has not yet been implemented.");
        },
        
        /**
         * Retrieve just one task, by its id.  This should be able to work both 
         * asynchronously and synchronously.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   link: linkItem, // item to get the related items for.
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getLinkById: function(kwArgs){
            throw new Error("getLinkById has not yet been implemented.");
        },
        
        /**
         * Retrieve just one task, by its id.  This should be able to work both 
         * asynchronously and synchronously.
         * 
         * 
         * kwArgs = {
         *   onLoad: function(result, kwArgs+ioArgs){},
         *   onError: function(kwArgs+ioArgs){},
         *   scope: object,  // scope within which to call onLoad and onError.
         *   synchronous: boolean,
         *
         *   task: taskItem, // item to get the related items for.
         *   parameters: {
         *      (param_) paramName : paramValue
         *   },
         *   properties: [] // array of properties to "pre-load"
         * }
         */
        _getTaskById: function(kwArgs){
            throw new Error("getTaskById has not yet been implemented.");
        },
        
        /**
         * Build a new instance of an implementation of _NavModelStore, based on 
         * a child dataset of the dataset described by this data store.
         *
         * @param item
         *      Must be a valid Child Dataset "item" object.  If not, an 
         *      exception will be thrown.
         * @return 
         *      New instance of a _NavModelStore implementation (the same 
         *      implementation as the calling object)
         */
        createDataStoreFromChildDataset: function(item){
            throw new Error("createDataStoreFromChildDataset has not yet been implemented.");
        },
        
        /********
         *    Functions to leave alone if possible.
         ********/
        
        /**
         * Called before the constructor is called, by dojo.
         */
        preamble: function(){
            this._TOPIC_LOG = navModelStoreLog;//= ibm.tivoli.data._NavModelStore.TOPIC_LOG;
            this._logLevel = 1;
        },
        
        /**
         * Called after the constructor is called, by dojo.
         */
        postscript: function(){
            /*
             * Reset the cache...
             */
            this.resetCache();
            /*
             * Set a timer that will, every 5 minutes, reset the cache.
             */
            if(!isNullish(this.cacheInterval)){
                clearInterval(this.cacheInterval);
            }
            this.cacheInterval = setInterval(dojo.hitch(this,this.resetCache), 300000);
        },
        
        // TODO: do this.
        resetCache: function(){
            this.cache.items = {};
            this.cache.links = {};
            this.cache.tasks = {};
            this.cache.childDatasets = {};
        },
        
        
        /**
         * summary:
         *        Returns a single attribute value.
         *        Returns defaultValue if and only if *item* does not have a value for *attribute*.
         *        Returns null if and only if null was explicitly set as the attribute value.
         *        Returns undefined if and only if the item does not have a value for the
         *        given attribute (which is the same as saying the item does not have the attribute). 
         * description:
         *        Saying that an "item x does not have a value for an attribute y"
         *        is identical to saying that an "item x does not have attribute y". 
         *        It is an oxymoron to say "that attribute is present but has no values" 
         *        or "the item has that attribute but does not have any attribute values".
         *        If store.hasAttribute(item, attribute) returns false, then
         *        store.getValue(item, attribute) will return undefined.
         *
         * exceptions:
         *        Throws an exception if *item* is not an item, or *attribute* is not a string
         *
         * @param {Object} item The item to access values on.
         * @param {Object} attribute The attribute to access represented as a 
         *                     string.
         * @param {Object} defaultValue Optional.  A default value to use for 
         *                     the getValue return in the attribute does not exist 
         *                     or has no value.
         */
        getValue: function(item, attribute, defaultValue, actualValue){
            if(actualValue === undefined){
                actualValue = false;
            }
            
            var values = this.getValues(item, attribute, actualValue);
            var hasAttribute = this.hasAttribute(item,attribute);
            
            if(!hasAttribute){
                if(!isNullish(defaultValue)){
                    return defaultValue;
                } else {
                    return undefined;
                }
            }
            
            if(values.length === 0){
                if(!isNullish(defaultValue)){
                    return defaultValue;
                } else {
                    return null;
                }
            }
            
            return values[0]; 
        },

        /**
         * This getValues() method works just like the getValue() method, but getValues()
         * always returns an array rather than a single attribute value.  The array
         * may be empty, may contain a single attribute value, or may contain
         * many attribute values.
         * If the item does not have a value for the given attribute, then getValues()
         * will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
         * has a return of false, then store.getValues(item, attribute) will return [].)
         * 
         * exceptions:
         * Throws an exception if *item* is not an item, or *attribute* is not a string
         * 
         * example:
         * var friendsOfLuke = store.getValues(lukeSkywalker, "friends");
         * 
         * @param {Object} item The item to access values on.
         * @param {Object} attribute The attribute to access represented as a string.
         */
        getValues: function(item, attribute){
            if(arguments.length < 2){
                throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
            }
            if(isNullish(item)){
                throw new Error("Specified item argument is null/undefined.");
            }
            if(!this.isItem(item)){
                throw new Error("Specified item argument is not a valid item.");
            }
            if(!dojo.isString(attribute)){
                throw new Error("Specified attribute argument is not a string.");
            }
            if(!this.isItemLoaded(item)){
                item = this.loadItem({item: item, synchronous:true}); // TODO: ADD SUPPORT FOR THIS.
            }
            var valueAttribute = 'displayValue';
            if(arguments.length == 3){
                if(arguments[2] === true){
                    valueAttribute = 'value';
                }
            }
            
            /*
             * If we don't have the attribute, return an empty array.
             */
            if(!this.hasAttribute(item, attribute)){
                return [];
            }
            
            /*
             * If the item is a top-level attribute, use it.
             */
            if(!isNullish(item[attribute])){
                if(dojo.isArray(item[attribute])){
                    return item[attribute];
                } else {
                    return [item[attribute]];
                }
            }
            
            /*
             * Iterate through the properties object if there is one.
             */
            if(!isNullish(item.properties)){
                if(dojo.isArray(item.properties)){
                    for(var i = 0; i < item.properties.length; i++){
                        if(item.properties[i].id == attribute){
                            var value = item.properties[i][valueAttribute];
                            if(dojo.isArray(value)){
                                return value;
                            } else {
                                return [value];
                            }
                        }
                    }
                }
            }
            
            // if we don't find it, just return an empty array.
            return [];
        },
    
        /**
         * Returns an array with all the attributes that this item has.  This
         * method will always return an array; if the item has no attributes
         * at all, getAttributes() will return an empty array: [].
         * 
         * exceptions:
         *  Throws an exception if *item* is not an item
         *  
         * example:
         *  var array = store.getAttributes(kermit);
         * 
         * @param {Object} item The item to access attributes on.
         */   
        getAttributes: function(item){    
            if(arguments.length != 1){
                throw new Error("Incorrect number of arguments specified: expecting 1 (item)");
            }
            if(isNullish(item)){
                throw new Error("Specified item argument is null/undefined.");
            }
            if(!this.isItem(item)){
                throw new Error("Specified item argument is not a valid item.");
            }
            if(!this.isItemLoaded(item)){
                this.loadItem({}); // TODO: ADD SUPPORT FOR THIS.
            }
            var attributes = [];
            
            for(var attr in item){
                if(attr == "properties"){
                    var props = item.properties;
                    var propslen = props.length;
                    for(var i = 0; i < propslen; i++){
                        if(attributes.lastIndexOf(props[i].id) < 0){
                            attributes.push(props[i].id);
                        }
                    }
                } else if(attr != "__loaded") {
                    attributes.push(attr);
                }
            }
            
            return attributes; 
        },
        
        /**
         * Returns true if the given *item* has a value for the given *attribute*.
         * This function IS case-sensitive.  If item is not loaded, load it.
         * 
         * exceptions:
         *  Throws an exception if *item* is not an item, or *attribute* is not a string
         * example:
         *  var trueOrFalse = store.hasAttribute(kermit, "color");
         * 
         * @param {Object} item The item to access attributes on.
         * @param {Object} attribute The attribute to access represented as a string.
         */
        hasAttribute: function(item, attribute){  
            if(arguments.length != 2){
                throw new Error("Incorrect number of arguments specified: expecting 2 (item, attribute)");
            }
            if(isNullish(item)){
                throw new Error("Specified item argument is null/undefined.");
            }
            if(!this.isItem(item)){
                throw new Error("Specified item argument is not a valid item.");
            }
            if(!dojo.isString(attribute)){
                throw new Error("Specified attribute argument is not a string.");
            }
            
            /*
             * If it's an item or a link, we can't say it's got the properties attribute.
             */
            if(attribute == "properties" && !(this._isTaskItem(item) || this._isChildDatasetItem(item))){
                return false;
            } 
            if(attribute == "__loaded"){
                return false;
            }
            
            /*
             * If it's a top-level attribute and it exists, return true.
             */
            if(!isNullish(item[attribute])){
                return true;
            }
            
            /*
             * TODO: If the item is not yet loaded, load it, then check for the 
             * attribute.  Do this after checking the top-level attributes. This 
             * way we only load an item when we really need to.
             */
            if(!this.isItemLoaded(item)){
                item = this.loadItem({item: item, synchronous: true});
            }
            
            /*
             * Look through the available properties array for the current dataset to see if it exists.. 
             * If it does, return true.
             */
            if(!isNullish(item.properties)){
                var length = item.properties.length;
                if(isNullish(length)){
                    throw new Error("Item argument does not have a valid properties array attribute.");
                }
                for(var i = 0; i < length; i++){
                    var prop = item.properties[i];
                    if(isNullish(prop)){
                        throw new Error("Item argument does not have a valid properties array attribute.");
                    }
                    if(prop.id == attribute){
                        return true;
                    }
                }
            }
            
            // otherwise, return false.
            return false; 
        },

        /**
         * Returns true if the given *value* is one of the values that getValues()
         * would return.
         * 
         * exceptions:
         *  Throws an exception if *item* is not an item, or *attribute* is not a string
         * example:
         *  var trueOrFalse = store.containsValue(kermit, "color", "green");
         * 
         * @param {Object} item The item to access values on.
         * @param {Object} attribute The attribute to access represented as a string.
         * @param {Object} value The value to match as a value for the attribute.
         */
        containsValue: function(item, attribute, value){ 
            if(arguments.length < 3){
                throw new Error("containsValue must be passed three arguments: item, attribute, value");
            }
            var useActual = false;
            if(arguments.length == 4){
                useActual = arguments[3];
            }
            var values = this.getValues(item,attribute,useActual);
            
            for(var i = 0; i < values.length; i++){
                if(this.valuesEqual(values[i], value)){
                    return true;
                }
            }
            
            return false; // boolean
        },
        
        /**
         * Not part of the dojo.data APIs, but this checks to see if two values are equalivalent.
         *
         * Always returns false if one or both of the values is a Function object.
         */
        valuesEqual: function(value1, value2){
            /*
             * If both arguments are arrays
             */
            if (dojo.isArray(value1) && dojo.isArray(value2)){
                if(value1.length != value2.length){
                    return false;
                }
                var membersEqual = true;
                for(var i = 0; i < value1.length; i++){
                    membersEqual = membersEqual && this.valuesEqual(value1[i], value2[i]);
                }
                return membersEqual;
            } 
            
            /*
             * If both arguments are objects
             */
            if((!isNullish(value1) && dojo.isObject(value1)) && (!isNullish(value2) && dojo.isObject(value2))){
                var membersEqual = true;
                for(var i in value1){
                    if(!dojo.isFunction(value1[i])){
                        if(!isNullish(value2[i])){
                            membersEqual = membersEqual && this.valuesEqual(value1[i], value2[i]);
                        } else {
                            return false;
                        }
                    }
                }
                return membersEqual;
            } 
            
            if(!isNullish(value1)){
                // if it's a boolean, turn it into a string.
                if(!dojo.isString(value1) && (value1.toString() == "true" || value1.toString() == "false")){
                    value1 = value1.toString();
                }
            }
            if(!isNullish(value2)){
                // if it's a boolean, turn it into a string.
                if(!dojo.isString(value2) && (value2.toString() == "true" || value2.toString() == "false")){
                    value2 = value2.toString();
                }
            }
            
            if((dojo.isArray(value1) && !dojo.isArray(value2)) || (!dojo.isArray(value1) && dojo.isArray(value2))){
                return false;
            }
            
            var result = value1 == value2;
            return result;
        },

        /**
         * Returns true if *something* is an item and came from the store 
         * instance.  Returns false if *something* is a literal, an item from 
         * another store instance, or is any object other than an item.
         * 
         * SPECIFICS:
         *      Returns true if the object has values for something.itemId, 
         *      something.itemUrl, and something.loaded
         * 
         * example:
         *    var yes = store.isItem(store.newItem());
         *    var no  = store.isItem("green");
         * 
         * @param {Object} something Can be anything.
         */
        isItem: function(something){ 
            return this._isItemItem(something) || this._isTaskItem(something) || this._isLinkItem(something) || this._isChildDatasetItem(something);
        },
        _isItemItem: function(something){
            return this._hasBaseItemAttributes(something) && !isNullish((/\/items\/(.)+/).exec(something.uri));
        },
        _isTaskItem: function(something){
            return this._hasBaseItemAttributes(something) && !isNullish((/\/tasks\/(.)+/).exec(something.uri));
        },
        _isLinkItem: function(something){
            return this._hasBaseItemAttributes(something) && !isNullish((/\/links\/(.)+/).exec(something.uri));
        },
        _isChildDatasetItem: function(something){
            return this._hasBaseItemAttributes(something) && !isNullish(something.itemsUri);
        },
        _hasBaseItemAttributes: function(something){
            if(!something){
                return false;
            }
            return !isNullish(something.id) && !isNullish(something.uri) && !isNullish(something.type);
        },

        /**
         * Returns false if isItem(something) is false.  Returns false if if 
         * isItem(something) is true but the the item is not yet loaded in 
         * local memory (for example, if the item has not yet been read from 
         * the server).
         * 
         * example:
         *    var yes = store.isItemLoaded(store.newItem());
         *    var no  = store.isItemLoaded("green");
         * 
         * @param {Object} something Can be anything.
         */
        isItemLoaded: function(something) { 
            if(!this.isItem(something)){
                return false;
            }
            
            var loaded = something.__loaded;
            return !isNullish(loaded) && loaded === true;
        },
        
        _setItemLoaded: function(item){
            item.__loaded = true;
        },
    
        /**
         * Given an item, this method loads the item so that a subsequent call 
         * to store.isItemLoaded(item) will return true.  If a call to 
         * isItemLoaded() returns true before loadItem() is even called, then 
         * loadItem() need not do any work at all and will not even invoke the 
         * callback handlers.  So, before invoking this method, check that the 
         * item has not already been loaded.
         * 
         * @param {Object} keywordArgs An anonymous object that defines the 
         *      item to load and callbacks to invoke when the load has 
         *      completed.  The format of the object is as follows: 
         *      { 
         *          item: object, 
         *                  The item parameter is an object that represents the 
         *                  item in question that should be contained by the 
         *                  store.  This attribute is required.
         *          onItem: Function, 
         *                  The onItem parameter is the callback to invoke when 
         *                  the item has been loaded.  It takes only one 
         *                  parameter, the fully loaded item.
         *          onError: Function, 
         *                  The onError parameter is the callback to invoke 
         *                  when the item load encountered an error.  It takes 
         *                  only one parameter, the error object
         *          scope: object, 
         *                  If a scope object is provided, all of the callback 
         *                  functions (onItem,  onError, etc) will be invoked 
         *                  in the context of the scope object. In the body of 
         *                  the callback function, the value of the "this" 
         *                  keyword will be the scope object.   If no scope 
         *                  object is provided, the callback functions will be 
         *                  called in the context of dojo.global(). For example, 
         *                  onItem.call(scope, item, request) vs.  
         *                  onItem.call(dojo.global(), item, request)
         *          synchronous: boolean,
         *                  Whether or not the call should be made synchronously, 
         *                  if so, we will also return the item as the result of 
         *                  this method.
         *      }
         */
        loadItem: function(keywordArgs){
            var settings = {
                item : null,
                onItem: function(item){
                },
                onError: function(error){
                },
                scope: null,
                synchronous: false
            };
            dojo.advancedMixin(settings, keywordArgs);
            settings.loadScope = this;
            
            if(isNullish(settings.item)){
                //throw new Error("Must specify an 'item' in the keywordArgs argument.");
                return;
            }
            
            if(this.isItemLoaded(settings.item)){
                return;
            }
            
            if(this._isItemItem(settings.item)){
                if(!isNullish(this.cache.items[settings.item.id])){
                    return this.cache.items[settings.item.id];
                }
                settings.onLoad = function(item, ioArgs){
                    settings.onItem.call(settings.scope, item);
                    settings.loadScope.cache.items[item.id] = item;
                    settings.loadScope._setItemLoaded(item);
                };
                settings.item = this._getItemById(settings);
            } else if(this._isLinkItem(settings.item)){
                if(!isNullish(this.cache.links[settings.item.id])){
                    return this.cache.items[settings.item.id];
                }
                settings.onLoad = function(item, ioArgs){
                    settings.onItem.call(settings.scope, item);
                    settings.loadScope.cache.links[item.id] = item;
                    settings.loadScope._setItemLoaded(item);
                };
                settings.item = this._getItemById(settings);
            } else if(this._isTaskItem(settings.item)){
                if(!isNullish(this.cache.tasks[settings.item.id])){
                    return this.cache.items[settings.item.id];
                }
                settings.onLoad = function(item, ioArgs){
                    settings.onItem.call(settings.scope, item);
                    settings.loadScope.cache.tasks[item.id] = item;
                    settings.loadScope._setItemLoaded(item);
                };
                settings.item = this._getTaskById(settings);
            } else {
                // do nothing...
            }
            return settings.item;
        },
    
        /**
         * Given a query and set of defined options, such as a start and count 
         * of items to return, this method executes the query and makes the 
         * results available as data items. The format and expectations of 
         * stores is that they operate in a generally asynchronous manner, 
         * therefore callbacks are always used to return items located by the 
         * fetch parameters.
         * 
         * A Request object will always be returned and is returned 
         * immediately. The basic request is nothing more than the keyword args 
         * passed to fetch and an additional function attached, abort().  The 
         * returned request object may then be used to cancel a fetch.  All 
         * data items returns are passed through the callbacks defined in the 
         * fetch parameters and are not present on the 'request' object.
         * 
         * This does not mean that custom stores can not add methods and 
         * properties to the request object returned, only that the API does 
         * not require it.  For more info about the Request API, see 
         * dojo.data.api.Request
         * 
         * exceptions:
         *     Throws an exception if the query is not valid, or if the query 
         *     is required but was not supplied.
         * 
         * @param {Object} keywordArgs
         *     The keywordArgs parameter may either be an instance of 
         *     conforming to dojo.data.api.Request or may be a simple anonymous 
         *     object that may contain any of the following:
         *     { 
         *         query: query-object or query-string, 
         *             
         *         queryOptions: object,
         *             {
         *                 ignoreCase: boolean, 
         *                     Whether or not the query should match case 
         *                     sensitively or not.  Default behaviour is false.
         *             }
         *             
         *         onBegin: Function(size, request), 
         *         
         *         onItem: Function(item, request), 
         *             
         *         onComplete: Function(items, request), 
         *         
         *         onError: Function(errorData, request+ioArgs),
         *          
         *         scope: object, 
         *             
         *         start: int, 
         *         
         *         count: int, 
         *             
         *         sort: array 
         *     }
         *     All implementations should accept keywordArgs objects with any 
         *     of the 9 standard properties: query, onBegin, onItem, onComplete, 
         *     onError scope, sort, start, and count.  Some implementations may 
         *     accept additional properties in the keywordArgs object as valid 
         *     parameters, such as {includeOutliers:true}.
         * @return
         *     The fetch() method will return a javascript object conforming to 
         *     the API defined in dojo.data.api.Request.  In general, it will 
         *     be the keywordArgs object returned with the required functions 
         *     in Request.js attached. Its general purpose is to provide a 
         *     convenient way for a caller to abort an ongoing fetch.
         *     
         *     The Request object may also have additional properties when it 
         *     is returned such as request.store property, which is a pointer 
         *     to the datastore object that fetch() is a method of.
         */
        fetch: function(keywordArgs){
            var settings = {
                onBegin: function(size, request){
                },
                onItem: function(item, request){
                },
                onComplete: function(items, request){
                },
                onError: function(errorData, request){
                },
                
                scope: null,
                start: 0,
                count: null,
                sort: [],
                query: {
                    navModel: {
                        itemTypes: ['item','link','task','childDataset']
                    }
                },
                queryOptions: {}
            };
            dojo.advancedMixin(settings,keywordArgs);
            
            /*
             * Inject the extension-specific settings into the settings.
             */
            dojo.mixin(settings, this._getNavModelSettings());
            
            /*
             * Get the target scope and error method;
             */
            var targetScope = settings.scope;
            var targetError = settings.onError;
            /*
             * Set the settings scope to our scope;
             */ 
            settings.scope = this;
            
            settings.onError = function(error, ioArgs){
                dojo.mixin(settings, ioArgs);
                targetError.call(targetScope, error, settings);
            };
            
            /*
             * Determine whether or not we're retrieving child datasets with this fetch.
             */
            var retrievingChildDatasets = false;
            if(!isNullish(settings.query.navModel)){
                if(!isNullish(settings.query.navModel.itemTypes)){
                    var dsregex = /childDataset/;
                    retrievingChildDatasets = dsregex.test(settings.query.navModel.itemTypes.join(" "));
                }
            }
            
            /**
             * Specific to this instance, not an actual setting.
             */
            settings.onLoad = function(result, requestinfo){
                var itemLength = !isNullish(result.items) ? result.items.length : 0;
                var taskLength = !isNullish(result.toolbarTasks) ? result.toolbarTasks.length : 0;
                var linkLength = !isNullish(result.links) ? result.links.length : 0;
                
                var childDatasetLength = !isNullish(result.childDatasets) && retrievingChildDatasets ? result.childDatasets.length : 0;
                
                /*
                 * If this was a dataset-head request, instead of a normal 
                 * item/link/task request.. use special total calculation.  
                 * otherwise use the normal one.
                 */
                if(!isNullish(result.toolbarTasks) && !isNullish(result.links)){
                    settings.onBegin.call(targetScope, itemLength + taskLength + linkLength + childDatasetLength, requestinfo);
                } else {
                    if(!isNullish(result.filteredRows)){
                        settings.onBegin.call(targetScope, result.filteredRows, settings, requestinfo);
                    } else {
                        settings.onBegin.call(targetScope, result.totalRows, settings, requestinfo);
                    }
                }
                
                // master list, returned with onComplete
                var masterList = [];
                
                /*
                 * Iterate through items.
                 */
                var items = result.items;
                if(!isNullish(items)){
                    var items_length = items.length;
                    for(var i = 0; i < items_length; i++){
                        this._setItemLoaded(items[i]);
                        settings.onItem.call(targetScope, items[i], settings, requestinfo);
                        
                        
                        /*
                         * Just in case we have to work with links/tasks (in the 
                         * case where we didn't request more than one type at once), 
                         * check here so we don't throw stuff in the wrong cache 
                         * slot.
                         */
                        var cacheSlot = "items";
                        if(this._isLinkItem(items[i])){
                            cacheSlot = "links";
                        } else if(this._isTaskItem(items[i])){
                            cacheSlot = "tasks";
                        }
                        
                        this.cache[cacheSlot][this.getIdentity(items[i])] = items[i];
                        masterList.push(items[i]);
                    }
                }
                
                /*
                 * Iterate through links.
                 */
                var links = result.links;
                if(!isNullish(links)){
                    var links_length = links.length;
                    for(var i = 0; i < links_length; i++){
                        this._setItemLoaded(links[i]);
                        settings.onItem.call(targetScope, links[i], settings, requestinfo);
                        this.cache.links[this.getIdentity(links[i])] = links[i];
                        masterList.push(links[i]);
                    }
                }
                
                /*
                 * Iterate through the tasks.
                 */
                var tasks = result.toolbarTasks;
                if(!isNullish(tasks)){
                    var tasks_length = tasks.length;
                    for(var i = 0; i < tasks_length; i++){
                        this._setItemLoaded(tasks[i]);
                        settings.onItem.call(targetScope, tasks[i], settings, requestinfo);
                        this.cache.tasks[this.getIdentity(tasks[i])] = tasks[i];
                        masterList.push(tasks[i]);
                    }
                }
                
                /*
                 * Iterate through the child datasets.
                 */
                var childDatasets = result.childDatasets;
                if(!isNullish(childDatasets) && retrievingChildDatasets){
                    var childDatasets_length = childDatasets.length;
                    for(var i = 0; i < childDatasets_length; i++){
                        this._setItemLoaded(childDatasets[i]);
                        settings.onItem.call(targetScope, childDatasets[i], settings, requestinfo);
                        this.cache.childDatasets[this.getIdentity(childDatasets[i])] = childDatasets[i];
                        masterList.push(childDatasets[i]);
                    }
                }
                
                settings.onComplete.call(targetScope, masterList, settings, requestinfo);
            };
            
            settings.req = this._getDataForFetch(settings);
            
            return settings; // an object conforming to the dojo.data.api.Request API
        },
    
        /**
         * The getFeatures() method returns an simple keyword values object 
         * that specifies what interface features the datastore implements.  
         * A simple CsvStore may be read-only, and the only feature it 
         * implements will be the 'dojo.data.api.Read' interface, so the
         * getFeatures() method will return an object like this one:
         * {'dojo.data.api.Read': true}.
         * A more sophisticated datastore might implement a variety of
         * interface features, like 'dojo.data.api.Read', 'dojo.data.api.Write', 
         * 'dojo.data.api.Identity', and 'dojo.data.api.Attribution'.
         */
        getFeatures: function(){
            return {
                'dojo.data.api.Read': true,
                'dojo.data.api.Write': true,
                'dojo.data.api.Identity': true,
                'ibm.tivoli.data.api.Relationship':true
            };
        },

        /**
         * The close() method is intended for instructing the store to 'close' 
         * out any information associated with a particular request.
         * 
         * The close() method is intended for instructing the store to 'close' 
         * out any information associated with a particular request.  In 
         * general, this APIexpects to recieve as a parameter a request object 
         * returned from a fetch.  It will then close out anything associated 
         * with that request, such as clearing any internal datastore caches 
         * and closing any 'open' connections.For some store implementations, 
         * this call may be a no-op.
         * 
         * example:
         *      var request = store.fetch({onComplete: doSomething});
         *      ...
         *      store.close(request);
         * 
         * @param {Object} 
         *      (dojo.data.api.Request || keywordArgs || null)request 
         *      An instance of a request for the store to use to identify what 
         *      to close out.If no request is passed, then the store should 
         *      clear all internal caches (if any)and close out all 'open' 
         *      connections.  It does not render the store unusable fromthere 
         *      on, it merely cleans out any current data and resets the store 
         *      to initial state.
         */
        close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
            // do nothing here... we can't really close the requests..
            // throw new Error('Unimplemented API: dojo.data.api.Read.close');
        },

        /**
         * Method to inspect the item and return a user-readable 'label' for the item
         * that provides a general/adequate description of what the item is.
         * 
         * Method to inspect the item and return a user-readable 'label' for the item
         * that provides a general/adequate description of what the item is.  In general
         * most labels will be a specific attribute value or collection of the attribute
         * values that combine to label the item in some manner.  For example for an item
         * that represents a person it may return the label as:  "firstname lastlame" where
         * the firstname and lastname are attributes on the item.  If the store is unable 
         * to determine an adequate human readable label, it should return undefined.  Users that wish
         * to customize how a store instance labels items should replace the getLabel() function on 
         * their instance of the store, or extend the store and replace the function in 
         * the extension class.
         * 
         * returns: 
         *  A user-readable string representing the item or undefined if no user-readable label can 
         *  be generated.
         * 
         * @param {Object} item The item to return the label for.
         */
        getLabel: function(item){
            if(!this.isItem(item)){
                throw new Error("Specified argument is not a valid item object.");
            }
            return item.label;   // will be undefined if the item doesn't have a label.
        },
    
        /**
         * Method to inspect the item and return an array of what attributes of the item were used 
         * to generate its label, if any.
         * 
         * Method to inspect the item and return an array of what attributes of the item were used 
         * to generate its label, if any.  This function is to assist UI developers in knowing what
         * attributes can be ignored out of the attributes an item has when displaying it, in cases
         * where the UI is using the label as an overall identifer should they wish to hide 
         * redundant information.
         * 
         * returns:
         *  An array of attribute names that were used to generate the label, or null if public attributes 
         *  were not used to generate the label.
         * 
         * @param {Object} item  The item to return the list of label attributes for.
         */
        getLabelAttributes: function(item){ 
            if(!this.isItem(item)){
                throw new Error("Specified argument is not a valid item object.");
            } 
            return ["label"];
        },
    
        /**
         * Returns a unique identifier for an item.  The return value will be 
         * either a string or something that has a toString() method (such as, 
         * for example, a dojox.uuid.Uuid object).
         * 
         * exceptions: 
         *      Conforming implementations may throw an exception or return 
         *      null if item is not an item.
         * 
         * @param {Object} item
         *      The item from the store from which to obtain its identifier.
         */
        getIdentity: function(item){
            var prefix = "navModelItem";
            var suffix = ")";
            if(this._isItemItem(item)){
                return prefix + "_item(" + item.id + suffix;
            } else if(this._isLinkItem(item)){
                return prefix + "_link(" + item.id + suffix;
            } else if(this._isTaskItem(item)){
                return prefix + "_task(" + item.id + suffix;
            } else if(this._isChildDatasetItem(item)){
                return prefix + "_childDataset(" + item.id + suffix;
            } else {
                throw new Error("Specified argument is not a valid item object.");
            }
        },
        
        /**
         * Takes the results from this.getIdentity and determines which type of 
         * item it is, and gives the id as well.
         * 
         * exceptions:
         *      Throws an error if the string does not match the grammar 
         *      specified when building an id in _NavModelStore.getIdentity(..)
         */
        getTypeAndId: function(idString){
            if(!dojo.isString(idString)){
                throw new Error("Invalid Identifier: Not a string.");
            }
            
            var regex = /navModelItem_(item|link|task|childDataset)\(([\w\d\(\)0-9!'"\-\.@\\\s]+)\)/;
            var matches = regex.exec(idString);
            if(isNullish(matches)){
                throw new Error("Invalid Identifier: Incorrectly formatted: "+idString);
            }
            return {type: matches[1], id: matches[2]};
        },
        
        getIdentityFromItemID: function(idString){
            return "navModelItem_item("+idString+")";
        },
        getIdentityFromTaskID: function(idString){
            return "navModelItem_task("+idString+")";
        },
        getIdentityFromLinkID: function(idString){
            return "navModelItem_link("+idString+")";
        },
    
        /**
         * Returns an array of attribute names that are used to generate the 
         * identity. For most stores, this is a single attribute, but for some 
         * complex stores such as RDB backed stores that use compound (multi-
         * attribute) identifiers it can be more than one.  If the identity is 
         * not composed of attributes on the item, it will return null.  This 
         * function is intended to identify the attributes that comprise the 
         * identity so that so that during a render of all attributes, the UI 
         * can hide the the identity information if it chooses.
         * 
         * @param {Object} item
         *      The item from the store from which to obtain the array of 
         *      public attributes that compose the identifier, if any.
         */
        getIdentityAttributes: function(item){
            return "id";
        },
    
        /**
         * Given the identity of an item, this method returns the item that 
         * has that identity through the onItem callback.  Conforming 
         * implementations should return null if there is no item with the 
         * given identity.
         * 
         * Implementations of fetchItemByIdentity() may sometimes return an 
         * item from a local cache and may sometimes fetch an item from a 
         * remote server.
         * 
         * @param {Object} keywordArgs
         *      An anonymous object that defines the item to locate and 
         *      callbacks to invoke when the item has been located and load has 
         *      completed.  The format of the object is as follows: 
         *      { 
         *          identity: string|object, 
         *              The identity parameter is the identity of the item you 
         *              wish to locate and load This attribute is required.  It 
         *              should be a string or an object that toString() can be 
         *              called on.
         *              
         *          onItem: Function(item), 
         *              The onItem parameter is the callback to invoke when the 
         *              item has been loaded.  It takes only one parameter, the 
         *              item located, or null if none found.
         *              
         *          onError: Function(error), 
         *              The onError parameter is the callback to invoke when 
         *              the item load encountered an error.  It takes only one 
         *              parameter, the error object
         *              
         *          scope: object 
         *              If a scope object is provided, all of the callback 
         *              functions (onItem, onError, etc) will be invoked in the 
         *              context of the scope object. In the body of the 
         *              callback function, the value of the "this" keyword will 
         *              be the scope object.   If no scope object is provided, 
         *              the callback functions will be called in the context of 
         *              dojo.global.
         *              
         *              For example, onItem.call(scope, item, request) vs. 
         *              onItem.call(dojo.global, item, request)
         *      }
         */
        fetchItemByIdentity: function(keywordArgs){
            var settings = {
                identity: null,
                onItem: function(item){
                },
                onError: function(item){
                },
                scope: null,
                loadScope: this
            };
            dojo.advancedMixin(settings, keywordArgs);
            
            /*
             * Inject the extension-specific settings into the settings.
             */
            dojo.mixin(settings, this._getNavModelSettings());
            
            var typeAndId = this.getTypeAndId(settings.identity);
            
            settings.type = typeAndId.type;
            settings.item = typeAndId.id;
            
            settings.onLoad=function(item){
                settings.loadScope._setItemLoaded(item);
                settings.onItem.call(settings.scope, item);
                settings.loadScope.cache[settings.type+'s'] = item;
            };
            
            
            /*
             * If the item is in the cache, return that.
             */
            if(this.cache[settings.type+'s'][settings.item]){
                settings.onItem.call(settings.scope, this.cache[settings.type+'s'][settings.item]);
                return this.cache[settings.type+'s'][settings.item];
            }
            
            /*
             * Make a call to the service/system.
             */
            if(settings.type === "item"){
                this._getItemById(settings);
            } else if(settings.type === "link"){
                this._getLinkById(settings);
            } else if(settings.type === "task"){
                this._getTaskById(settings);
            } 
            
            return null; // string
        },
        
        
        /************
         * dojo.data.api.Write Functionality.
         ************/
        
        
        newItem: function(/* Object? */ keywordArgs, /*Object?*/ parentInfo){
            // summary:
            //      Returns a newly created item.  Sets the attributes of the new
            //      item based on the *keywordArgs* provided.  In general, the attribute
            //      names in the keywords become the attributes in the new item and as for
            //      the attribute values in keywordArgs, they become the values of the attributes
            //      in the new item.  In addition, for stores that support hierarchical item
            //      creation, an optional second parameter is accepted that defines what item is the parent
            //      of the new item and what attribute of that item should the new item be assigned to.
            //      In general, this will assume that the attribute targetted is multi-valued and a new item
            //      is appended onto the list of values for that attribute.
            //
            //  keywordArgs:
            //      A javascript object defining the initial content of the item as a set of JavaScript 'property name: value' pairs.
            //  parentInfo:
            //      An optional javascript object defining what item is the parent of this item (in a hierarchical store.  Not all stores do hierarchical items),
            //      and what attribute of that parent to assign the new item to.  If this is present, and the attribute specified
            //      is a multi-valued attribute, it will append this item into the array of values for that attribute.  The structure
            //      of the object is as follows:
            //      {
            //          parent: someItem,
            //          attribute: "attribute-name-string"
            //      }
            //
            //  exceptions:
            //      Throws an exception if *keywordArgs* is a string or a number or
            //      anything other than a simple anonymous object.
            //      Throws an exception if the item in parentInfo is not an item from the store
            //      or if the attribute isn't an attribute name string.
            //  examples:
            //      var kermit = store.newItem({name: "Kermit", color:[blue, green]});
            
            throw new Error("newItem is not supported by NavModelStore.");
        },
        
        deleteItem: function(/* item */ item){
            //  summary:
            //      Deletes an item from the store.
            //
            //  item:
            //      The item to delete.
            //
            //  exceptions:
            //      Throws an exception if the argument *item* is not an item
            //      (if store.isItem(item) returns false).
            //  examples:
            //      var success = store.deleteItem(kermit);
            
            throw new Error("deleteItem is not supported by NavModelStore.");
        },
        
        setValue: function(/* item */ item, /* string */ attribute, /* almost anything */ value){
            //  summary:
            //      Sets the value of an attribute on an item.
            //      Replaces any previous value or values.
            //
            //  item:
            //      The item to modify.
            //  attribute:
            //      The attribute of the item to change represented as a string name.
            //  value:
            //      The value to assign to the item.
            //
            //  exceptions:
            //      Throws an exception if *item* is not an item, or if *attribute*
            //      is neither an attribute object or a string.
            //      Throws an exception if *value* is undefined.
            //  examples:
            //      var success = store.set(kermit, "color", "green");
        },
        
        setValues: function(/* item */ item, /* string */ attribute, /* array */ values){
            //  summary:
            //      Adds each value in the *values* array as a value of the given
            //      attribute on the given item.
            //      Replaces any previous value or values.
            //      Calling store.setValues(x, y, []) (with *values* as an empty array) has
            //      the same effect as calling store.unsetAttribute(x, y).
            //
            //  item:
            //      The item to modify.
            //  attribute:
            //      The attribute of the item to change represented as a string name.
            //  values:
            //      An array of values to assign to the attribute..
            //
            //  exceptions:
            //      Throws an exception if *values* is not an array, if *item* is not an
            //      item, or if *attribute* is neither an attribute object or a string.
            //  examples:
            //      var success = store.setValues(kermit, "color", ["green", "aqua"]);
            //      success = store.setValues(kermit, "color", []);
            //      if (success) {assert(!store.hasAttribute(kermit, "color"));}
        },
        
        unsetAttribute: function(    /* item */ item, /* string */ attribute){
            //  summary:
            //      Deletes all the values of an attribute on an item.
            //
            //  item:
            //      The item to modify.
            //  attribute:
            //      The attribute of the item to unset represented as a string.
            //
            //  exceptions:
            //      Throws an exception if *item* is not an item, or if *attribute*
            //      is neither an attribute object or a string.
            //  examples:
            //      var success = store.unsetAttribute(kermit, "color");
            //      if (success) {assert(!store.hasAttribute(kermit, "color"));}
        },
        
        save: function(/* object */ keywordArgs){
            //  summary:
            //      Saves to the server all the changes that have been made locally.
            //      The save operation may take some time and is generally performed
            //      in an asynchronous fashion.  The outcome of the save action is
            //      is passed into the set of supported callbacks for the save.
            //
            //  keywordArgs:
            //      {
            //          onComplete: function
            //          onError: function
            //          scope: object
            //      }
            //
            //  The *onComplete* parameter.
            //      function();
            //
            //      If an onComplete callback function is provided, the callback function
            //      will be called just once, after the save has completed.  No parameters
            //      are generally passed to the onComplete.
            //
            //  The *onError* parameter.
            //      function(errorData);
            //
            //      If an onError callback function is provided, the callback function
            //      will be called if there is any sort of error while attempting to
            //      execute the save.  The onError function will be based one parameter, the
            //      error.
            //
            //  The *scope* parameter.
            //      If a scope object is provided, all of the callback function (
            //      onComplete, onError, etc) will be invoked in the context of the scope
            //      object.  In the body of the callback function, the value of the "this"
            //      keyword will be the scope object.   If no scope object is provided,
            //      the callback functions will be called in the context of dojo.global.
            //      For example, onComplete.call(scope) vs.
            //      onComplete.call(dojo.global)
            //
            //  returns:
            //      Nothing.  Since the saves are generally asynchronous, there is
            //      no need to return anything.  All results are passed via callbacks.
            //  examples:
            //      store.save({onComplete: onSave});
            //      store.save({scope: fooObj, onComplete: onSave, onError: saveFailed});
        },
        
        revert: function(){
            //  summary:
            //      Discards any unsaved changes.
            //  description:
            //      Discards any unsaved changes.
            //
            //  examples:
            //      var success = store.revert();
        },
        
        isDirty: function(/* item? */ item){
            //  summary:
            //      Given an item, isDirty() returns true if the item has been modified
            //      since the last save().  If isDirty() is called with no *item* argument,
            //      then this method returns true if any item has been modified since
            //      the last save().
            //
            //  item:
            //      The item to check.
            //
            //  exceptions:
            //      Throws an exception if isDirty() is passed an argument and the
            //      argument is not an item.
            //  examples:
            //      var trueOrFalse = store.isDirty(kermit); // true if kermit is dirty
            //      var trueOrFalse = store.isDirty();       // true if any item is dirty
        },
        
        /***********
         *  Hierarchical stuff...
         ***********/

        /**
         * Returns whether or not the item has relationships.  
         */
        hasRelatedItems: function(item){
            if(!this._isItemItem(item)){
                return false;
            }
            if(!isNullish(item.relationshipsKey)){
                return true;
            }
            return false;
        },
        
        /**
         * Provides fetch functionality for getting related items.
         */
        fetchRelatedItems: function(keywordArgs){
            var settings = {
                onBegin: function(size, request){
                },
                onItem: function(item, request){
                },
                onComplete: function(items, request){
                },
                onError: function(errorData, request){
                },
                
                scope: null,
                
                start: 0,
                count: null,
                sort: [],
                query: {},
                queryOptions: {}
            };
            dojo.advancedMixin(settings,keywordArgs);
            
            
            /*
             * Inject the extension-specific settings into the settings.
             */
            dojo.mixin(settings, this._getNavModelSettings());
            
            /*
             * Get the target scope and error method;
             */
            var targetScope = settings.scope;
            var targetError = settings.onError;
            /*
             * Set the settings scope to our scope;
             */ 
            settings.scope = this;
            
            settings.onError = function(error, requestinfo){
                targetError.call(targetScope, error, settings);
            };
            
             
            /**
             * Specific to this instance, not an actual setting.
             */
            settings.onLoad = function(result, requestinfo){
                var taskLength = !isNullish(result.globalTasks) ? result.globalTasks.length : 0;
                var linkLength = !isNullish(result.links) ? result.links.length : 0;
                settings.onBegin.call(targetScope, result.items.length + taskLength + linkLength);
                
                
                // master list, returned with onComplete
                var masterList = [];
                
                /*
                 * Iterate through items.
                 */
                var items = result.items;
                var items_length = items.length;
                for(var i = 0; i < items_length; i++){
                    this._setItemLoaded(items[i]);
                    settings.onItem.call(targetScope, items[i], settings);
                    
                    
                    /*
                     * Just in case we have to work with links/tasks (in the 
                     * case where we didn't request more than one type at once), 
                     * check here so we don't throw stuff in the wrong cache 
                     * slot.
                     */
                    var cacheSlot = "items";
                    if(this._isLinkItem(items[i])){
                        cacheSlot = "links";
                    } else if(this._isTaskItem(items[i])){
                        cacheSlot = "tasks";
                    }
                    
                    this.cache[cacheSlot][this.getIdentity(items[i])] = items[i];
                    masterList.push(items[i]);
                }
                
                /*
                 * Iterate through links.
                 */
                var links = result.links;
                if(!isNullish(links)){
                    var links_length = links.length;
                    for(var i = 0; i < links_length; i++){
                        this._setItemLoaded(links[i]);
                        settings.onItem.call(targetScope, links[i], settings);
                        this.cache.links[this.getIdentity(links[i])] = links[i];
                        masterList.push(links[i]);
                    }
                }
                
                /*
                 * Iterate through the tasks.
                 */
                var tasks = result.globalTasks;
                if(!isNullish(tasks)){
                    var tasks_length = tasks.length;
                    for(var i = 0; i < tasks_length; i++){
                        this._setItemLoaded(tasks[i]);
                        settings.onItem.call(targetScope, tasks[i], settings);
                        this.cache.tasks[this.getIdentity(tasks[i])] = tasks[i];
                        masterList.push(tasks[i]);
                    }
                }
                
                settings.onComplete.call(targetScope, masterList, settings);
            };
            
            
            this._getRelatedItems(settings);
            
            return settings; // an object conforming to the dojo.data.api.Request API
        }
    });
    
    ibm.tivoli.data._NavModelStore.TOPIC_LOG = navModelStoreLog;
    
})(dojo);
