/**
 * @author jwfeinst
 */
(function(dojo){
        
    dojo.provide("ibm.tivoli.data.api.Relationship");
    
    /*
     * Other imports
     */
    dojo.require("dojo.data.api.Request");
    
    /**
     * Interface to define the functions required of a hierarchical data store.
     */
    dojo.declare("ibm.tivoli.data.api.Relationship", [], {
        /**
         * Returns whether or not the item has relationships.  
         */
        hasRelatedItems: function(item){
            throw new Error("Not implemented yet.");
        },
    
        /**
         * Fetches the items related to a specified item (e.g. Children nodes
         * for a node in a tree structure)
         *
         * Given a query and set of defined options, such as a start and count 
         * of items to return, this method executes the query and makes the 
         * results available as data items. The format and expectations of 
         * stores is that they operate in a generally asynchronous manner, 
         * therefore callbacks are always used to return items located by the 
         * fetch parameters.
         * 
         * A Request object will always be returned and is returned 
         * immediately. The basic request is nothing more than the keyword args 
         * passed to fetch and an additional function attached, abort().  The 
         * returned request object may then be used to cancel a fetch.  All 
         * data items returns are passed through the callbacks defined in the 
         * fetch parameters and are not present on the 'request' object.
         * 
         * This does not mean that custom stores can not add methods and 
         * properties to the request object returned, only that the API does 
         * not require it.  For more info about the Request API, see 
         * dojo.data.api.Request
         * 
         * exceptions:
         *     Throws an exception if the query is not valid, or if the query 
         *     is required but was not supplied.
         * 
         * @param {Object} keywordArgs
         *     The keywordArgs parameter may either be an instance of 
         *     conforming to dojo.data.api.Request or may be a simple anonymous 
         *     object that may contain any of the following:
         *     {
         *         baseItem: Item
         *             Item we're trying to inspect to determine if it has
         *             relationships.
         *         query: query-object or query-string, 
         *             The query may be optional in some data store 
         *             implementations.
         *             
         *             The dojo.data.api.Read API does not specify the syntax 
         *             or semantics of the query itself -- each different data 
         *             store implementation may have its own notion of what a 
         *             query should look like.
         *             
         *             However, as of dojo 0.9, 1.0, and 1.1, all the provided 
         *             datastores in dojo.data and dojox.data support an object 
         *             structure query, where the object is a set of name/value 
         *             parameters such as 
         *             { 
         *                 attrFoo: valueBar, 
         *                 attrFoo1: valueBar1
         *             }.  
         *             Most of the dijit widgets, such as ComboBox assume this 
         *             to be the case when working with a datastore when they 
         *             dynamically update the query.  Therefore, for maximum 
         *             compatibility with dijit widgets the recommended query 
         *             parameter is a key/value object.  That does not mean 
         *             that the the datastore may not take alternative query 
         *             forms, such as a simple string, a Date, a number, or a 
         *             mix of such.  Ultimately, The dojo.data.api.Read API is 
         *             agnostic about what the query format.
         *             
         *             Further note:  In general for query objects that accept 
         *             strings as attribute value matches, the store should 
         *             also support basic filtering capability, such as * 
         *             (match any character) and ? (match single character).  
         *             An example query that is a query object would be like: 
         *             { 
         *                 attrFoo: "value*"
         *             }.  
         *             Which generally means match all items where they have an 
         *             attribute named attrFoo, with a value that starts with 
         *             'value'.
         *             
         *         queryOptions: object,
         *             The queryOptions parameter is an optional parameter used 
         *             to specify options that may modify the query in some 
         *             fashion, such as doing a case insensitive search, or 
         *             doing a deep search where all items in a hierarchical 
         *             representation of data are scanned instead of just the 
         *             root items.  It currently defines two options that all 
         *             datastores should attempt to honor if possible:
         *             {
         *                 ignoreCase: boolean, 
         *                     Whether or not the query should match case 
         *                     sensitively or not.  Default behaviour is false.
         *                     
         *                 deep: boolean 
         *                     Whether or not a fetch should do a deep search 
         *                     of items and all child items instead of just 
         *                     root-level items in a datastore.  Default is 
         *                     false.
         *             }
         *             
         *         onBegin: Function(size, request), 
         *             If an onBegin callback function is provided, the 
         *             callback function will be called just once, before the 
         *             first onItem callback is called. The onBegin callback 
         *             function will be passed two arguments, the the total 
         *             number of items identified and the Request object.  If 
         *             the total number is unknown, then size will be -1.  Note 
         *             that size is not necessarily the size of the collection 
         *             of items returned from the query, as the request may 
         *             have specified to return only a subset of the total set 
         *             of items through the use of the start and count 
         *             parameters.
         *         
         *         onItem: Function(item, request), 
         *             If an onItem callback function is provided, the callback 
         *             function will be called as each item in the result is 
         *             received. The callback function will be passed two 
         *             arguments: the item itself, and the Request object.
         *             
         *         onComplete: Function(items, request), 
         *             If an onComplete callback function is provided, the 
         *             callback function will be called just once, after the 
         *             last onItem callback is called. Note that if the onItem 
         *             callback is not present, then onComplete will be passed 
         *             an array containing all items which matched the query 
         *             and the request object. If the onItem callback is 
         *             present, then onComplete is called as: 
         *             onComplete(null, request).
         *         
         *         onError: Function(errorData, request),
         *             If an onError callback function is provided, the 
         *             callback function will be called if there is any sort of 
         *             error while attempting to execute the query.
         *             
         *             The onError callback function will be passed two 
         *             arguments: an Error object and the Request object.
         *          
         *         scope: object, 
         *             If a scope object is provided, all of the callback 
         *             functions (onItem, onComplete, onError, etc) will be 
         *             invoked in the context of the scope object.  In the body 
         *             of the callback function, the value of the "this" 
         *             keyword will be the scope object. If no scope object is 
         *             provided, the callback functions will be called in the 
         *             context of dojo.global(). 
         *             
         *             For example, onItem.call(scope, item, request) vs. 
         *             onItem.call(dojo.global(), item, request)
         *             
         *         start: int, 
         *             If a start parameter is specified, this is a indication 
         *             to the datastore to only start returning items once the 
         *             start number of items have been located and skipped.  
         *             When this parameter is paired withh 'count', the store 
         *             should be able to page across queries with millions of 
         *             hits by only returning subsets of the hits for each 
         *             query
         *         
         *         count: int,
         *             If a count parameter is specified, this is a indication 
         *             to the datastore to only return up to that many items.  
         *             This allows a fetch call that may have millions of item 
         *             matches to be paired down to something reasonable. 
         *             
         *         sort: array 
         *             If a sort parameter is specified, this is a indication 
         *             to the datastore to sort the items in some manner before 
         *             returning the items.  The array is an array of 
         *             javascript objects that must conform to the following 
         *             format to be applied to the fetching of items:
         *             {
         *                 attribute: attribute || attribute-name-string, 
         *                 descending: true|false;   
         *                     Optional.  Default is false. 
         *             }
         *             Note that when comparing attributes, if an item contains 
         *             no value for the attribute (undefined), then it the 
         *             default ascending sort logic should push it to the 
         *             bottom of the list.  In the descending order case, it 
         *             such items should appear at the top of the list.
         *     }
         *     All implementations should accept keywordArgs objects with any 
         *     of the 9 standard properties: query, onBegin, onItem, onComplete, 
         *     onError scope, sort, start, and count.  Some implementations may 
         *     accept additional properties in the keywordArgs object as valid 
         *     parameters, such as {includeOutliers:true}.
         * @return
         *     The fetch() method will return a javascript object conforming to 
         *     the API defined in dojo.data.api.Request.  In general, it will 
         *     be the keywordArgs object returned with the required functions 
         *     in Request.js attached. Its general purpose is to provide a 
         *     convenient way for a caller to abort an ongoing fetch.
         *     
         *     The Request object may also have additional properties when it 
         *     is returned such as request.store property, which is a pointer 
         *     to the datastore object that fetch() is a method of.
         */
        fetchRelatedItems: function(keywordArgs){
            throw new Error("Not implemented yet.");
        }
    });
})(dojo);