/**
 * This store is not meant to retrieve actual information, it's meant for 
 * navigating a NavModel REST service to find the configuration information 
 * necessary to instantiate a NavModelStore.
 *
 * Notes:
 *  - This file should be brought into the larger NavModelStore project 
 *    eventually.
 *  - This should be put into the inheritance pattern that NavModelStore uses
 *    (like iWidgetNavModelStore vs. AJAXNavModelStore, etc.)
 *  - There is MINIMAL error checking in this.
 *
 * @author Jason Feinstein (jwfeinst@us.ibm.com)
 */
dojo.provide("ibm.tivoli.data.iWidgetNavigationalStore");
dojo.require("dojo.data.api.Read");
dojo.require("dojo.data.api.Identity");

dojo.declare("ibm.tivoli.data.iWidgetNavigationalStore", [], {
    baseUrl: "",
    iContextIO: null,
    
    noDatasource: "navstore_NONE",
    
    constructor: function(args){
        dojo.mixin(this, args);
    },
    
    getValue: function(item, attribute){
        return this.getValues(item,attribute)[0];
    },
    
    getValues: function(/* item */ item, /* attribute-name-string */ attribute){
        if(!this.hasAttribute(item,attribute)){
            return [];
        }
        var value = item[attribute];
        if(dojo.isArray(value)){
            return value;
        }
        return [value];
    },
    
    getAttributes: function(/* item */ item){
        var attrs = [];
        
        for(var i in item){
            if(item.hasOwnProperty(i) && i !== "query"){
                attrs.push(i);
            }
        }
        
        return attrs;
    },
    
    hasAttribute: function(/* item */ item, /* attribute-name-string */ attribute){
        var undefined;
        
        return item[attribute] !== undefined;
    },
    
    containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value){
    },
    
    isItem: function(/* anything */ something){
        var undefined;
        return something.query !== undefined;
    },
    
    isItemLoaded: function(/* anything */ something){
        return true;
    },
    
    loadItem: function(/* object */ keywordArgs){
        throw new Error("iWidgetNavigationalStore.loadItem not implemented yet");
    },
    
    /**
     *
     *
     * keywordArgs = {
     *    query: query-string or query-object,
     *    queryOptions: object,
     *    onBegin: Function,
     *    onItem: Function,
     *    onComplete: Function,
     *    onError: Function,
     *    scope: object,
     *    start: int
     *    count: int
     *    sort: array
     * }
     *
     * query: {
     *    provider: {something} or null,
     *    datasource: {something} or {something with id='none'} or null
     * }
     *
     * If 'provider' is null, we'll look for a list of providers.
     * If 'provider' is not null and 'datasource' is not null, we'll look for datasets.
     * Else we'll look for datasources.
     */
    fetch: function(/* Object */ keywordArgs){
        var settings = {
            query: {},
            onBegin: function(){},
            onItem: function(){},
            onComplete: function(){},
            onError: function(){},
            scope: this
        };
        settings = dojo.mixin(settings, keywordArgs);
        settings.query = this._sanitizeQuery(settings.query);
        
        var url = this._buildUrl(settings.query);
        var queryType = this._getQueryType(settings.query);
        var me = this;
        
        dojo.xhrGet({
            url: this.manipulateURI(url),
            content: this._buildContent(settings),
            handleAs: "json",
            load: function(result, ioArgs){
                var args = dojo.mixin(ioArgs, settings);
                var numRows = result.numRows;
                numRows = numRows ? numRows : 0;
                var items = result.items;
                
                /*
                 * If we're looking for columns, we need to change the numRows to mean
                 * number of properties.
                 */
                if((/columns/).test(queryType)){
                    if(numRows > 0){
                        numRows = result.items[0].properties.length;
                        items = result.items[0].properties;
                    }
                }
                
                settings.onBegin.call(settings.scope, numRows, args);
                
                var foundItems = [];
                if(queryType == "datasource"){
                    var item = {
                        datasetsUri: "n\/a",
                        description: "Not a datasource, used as a placeholder.",
                        id: me.noDatasource,
                        label: "None",
                        type: "n\/a",
                        uri: "n\/a",
                        query: settings.query
                    };
                    if(me._testItemAgainstQuery(item, settings.query)){
                        settings.onItem.call(settings.scope, item, args);
                        foundItems.push(item);
                    }
                }
                
                for(var i = 0; i < numRows; i++){
                    items[i].query = settings.query;
                    if(me._testItemAgainstQuery(items[i], settings.query)){
                        settings.onItem.call(settings.scope, items[i], args);
                        foundItems.push(items[i]);
                    }
                }
                
                settings.onComplete.call(settings.scope, foundItems, args);
            }, 
            error: function(result, ioArgs){
                var args = dojo.mixin(ioArgs, settings);
                settings.onError.call(settings.scope, result, args);
            }
        });
    },
    _testItemAgainstQuery: function(item, query){
        for(var i in query){
            if(query.hasOwnProperty(i) && i !== "provider" && i !== "datasource" && i !== "dataset"){
                var filter = query[i];
                
                if(filter == "*"){
                    continue;
                }
                
                var wildCardIndex = filter.indexOf("*");
                
                var prefix = "^";
                var body = "";
                var suffix = "$";
                if(wildCardIndex == 0){
                    prefix = "";
                    body = filter.substring(1, filter.length);
                } else if(wildCardIndex == filter.length-1){
                    suffix = "";
                    body = filter.substring(0, filter.length-1);
                } else if(wildCardIndex > 0){
                    body = filter.substring(0, wildCardIndex-1) + "(.*)" + filter.substring(wildCardIndex+1, filter.length);
                } else {
                    body = filter;
                }
                
                var regex = new RegExp(prefix+body+suffix);
                if(!regex.test(item[i])){
                    return false;
                }
            }
        }
        return true;
    },
    _sanitizeQuery: function(query){
        var toSanitize = ["provider","datasource","dataset"];
        for(var i = 0, len = toSanitize.length; i < len; i++){
            if(query[toSanitize[i]] != null){
                if(dojo.isString(query[toSanitize[i]])){
                    query[toSanitize[i]] = {
                        id: query[toSanitize[i]]
                    };
                }
            }
        }
        return query;
    },
    _getQueryType: function(query){
        if(query.provider != null){
            if(query.datasource == null){
                return "datasource"
            } else if(query.datasource.id == this.noDatasource) {
                if(query.dataset == null){
                    return "dataset";
                } else {
                    return "columns";
                }
            } else {
                if(query.dataset == null){
                    return "datasetWithDataSource";
                } else {
                    return "columnsWithDataSource";
                }
            }
        } else {
            return "provider";
        }
    },
    _buildContent: function(settings){
        var queryType = this._getQueryType(settings.query);
        var content = {};
        
        if(queryType === "provider"){
        } else if(queryType === "datasource"){
        } else if(queryType === "datasetWithDataSource" || queryType === "dataset"){
        } else if(queryType === "columnsWithDataSource" || queryType === "columns"){
            content.metadata="true";
            content.start="0";
            content.count="1";
            content.properties="all";
        }
        
        return content;
    },
    _buildUrl: function(query){
        var queryType = this._getQueryType(query);
        
        var url = this.baseUrl+"/providers";
        
        if(queryType === "provider"){
            return url;
        } else if(queryType === "datasource"){
            return url+"/"+query.provider.id+"/datasources";
        } else if(queryType === "datasetWithDataSource"){
            return url+"/"+query.provider.id+"/datasources/"+query.datasource.id+"/datasets";
        } else if(queryType === "columnsWithDataSource"){
            return url+"/"+query.provider.id+"/datasources/"+query.datasource.id+"/datasets/"+query.dataset.id+"/items";
        } else if(queryType === "dataset"){
            return url+"/"+query.provider.id+"/datasets";
        } else { // columns
            return url+"/"+query.provider.id+"/datasets/"+query.dataset.id+"/items";
        }
    },
    
    getFeatures: function(){
        return {
            "dojo.data.api.Read": true,
            "dojo.data.api.Identity": true
        };
    },
    
    close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
    },
    
    getLabel: function(/* item */ item){
        if(item.label){
            return item.label;
        }
        return item.id;
    },
    
    getLabelAttributes: function(/* item */ item){
        if(item.label){
            return "label";
        }
        return "id";
    },
    
    getIdentity: function(/* item */ item){
        return dojo.toJson(item);
    },
    
    getIdentityAttributes: function(/* item */ item){
        var attrs = [];
        for(var i in item){
            if(item.hasOwnProperty()){
                attrs.push(i);
            }
        }
        return attrs;
    },
    
    /**
     * {
     *     identity: string|object,
     *     onItem: Function,
     *     onError: Function,
     *     scope: object
     * }
     */
    fetchItemByIdentity: function(/* object */ keywordArgs){
        var settings = {
            identity: "{}",
            onItem: function(){},
            onError: function(){},
            scope: this
        };
        settings = dojo.mixin(settings,keywordArgs);
        
        var item = dojo.fromJson(keywordArgs.identity);
        
        settings.onItem.call(settings.scope, item);
        
        return item;
    },
    
    manipulateURI: function(uri){
        return this.iContextIO.rewriteURI(uri);
    }
});