/**
 * This file contains the unit tests for the NavModelStore 'class'
 * @author Jason Feinstein (jwfeinst@us.ibm.com)
 */

//Declare out the name of the test module to make dojo's module loader happy.
dojo.provide("ibm.tivoli.data.tests._NavModelStore");
dojo.require("ibm.tivoli.data._NavModelStore");

/**
 * Makes sure that the element ID decoder function works correctly.
 */
doh.registerGroup("_NavModelStore.getTypeAndId",
    [
        function testGood(t){
            var store = this.group._store;
            
            t.assertEqual({type: "item", id: "1"}, store.getTypeAndId("navModelItem_item(1)"));
            t.assertEqual({type: "item", id: "asdf"}, store.getTypeAndId("navModelItem_item(asdf)"));
            t.assertEqual({type: "item", id: "\\test\\a"}, store.getTypeAndId("navModelItem_item(\\test\\a)"));
            t.assertEqual({type: "item", id: "jason was here"}, store.getTypeAndId("navModelItem_item(jason was here)"));
            t.assertEqual({type: "task", id: "1"}, store.getTypeAndId("navModelItem_task(1)"));
            t.assertEqual({type: "link", id: "1"}, store.getTypeAndId("navModelItem_link(1)"));
            t.assertEqual({type: "link", id: "www.ij.net"}, store.getTypeAndId("navModelItem_link(www.ij.net)"));
            t.assertEqual({type: "link", id: "jwfeinst@us.ibm.com"}, store.getTypeAndId("navModelItem_link(jwfeinst@us.ibm.com)"));
            t.assertEqual({type: "childDataset", id: "jwfeinst@us.ibm.com"}, store.getTypeAndId("navModelItem_childDataset(jwfeinst@us.ibm.com)"));
        },
        /**
         * These tests make sure that when given bad information or incorrectly 
         * formatted input, errors are thrown.
         */
        function testBad(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getTypeAndId", 
                ["nav_item(1)"]
            );
            t.assertError(Error,
                store, "getTypeAndId", 
                ["navModelItem_taska(1)"]
            );
            t.assertError(Error,
                store, "getTypeAndId", 
                ["NavModelItem_item(1)"],
                "It should be case-sensitive."
            );
            t.assertError(Error,
                store, "getTypeAndId", 
                ["navModelItem_task1)"]
            );
            t.assertError(Error,
                store, "getTypeAndId", 
                ["navModelItem_task(1"]
            );
            t.assertError(Error,
                store, "getTypeAndId", 
                ["navModelItem_task()"]
            );
            t.assertError(Error,
                store, "getTypeAndId", 
                ["navModelItem_(1)"]
            );
            t.assertError(Error,
                store, "getTypeAndId", 
                [null]
            );
        }
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });   
    },
    function(){
        this._store = null;
    }
);

/**
 * Check our object equality mechanism.
 */
doh.registerGroup("_NavModelStore.valuesEqual",
    [
        function evaluateSimple(t){
            var store = this.group._store;
            
            t.assertTrue(store.valuesEqual(null,null), "null == null");
            t.assertFalse(store.valuesEqual(0,null), "0 != null");
            t.assertFalse(store.valuesEqual([],null), "[] != null");
            
            t.assertTrue(store.valuesEqual(true,true));
            t.assertFalse(store.valuesEqual(true,[]),"true != []");
            t.assertFalse(store.valuesEqual(true,1), "true != 1");
            
            t.assertTrue(store.valuesEqual(1,1), "1 == 1");
            t.assertTrue(store.valuesEqual(1,"1"), "1 == '1'");
            t.assertFalse(store.valuesEqual(1,[1]), "1 != [1]");
            t.assertFalse(store.valuesEqual(1,null), "1 != null");
            t.assertFalse(store.valuesEqual(1,{one:1}), "1 != {one:1}");
            
            t.assertTrue(store.valuesEqual("test","test"));
            t.assertFalse(store.valuesEqual("test",'"test"'), "test != 'test'");
            t.assertFalse(store.valuesEqual("test",["test"]), "test != ['test']");
            t.assertFalse(store.valuesEqual("test",null));
            t.assertFalse(store.valuesEqual("test",{test:"test"}));
        },
        
        function evaluateComplex(t){
            var store = this.group._store;
            
            t.assertTrue(store.valuesEqual(
                {
                    a: {
                        b: {
                            c: "c"
                        },
                        d: [1,2]
                    },
                    e: "abc",
                    f: true
                },{
                    a: {
                        b: {
                            c: "c"
                        },
                        d: [1,2]
                    },
                    e: "abc",
                    f: true
                }
            ));
            t.assertFalse(store.valuesEqual(
                {
                    a: {
                        b: {
                            c: "c"
                        },
                        d: [1,2]
                    },
                    e: "abc",
                    f: true
                },{
                    a: {
                        b: {
                            c: "c"
                        },
                        d: [1,3]
                    },
                    e: "abc",
                    f: true
                }
            ));
        }
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });   
    },
    function(){
        this._store = null;
    }
);

/**
 * Check to verify that .containsValue works correctly in the NavModelStore.
 */
doh.registerGroup("_NavModelStore.containsValue",
    [
        /**
         * Make sure things work correctly when we have null or empty input.
         */
        function emptyTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "containsValue", 
                [], 
                "An error should be thrown when no arguments are given."
            );
            t.assertError(Error,
                store, "containsValue", 
                [null, "attrName", "value"], 
                "An error should be thrown when any null arguments are given."
            );
            t.assertError(Error,
                store, "containsValue", 
                [{}, null, "value"], 
                "An error should be thrown when any null arguments are given."
            );
            t.assertError(Error,
                store, "containsValue", 
                [{}, "attrName", null], 
                "An error should be thrown when any null arguments are given."
            );
        },
        
        /**
         * Work with non-item input.
         */
        function notAnItemTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "containsValue", 
                [0, "something", "value"], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "containsValue", 
                ["not an item", "something", "value"], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "containsValue", 
                [
                    {
                        id: "",
                        uri: "",
                        type: ""
                    }, "something", "value"
                ], 
                "An error should be thrown when any arguments are wrong."
            );
        },
        
        /**
         * Work with non-string attribute names.
         */
        function notAnAttributeTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getValues", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, 0, ""
                ], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValues", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, true, ""
                ], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValues", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, {}, ""
                ], 
                "An error should be thrown when any arguments are wrong."
            );
        },
        
        /**
         * Test with an actual item...
         */
        function itemItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._itemItem;
            t.assertTrue(store.containsValue(item, "id", item.id));
            t.assertFalse(store.containsValue(item, "id", "not an id"));
            t.assertFalse(store.containsValue(item, "id", null));
            t.assertTrue(store.containsValue(item, "defaultTask",item.defaultTask));
            t.assertTrue(store.containsValue(item, "uri",item.uri));
            t.assertFalse(store.containsValue(item, "uri",item.properties[0].value, true), 
                    "When there's an overlap in attributes vs. properties, use the attribute's value.");
            t.assertTrue(store.containsValue(item, "anotherProperty", item.properties[1].value[0], true));
            t.assertTrue(store.containsValue(item, "anotherProperty", item.properties[1].value[1], true));
        }, 
        
        function linkItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._linkItem;
            t.assertTrue(store.containsValue(item, "id", item.id));
            t.assertFalse(store.containsValue(item, "id", "not an id"));
            t.assertFalse(store.containsValue(item, "id", null));
            t.assertTrue(store.containsValue(item, "relationships", item.relationships[0]));
            t.assertTrue(store.containsValue(item, "uri",item.uri));
            t.assertTrue(store.containsValue(item, "anotherProperty", item.properties[1].value[0]));
            t.assertTrue(store.containsValue(item, "anotherProperty", item.properties[1].value[1]));
            t.assertFalse(store.containsValue(item, "anotherProperty", "not a value"));
            t.assertFalse(store.containsValue(item, "something that doesnt exist", "value"), 
                    "Should return false when the attribute has no value in the 'item'.");
        },
        
        function taskItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._taskItem;
            t.assertTrue(store.containsValue(item, "id", item.id));
            t.assertTrue(store.containsValue(item, "subTasks",item.subTasks[0]));
        },
        
        function childDatasetItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._childDatasetItem;
            t.assertTrue(store.containsValue(item, "id", item.id));
            t.assertTrue(store.containsValue(item, "itemsUri",item.itemsUri));
            t.assertTrue(store.containsValue(item, "eventId",item.eventId));
        }
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });
                
    },
    function(){
        this._store = null;
    }
);

/**
 * Test to make sure that the getValue function works correctly
 */
doh.registerGroup("_NavModelStore.getValue",
    [
        /**
         * Make sure things work correctly when we have null or empty input.
         */
        function emptyTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getValue", 
                [], 
                "An error should be thrown when no arguments are given."
            );
            t.assertError(Error,
                store, "getValue", 
                [null, "attrName"], 
                "An error should be thrown when any null arguments are given."
            );
            t.assertError(Error,
                store, "getValue", 
                [{}, null], 
                "An error should be thrown when any null arguments are given."
            );
        },
        
        /**
         * Work with non-item input.
         */
        function notAnItemTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getValue", 
                [0, "something"], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValue", 
                ["not an item", "something"], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValue", 
                [
                    {
                        id: "",
                        uri: "",
                        type: ""
                    }, "something"
                ], 
                "An error should be thrown when any arguments are wrong."
            );
        },
        
        /**
         * Work with non-string attribute names.
         */
        function notAnAttributeTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getValue", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, 0
                ], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValue", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, true
                ], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValue", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, {}
                ], 
                "An error should be thrown when any arguments are wrong."
            );
        },
        
        /**
         * Test with an actual item...
         */
        function itemItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._itemItem;
            t.assertEqual(item.id, store.getValue(item, "id"));
            t.assertEqual(item.defaultTask, store.getValue(item, "defaultTask"));
            t.assertEqual(item.uri, store.getValue(item, "uri"));
            t.assertEqual(item.properties[1].value[0], store.getValue(item, "anotherProperty"));
            t.assertEqual(undefined, store.getValue(item, "something that doesnt exist"), 
                    "Should return a undefined response when the attribute has no value in the 'item'.");
            t.assertEqual(1, store.getValue(item, "something that doesnt exist", 1), 
                    "Should return the default value when the attribute has no value in the 'item' but a default was specified.");
        }, 
        
        function linkItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._linkItem;
            t.assertEqual(item.id, store.getValue(item, "id"));
            t.assertEqual(item.relationships[0], store.getValue(item, "relationships"));
            t.assertEqual(item.uri, store.getValue(item, "uri"));
            t.assertEqual(item.properties[1].value[0], store.getValue(item, "anotherProperty"));
            t.assertEqual(undefined, store.getValue(item, "something that doesnt exist"), 
                    "Should return an undefined response when the attribute has no value in the 'item'.");
            t.assertEqual(1, store.getValue(item, "something that doesnt exist", 1), 
                    "Should return the default value when the attribute has no value in the 'item' but a default was specified.");
        },
        
        function taskItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._taskItem;
            t.assertEqual(item.id, store.getValue(item, "id"));
            t.assertEqual(item.subTasks[0], store.getValue(item, "subTasks"));
            t.assertEqual(undefined, store.getValue(item, "something that doesnt exist"), 
                    "Should return undefined when the attribute has no value in the 'item'.");
        },
        
        function childDatasetItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._childDatasetItem;
            t.assertEqual(item.id, store.getValue(item, "id"));
            t.assertEqual(item.eventId, store.getValue(item, "eventId"));
            t.assertEqual(item.itemsUri, store.getValue(item, "itemsUri"));
            t.assertEqual(undefined, store.getValue(item, "something that doesnt exist"), 
                    "Should return undefined when the attribute has no value in the 'item'.");
        }
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });
                
    },
    function(){
        this._store = null;
    }
);

/**
 * Test to make sure that the getValues function works correctly
 */
doh.registerGroup("_NavModelStore.getValues",
    [
        /**
         * Make sure things work correctly when we have null or empty input.
         */
        function emptyTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getValues", 
                [], 
                "An error should be thrown when no arguments are given."
            );
            t.assertError(Error,
                store, "getValues", 
                [null, "attrName"], 
                "An error should be thrown when any null arguments are given."
            );
            t.assertError(Error,
                store, "getValues", 
                [{}, null], 
                "An error should be thrown when any null arguments are given."
            );
        },
        
        /**
         * Work with non-item input.
         */
        function notAnItemTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getValues", 
                [0, "something"], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValues", 
                ["not an item", "something"], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValues", 
                [
                    {
                        id: "",
                        uri: "",
                        type: ""
                    }, "something"
                ], 
                "An error should be thrown when any arguments are wrong."
            );
        },
        
        /**
         * Work with non-string attribute names.
         */
        function notAnAttributeTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getValues", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, 0
                ], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValues", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, true
                ], 
                "An error should be thrown when any arguments are wrong."
            );
            t.assertError(Error,
                store, "getValues", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        properties: []
                    }, {}
                ], 
                "An error should be thrown when any arguments are wrong."
            );
        },
        
        /**
         * Test with an actual item...
         */
        function itemItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._itemItem;
            t.assertEqual([item.id], store.getValues(item, "id"));
            t.assertEqual([item.defaultTask], store.getValues(item, "defaultTask"));
            t.assertEqual([item.uri], store.getValues(item, "uri"));
            t.assertEqual(item.properties[1].displayValue, store.getValues(item, "anotherProperty"));
            t.assertEqual([], store.getValues(item, "something that doesnt exist"), 
                    "Should return an empty array when the attribute has no value in the 'item'.");
        }, 
        
        function linkItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._linkItem;
            t.assertEqual([item.id], store.getValues(item, "id"));
            t.assertEqual(item.relationships, store.getValues(item, "relationships"));
            t.assertEqual([item.uri], store.getValues(item, "uri"));
            t.assertEqual(item.properties[1].value, store.getValues(item, "anotherProperty"));
            t.assertEqual([], store.getValues(item, "something that doesnt exist"), 
                    "Should return an empty array when the attribute has no value in the 'item'.");
        },
        
        function taskItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._taskItem;
            t.assertEqual([item.id], store.getValues(item, "id"));
            t.assertEqual(item.subTasks, store.getValues(item, "subTasks"));
            t.assertEqual([], store.getValues(item, "something that doesnt exist"), 
                    "Should return an empty array when the attribute has no value in the 'item'.");
        },
        
        function childDatasetItemTest(t){
            var store = this.group._store;
            
            var item = ibm.tivoli.data.tests._NavModelStore._childDatasetItem;
            t.assertEqual([item.id], store.getValues(item, "id"));
            t.assertEqual([item.eventId], store.getValues(item, "eventId"));
            t.assertEqual([item.itemsUri], store.getValues(item, "itemsUri"));
            t.assertEqual([], store.getValues(item, "something that doesnt exist"), 
                    "Should return an empty array when the attribute has no value in the 'item'.");
        }
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });
                
    },
    function(){
        this._store = null;
    }
);

/**
 * Make sure that the getAttributes(item) function is working correctly.
 */
doh.registerGroup("_NavModelStore.getAttributes", 
    [
        /**
         * Make sure it works with empty arguments.
         */
        function emptyTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getAttributes", 
                [null], 
                "An error should be thrown when any null arguments are given."
            );
            t.assertError(Error,
                store, "getAttributes", 
                [], 
                "An error should be thrown when any undefined arguments are given."
            );
        },
        
        /**
         * Make sure it works when there is a non-item argument.
         */
        function notAnItemTest(t){
            var store = this.group._store;
            
            t.assertError(Error,
                store, "getAttributes", 
                [1], 
                "An error should be thrown when any non-item arguments are given."
            );
            t.assertError(Error,
                store, "getAttributes", 
                ["test"], 
                "An error should be thrown when any non-item arguments are given."
            );
            t.assertError(Error,
                store, "getAttributes", 
                [
                    {
                        id: "",
                        uri: "",
                        type: ""
                    }
                ], 
                "An error should be thrown when any non-item arguments are given, look closely, this one isn't a true item."
            );
        },
        
        function itemItemTest(t){
            var store = this.group._store;
            var item = ibm.tivoli.data.tests._NavModelStore._itemItem;
            var expectedAttributes = ibm.tivoli.data.tests._NavModelStore._itemItemAttributes;
            
            t.assertEqual(expectedAttributes, store.getAttributes(item).sort());
        },
        
        function linkItemTest(t){
            var store = this.group._store;
            var item = ibm.tivoli.data.tests._NavModelStore._linkItem;
            var expectedAttributes = ibm.tivoli.data.tests._NavModelStore._linkItemAttributes;
            
            t.assertEqual(expectedAttributes, store.getAttributes(item).sort());

        },
        
        function taskItemTest(t){
            var store = this.group._store;
            var item = ibm.tivoli.data.tests._NavModelStore._taskItem;
            var expectedAttributes = ibm.tivoli.data.tests._NavModelStore._taskItemAttributes;
            
            t.assertEqual(expectedAttributes, store.getAttributes(item).sort());
        },
        
        function childDatasetItemTest(t){
            var store = this.group._store;
            var item = ibm.tivoli.data.tests._NavModelStore._childDatasetItem;
            var expectedAttributes = ibm.tivoli.data.tests._NavModelStore._childDatasetItemAttributes;
            
            t.assertEqual(expectedAttributes, store.getAttributes(item).sort());
        }
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });
    },
    function(){
        this._store = null;
        this._itemItem = null;
        this._linkItem = null;
        this._taskItem = null;
    }
);

/**
 * Evaluate the correctness of NavModelStore.hasAttribute.
 */
doh.registerGroup("_NavModelStore.hasAttribute",
    [
        /**
         * Make sure that it functions correctly when given empty/null arguments.
         */
        function emptyTest(t){
            var store = this.group._store;
            
            t.assertError(
                Error, 
                store, "hasAttribute", 
                [], 
                "An error should be thrown when no arguments are given."
            );
            t.assertError(
                Error, 
                store, "hasAttribute", 
                [{}], 
                "An error should be thrown when only one argument is given."
            );
            t.assertError(
                Error, 
                store, "hasAttribute", 
                ["fakeAttributeName"], 
                "An error should be thrown when only one argument is given."
            );
            t.assertError(
                Error, 
                store, "hasAttribute", 
                [null,null], 
                "An error should be thrown when null arguments are given."
            );
            t.assertError(
                Error, 
                store, "hasAttribute", 
                [null,"fakeAttributeName"], 
                "An error should be thrown when any null arguments are given."
            );
            t.assertError(
                Error, 
                store, "hasAttribute", 
                [
                    {
                        id: "",
                        uri: "",
                        type: "",
                        category: ""
                    },
                    null
                ], 
                "An error should be thrown when any null arguments are given."
            );
        },
        
        /**
         * Make sure that an exception is thrown when the function is passed a 
         * non-item.
         */
        function notAnItemTest(t){
            var store = this.group._store;
            
            t.assertError(
                Error, 
                store, "hasAttribute", 
                [
                    {
                        id: "",
                        uri: "",
                        type: ""
                    },
                    null
                ], 
                "This item is *almost* an item, but not quite, so the function should error out.."
            );
        },
        
        /**
         * Make sure that hasAttribute works well for all the types of 
         * attributes we'd ask for from an "Item" item.
         */
        function itemItemTest(t){
            var store = this.group._store;
            var itemItem = ibm.tivoli.data.tests._NavModelStore._itemItem;
            
            t.assertTrue(store.hasAttribute(itemItem, "id"));
            t.assertTrue(store.hasAttribute(itemItem, "uri"));
            t.assertTrue(store.hasAttribute(itemItem, "type"));
            t.assertTrue(store.hasAttribute(itemItem, "origin"));
            t.assertTrue(store.hasAttribute(itemItem, "description"));
            t.assertTrue(store.hasAttribute(itemItem, "label"));
            t.assertTrue(store.hasAttribute(itemItem, "labelQualifier"));
            t.assertTrue(store.hasAttribute(itemItem, "typeLabel"));
            t.assertTrue(store.hasAttribute(itemItem, "tooltip"));
            t.assertTrue(store.hasAttribute(itemItem, "relatedItemsUri"));
            t.assertTrue(store.hasAttribute(itemItem, "relationshipsKey"));
            t.assertTrue(store.hasAttribute(itemItem, "defaultTask"));
            t.assertTrue(store.hasAttribute(itemItem, "anotherProperty"));
            
            
            t.assertFalse(store.hasAttribute(itemItem, "properties"), 
                    "Although 'properties' is an attribute of the item object, it should not be surfaced as such to the user of the NavModelStore");
            t.assertFalse(store.hasAttribute(itemItem, "targetedAndNontargeted"), 
                    "_NavModelStore.hasAttribute() should not drill down into the defaultTask attribute.");
            t.assertFalse(store.hasAttribute(itemItem, "nonexistantAttribute"), 
                    "When looking for attribute that doesn't exist, NavModelStore.hasAttribute() shouldn't return true... duh.");
        },
        
        /**
         * Make sure that hasAttribute works well for all the types of 
         * attributes we'd ask for from a "Link" item.
         */
        function linkItemTest(t){
            var store = this.group._store;
            var linkItem = ibm.tivoli.data.tests._NavModelStore._linkItem;
            
            t.assertTrue(store.hasAttribute(linkItem, "id"));
            t.assertTrue(store.hasAttribute(linkItem, "uri"));
            t.assertTrue(store.hasAttribute(linkItem, "type"));
            t.assertTrue(store.hasAttribute(linkItem, "label"));
            t.assertTrue(store.hasAttribute(linkItem, "tooltip"));
            t.assertTrue(store.hasAttribute(linkItem, "direction"));
            t.assertTrue(store.hasAttribute(linkItem, "containment"));
            t.assertTrue(store.hasAttribute(linkItem, "dataset1"));
            t.assertTrue(store.hasAttribute(linkItem, "dataset2"));
            t.assertTrue(store.hasAttribute(linkItem, "item1"));
            t.assertTrue(store.hasAttribute(linkItem, "item2"));
            t.assertTrue(store.hasAttribute(linkItem, "relationships"));
            t.assertTrue(store.hasAttribute(linkItem, "anotherProperty"));
            
            t.assertFalse(store.hasAttribute(linkItem, "properties"),
                    "Although 'properties' is an attribute of the link object, it should not be surfaced as such to the user of the NavModelStore");
            t.assertFalse(store.hasAttribute(linkItem, "sourceDataset"), 
                    "_NavModelStore.hasAttribute() should not drill down into the relationships array's elements.");
            t.assertFalse(store.hasAttribute(linkItem, "nonexistantAttribute"), 
                    "When looking for attribute that doesn't exist, NavModelStore.hasAttribute() shouldn't return true... duh.");
        },
        
        /**
         * Make sure that hasAttribute works well for all the types of 
         * attributes we'd ask for from a "Task" item.
         */
        function taskItemTest(t){
            var store = this.group._store;
            var taskItem = ibm.tivoli.data.tests._NavModelStore._taskItem;
            
            t.assertTrue(store.hasAttribute(taskItem, "id"));
            t.assertTrue(store.hasAttribute(taskItem, "uri"));
            t.assertTrue(store.hasAttribute(taskItem, "type"));
            t.assertTrue(store.hasAttribute(taskItem, "label"));
            t.assertTrue(store.hasAttribute(taskItem, "category"));
            t.assertTrue(store.hasAttribute(taskItem, "description"));
            t.assertTrue(store.hasAttribute(taskItem, "enabled"));
            t.assertTrue(store.hasAttribute(taskItem, "default"));
            t.assertTrue(store.hasAttribute(taskItem, "targeted"));
            t.assertTrue(store.hasAttribute(taskItem, "targetedAndNontargeted"));
            t.assertTrue(store.hasAttribute(taskItem, "multiTargeted"));
            t.assertTrue(store.hasAttribute(taskItem, "icon"));
            t.assertTrue(store.hasAttribute(taskItem, "subTasks"));
            
            t.assertFalse(store.hasAttribute(taskItem, "nonexistantAttribute"), 
                    "When looking for attribute that doesn't exist, NavModelStore.hasAttribute() shouldn't return true... duh.");
        },
        
        function childDatasetItemTest(t){
            var store = this.group._store;
            var taskItem = ibm.tivoli.data.tests._NavModelStore._childDatasetItem;
            
            t.assertTrue(store.hasAttribute(taskItem, "id"));
            t.assertTrue(store.hasAttribute(taskItem, "uri"));
            t.assertTrue(store.hasAttribute(taskItem, "type"));
            t.assertTrue(store.hasAttribute(taskItem, "label"));
            t.assertTrue(store.hasAttribute(taskItem, "itemsUri"));
            t.assertTrue(store.hasAttribute(taskItem, "eventId"));
            
            t.assertFalse(store.hasAttribute(taskItem, "nonexistantAttribute"), 
                    "When looking for attribute that doesn't exist, NavModelStore.hasAttribute() shouldn't return true... duh.");
        }
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });        
    },
    function(){
        this._store = null;
    }
);

/**
 * Evaluate the correctness of NavModelStore.isItem.
 * 
 * Note: we will not check for absolute schema-correctness, but we will check 
 * cases where certain attributes are nonexistant.
 */
doh.registerGroup("_NavModelStore.isItem",
    [
        /**
         * Make sure that it functions correctly when given empty 
         * arguments.
         */
        function emptyTest(t){                    
            var store = this.group._store;
            
            t.assertEqual(
                store.isItem({
                }), 
                false,
                "Empty object should return false as it doesn't match the correct format."
            );
            t.assertEqual(
                store.isItem(null), 
                false,
                "Null argument should return false because null is not a valid Item."
            );
            t.assertEqual(
                store.isItem(), 
                false,
                "Undefined arguments should also return false because nothing is definitely not a valid Item."
            );
        },
        
        /**
         * Make sure that it functions correctly when given non-
         * object arguments.
         */
        function nonObjectTest(t){
            var store = this.group._store;
            
            t.assertEqual(
                store.isItem(1),
                false,
                "Number literals should not return true, they are not items."
            );
            t.assertEqual(
                store.isItem('abc'),
                false,
                "String literals should not return true, they are not items."
            );
            t.assertEqual(
                store.isItem(/regex/),
                false,
                "Regex's obviously aren't items."
            );
            t.assertEqual(
                store.isItem(function(){}),
                false,
                "Functions aren't items."
            );
            t.assertEqual(
                store.isItem(function namedFunc(){}),
                false,
                "Named functions also aren't items."
            );
        },
        
        /**
         * Make sure that, even when given objects - ones that don't 
         * match what we're looking for - we can return false correctly.
         */
        function badObjectTest(t){
            var store = this.group._store;
            
            t.assertEqual(
                store.isItem({
                }),
                false
            );
            t.assertEqual(
                store.isItem({
                    uri : "",
                    type : ""
                }),
                false,
                "This object is missing the id."
            );  
            t.assertEqual(
                store.isItem({
                    id : "",
                    type : ""
                }),
                false,
                "This object is missing the URI"
            );
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : ""
                }),
                false,
                "This object is missing the type"
            );
        },
        
        /**
         * When given a candidate "Item" item, make sure it meets the 
         * criteria.
         */
        function itemItemTest(t){
            var store = this.group._store;
            
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/datasets/d/items/1",
                    type : ""
                }),
                true,
                "This object contains all the required attributes."
            );
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/datasets/d/1",
                    type : ""
                }),
                false,
                "This object is missing the /items part of the url.."
            );
        },
        
        /**
         * When given a candidate "Link" item, make sure it meets the 
         * criteria.
         */
        function linkItemTest(t){
            var store = this.group._store;
            
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/providers/1/datasets/2/links/3",
                    type : ""
                }),
                true,
                "This object contains all the required attributes."
            );
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/providers/1/datasets/2/asdf/3",
                    type : ""
                }),
                false,
                "This object is missing the /links part of the URI to be considered a valid 'Link' item.."
            );
        },
        
        /**
         * When given a candidate "Task" item, make sure it meets the 
         * criteria.
         */
        function taskItemTest(t){
            var store = this.group._store;
            
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/providers/1/datasets/2/tasks/3",
                    type : ""
                }),
                true,
                "This object contains all the required attributes."
            );
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/providers/1/datasets/2/asdf/3",
                    type : ""
                }),
                false,
                "This object is missing the /tasks part of the URI to be considered a valid 'Task' item.."
            );
        },
        
        /**
         * When given a candidate "Task" item, make sure it meets the 
         * criteria.
         */
        function childDatasetItemTest(t){
            var store = this.group._store;
            
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/providers/1/datasets/2",
                    type : "",
                    itemsUri: "",
                    eventId: ""
                }),
                true,
                "This object contains all the required attributes."
            );
            t.assertEqual(
                store.isItem({
                    id : "",
                    uri : "/providers/1/datasets/2",
                    type : "",
                    eventId: ""
                }),
                false,
                "This object is missing the itemsUri attribute, and cannot be considered a valid 'Child Dataset' item.."
            );
        }
        // ...
    ],
    function(){
        this._store = new ibm.tivoli.data._NavModelStore({
            // TODO: setup stuff here.
        });
    },
    function(){
        this._store = null;
    }
);












/*
 * ==========================================================
 *    Start of test data.
 * ==========================================================
 */


/**
 * A sample task to work with.
 */
ibm.tivoli.data.tests._NavModelStore._childDatasetItem = {
    id: "1", uri: "/providers/1/datasets/2/tasks/1", type: "something",
    label: "something",
    eventId: "eventid",
    itemsUri: "itemsuri/a/b/c",
    __loaded: true
};
ibm.tivoli.data.tests._NavModelStore._childDatasetItemAttributes = "id uri type label eventId itemsUri".split(" ").sort();

/**
 * A sample task to work with.
 */
ibm.tivoli.data.tests._NavModelStore._taskItem = {
    id: "1", uri: "/providers/1/datasets/2/tasks/1", type: "something",
    label: "something",
    category: "",
    description: "description",
    enabled: true,
    "default": true,
    targeted: true,
    targetedAndNontargeted: false,
    multiTargeted: true,
    icon: "/bob.gif",
    __loaded: true,
    subTasks: [
        {
            id: "2", uri: "http://google.com", type: "something",
            label: "something",
            category: "",
            description: "description",
            enabled: true,
            "default": true,
            targeted: true,
            targetedAndNontargeted: false,
            multiTargeted: true,
            icon: "/bob.gif",
            subTasks: []
        }
    ]
};
ibm.tivoli.data.tests._NavModelStore._taskItemAttributes = "id uri type label category description enabled default targeted targetedAndNontargeted multiTargeted icon subTasks".split(" ").sort();

/**
 * A sample link to play with.
 */
ibm.tivoli.data.tests._NavModelStore._linkItem = {
    id: "1", uri: "/providers/1/datasets/2/links/1", type: "something",
    label: "something",
    tooltip: "tooltip",
    direction: 1,
    containment: false,
    dataset1: 'ds1',
    dataset2: 'ds2',
    item1: 'item1',
    item2: 'item2',
    __loaded: true,
    relationships: [
        {
            id: "some stuff in here",
            type: "type",
            label: "label",
            sourceDataset: "source dataset",
            sourceItem: "itemid",
            targetDataSet: "target dataset",
            targetItem: "targetItem"
        },
    ],
    properties: [
        {
            id: "uri",
            value: "http://google.com/something else",
            valueType: 1,
            valueState: 1,
            displayValue: "not equal to the item's uri attribute, overlap issue here!",
            task: null,
            units: "inches",
            required: false,
            editable: false,
            sensitive: false,
            valueMin: 5,
            valueMax: 10,
            valueDefault: 6,
            enumValues: [5,6,7,8,9,10]
        },
        {
            id: "anotherProperty",
            value: ["value","two"],
            valueType: 1,
            valueState: 1,
            displayValue: ["value","two"],
            task: null,
            units: "inches",
            required: false,
            editable: false,
            sensitive: false,
            valueMin: 5,
            valueMax: 10,
            valueDefault: 6,
            enumValues: [5,6,7,8,9,10]
        }
    ]
};
ibm.tivoli.data.tests._NavModelStore._linkItemAttributes = "id uri type label tooltip direction containment dataset1 dataset2 item1 item2 relationships anotherProperty".split(" ").sort();

/**
 * A sample item to work with.
 */
ibm.tivoli.data.tests._NavModelStore._itemItem = {
    id: "1", uri: "/providers/1/datasets/2/items/1", type: "something",
    origin: true,
    description: "something",
    label: "something",
    labelQualifier: "something",
    typeLabel: "something",
    tooltip: "tooltip",
    relatedItemsUri: "///",
    relationshipsKey: "key",
    __loaded: true,
    defaultTask: {
        id: "some stuff in here",
        uri: "////",
        type: "type",
        category: "category",
        enabled: true,
        "default": true,
        targeted: true,
        targetedAndNontargeted: true,
        multiTargeted: true,
        icon: "///",
        subTasks: ""
    },
    properties: [
        {
            id: "uri",
            value: "http://google.com/something else",
            valueType: 1,
            valueState: 1,
            displayValue: "not equal to the item's uri attribute, overlap issue here!",
            task: null,
            units: "inches",
            required: false,
            editable: false,
            sensitive: false,
            valueMin: 5,
            valueMax: 10,
            valueDefault: 6,
            enumValues: [5,6,7,8,9,10]
        },
        {
            id: "anotherProperty",
            value: ["value", "array"],
            valueType: 1,
            valueState: 1,
            displayValue: "value",
            task: null,
            units: "inches",
            required: false,
            editable: false,
            sensitive: false,
            valueMin: 5,
            valueMax: 10,
            valueDefault: 6,
            enumValues: [5,6,7,8,9,10]
        }
    ]
};
ibm.tivoli.data.tests._NavModelStore._itemItemAttributes = "id uri type origin description label labelQualifier typeLabel tooltip relatedItemsUri relationshipsKey defaultTask anotherProperty".split(" ").sort();
