/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractWidgetDef"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.WidgetDelta");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.AbstractWidgetDef._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractWidgetDef");

////////////////////////////////////
// AbstractWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractWidgetDef", null, {
	// private members       
	_args: null,
	_parentWidgetDef: null,	
	_widgetDef: null,
	_userDelta: null,
	_sharedDelta: null,
	_destroyed: false,
	_trace: ibm.tivoli.navmodel.AbstractWidgetDef._trace,

	/*=====
	 constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, profileID, parentWidgetDef, registryLocation, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// profileID: String
		//		optional unique id to associate with reading and writing the
		//		"user" and "shared" deltas to the widget definition.  If not
		//		supplied; a uniqueId is created consisting of all the related
		//		information (i.e. providerType, datasourceType, widgetType, etc)
		// parentWidgetDef: ibm.tivoli.navmodel.AbstractWidgetDef  (or descendent)
		//		optional parent widget definition
		// registryLocation: String
		//		Indicates if "remote" provider definition is in local registry or is
		//		located at the remote rest service.  
		//		values are: "local" or "remote"  (Default for TIP is "local")
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs AbstractWidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			baseUrl: null,
			provider: null,
			datasource: null,
			widget: null,
			definition: null,
			type: "user",
			profileID: null,
			parentWidgetDef: null,
			// TIP policy is to have remote provider widget definitions in local database
			registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
		};
		dojo.mixin(this._args, kwArgs);
		
		// convert datasource of "" to null
		if (this._args.datasource && this._args.datasource.length === 0) {
			this._args.datasource = null;
		}
		
		this._parentWidgetDef = this._args.parentWidgetDef;
		this._fetchWidgetDef();
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Sub-class can override this method and validate/set any defaults that were
		//		not present in the widget def contents
	},
	
	getParentWidgetDef: function() {
		// summary:
		//		Returns parent widget definition 
		return this._parentWidgetDef;
	},
	
	setParentWidgetDef: function(parent) {
		// summary:
		//		Set parent widget definition; will force a
		//		re-fetch of the widget definition from server if
		//		parent has a profileID set and this definition does not.
		this._parentWidgetDef = parent;
		
		// if we do NOT have a profileID; and the new parent does;
		// then we need to re-fetch with the parents profileID
		if (parent && !this._args.profileID && parent.getProfileID()) {
			this._fetchWidgetDef();
		} 
	},
	
	getProfileID: function() {
		// summary:
		//		Returns delta id value if specified;  will inherit value from
		//		parent widget definition chain if not explicitly set
		var profileID = null;
		
		// profileID explicitly declared		
		if (this._args.profileID) {
			profileID = this._args.profileID;
		}
		// inherit profileID from parent
		else if (this._parentWidgetDef) {
			profileID = this._parentWidgetDef.getProfileID();			
		}
		
		return profileID;
	},
	
	setProfileID: function(/*String*/ profileID) {
		// summary:
		//		Set new profileID for the widget definition;
		//		will force a re-fetch of the widget definition from server
		this._args.profileID = profileID;
		this._fetchWidgetDef();
	},	
			
	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the widget def
		return this._args[key];
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		this._destroyed = true;
	},

	isDestroyed: function() {
		// summary:
		//		Returns true if the widget def has already been destroyed
		return this._destroyed;
	},

	getDef: function() {
		// summary:
		//		Returns the entire widget definition
		if (this._widgetDef) {
			return this._widgetDef;
		}

		return null;
	},

	getSection: function(section) {
		// summary:
		//		Returns the requested section of the widget definition
		if (this._widgetDef) {
			return this._widgetDef[section];
		}

		return null;
	},
	
	/////////////////////////////
	// meta section data
	/////////////////////////////
	
	getMeta: function() {
		return this._widgetDef.meta;		
	},
	
	getWidgetId: function() {
		return this._widgetDef.meta.id;
	},
	
	getWidgetType: function() {
		return this._widgetDef.meta.type;
	},
	
	getWidgetVersion: function() {
		return this._widgetDef.meta.version;
	},
	
	getWidgetLabel: function() {
		return this._widgetDef.meta.label.value;
	},
	
	getWidgetDescription: function() {
		return this._widgetDef.meta.description.value;
	},
	
	setMeta: function(meta) {
		this._widgetDef.meta = meta;		
	},
	
	setWidgetId: function(id) {
		this._widgetDef.meta.id = id;
	},
	
	setWidgetType: function(type) {
		this._widgetDef.meta.type = type;
	},
	
	setWidgetVersion: function(version) {
		this._widgetDef.meta.version = version;
	},
	
	setWidgetLabel: function(value) {
		this._widgetDef.meta.label.value = value;
	},
	
	setWidgetDescription: function(value) {
		this._widgetDef.meta.description.value = value;
	},

	/////////////////////////////
	// provider section data
	/////////////////////////////
	
	getProvider: function() {
		return this._widgetDef.provider;		
	},

	getProviderType: function() {
		return this._widgetDef.provider.type;
	},
	
	getProviderLabel: function() {
		return this._widgetDef.provider.label.value;
	},
	
	getProviderDescription: function() {
		return this._widgetDef.provider.description.value;
	},
	
	setProvider: function(provider) {
		this._widgetDef.provider = provider;		
	},

	setProviderType: function(type) {
		this._widgetDef.provider.type = type;
	},
	
	setProviderLabel: function(value) {
		this._widgetDef.provider.label.value = value;
	},
	
	setProviderDescription: function(value) {
		this._widgetDef.provider.description.value = value;
	},
	
	/////////////////////////////
	// datasource section data
	/////////////////////////////
	
	getDatasource: function() {
		return this._widgetDef.datasource;		
	},
	
	getDatasourceType: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.type;
		}
		
		return null;
	},
	
	getDatasourceLabel: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.label.value;
		}
		
		return null;
	},
	
	getDatasourceDescription: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.description.value;
		}
		
		return null;
	},
	
	setDatasource: function(datasource) {
		this._widgetDef.datasource = datasource;		
	},
	
	setDataSetDatasourceType: function(type) {
		this._widgetDef.datasets[0].datasourceType = type;
	}, 
	
	setDatasourceType: function(type) {
		this._widgetDef.datasource.type = type;
	},
	
	setDatasourceLabel: function(value) {
		this._widgetDef.datasource.label.value = value;
	},
	
	setDatasourceDescription: function(value) {
		this._widgetDef.datasource.description.value = value;
	},	

	/////////////////////////////
	// details section data
	/////////////////////////////
	
	getDetails: function() {
		return this._widgetDef.details;		
	},
	
	getDetailsAttribute: function(attr) {
		return this._widgetDef.details[attr];
	},
	
	setDetails: function(details) {
		this._widgetDef.details = details;
	},
	
	setDetailsAttribute: function(key, value) {
		if (!this._widgetDef.details) {
			this._widgetDef.details = {};
		}
		
		this._widgetDef.details[key] = value;
	},
		
	/////////////////////////////
	// WidgetDelta
	/////////////////////////////
	getWidgetDelta: function(/*String*/ type) {
		// summary:
		//		Get ibm.tivoli.navmodel.WidgetDelta object
		//	type: String
		//		The delta type: "user" or "shared";  defaults to the 
		//		type passed to widget def constructor if not supplied.		
		if (!type) {
			type = this._args.type;			
		}
		
		if (!type || type === "base") {
			type = "user";
		}
		
		var delta = null;
		switch(type) {
			case "user":
				if (!this._userDelta) {
					this._userDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._userDelta;
				break;
			case "shared":
				if (!this._sharedDelta) {
					this._sharedDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._sharedDelta;
		}
				
		return delta;
	},
	
	saveWidgetDelta: function() {
		// summary:
		//		Save pending changes on ibm.tivoli.navmodel.WidgetDelta object(s)
		if (this._userDelta && this._userDelta.hasPendingChanges()) {
			this._userDelta.save();
		}
		
		if (this._sharedDelta && this._sharedDelta.hasPendingChanges()) {
			this._sharedDelta.save();
		}
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if ibm.tivoli.navmodel.WidgetDelta object(s) have pending changes
		return ((this._userDelta && this._userDelta.hasPendingChanges()) || (this._sharedDelta && this._sharedDelta.hasPendingChanges()));
	},
	
	/////////////////////////////
	// raw user delta
	/////////////////////////////
	hasUserDelta: function() {
		// summary:
		//		indicates if raw user delta json object currently exists
		if (this._widgetDef.user) {
			return true;
		}
		
		return false;
	},
	
	getUserDelta: function() {
		// summary:
		//		Get raw user delta json object		
		return this._widgetDef.user;
	},
	
	setUserDelta: function(userDelta) {
		// summary:
		//		PUT raw user delta json object to the server	
		this._putDelta(userDelta, "user");
		this._fetchWidgetDef();
	},
	
	deleteUserDelta: function() {
		// summary:
		//		DELETE raw user delta json object on server
		this._deleteDelta("user");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// raw shared delta
	/////////////////////////////
	hasSharedDelta: function() {
		// summary:
		//		indicates if raw shared delta json object currently exists
		if (this._widgetDef.shared) {
			return true;
		}
		
		return false;
	},
	
	getSharedDelta: function() {
		// summary:
		//		Get raw shared delta json object		
		return this._widgetDef.shared;
	},
	
	setSharedDelta: function(sharedDelta) {
		// summary:
		//		PUT raw shared delta json object to the server	
		this._putDelta(sharedDelta, "shared");
		this._fetchWidgetDef();
	},
	
	deleteSharedDelta: function() {
		// summary:
		//		DELETE raw shared delta json object on server
		this._deleteDelta("shared");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	getSelectionMode: function() {
		return null;
	},
	
	getItemStyles: function() {
		return null;		
	},
	
	getItemStyle: function(type) {
		return null;
	},
	
	getDefaultItemStyle: function() {
		return null;
	},
	
	getLinkStyles: function() {
		return null;		
	},
	
	getLinkStyle: function(type) {
		return null;		
	},

	getDefaultLinkStyle: function() {
		return null;
	},
	
	getItemStatuses: function() {
		return null;
	},
	
	getItemStatus: function(id) {
		return null;
	},
	
	getDefaultItemStatus: function() {
		return null;
	},
	
	getLinkStatuses: function() {
		return null;	
	},
	
	getLinkStatus: function(id) {
		return null;
	},
	
	getDefaultLinkStatus: function() {
		return null;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._args.version;
	},
	
	_fetchWidgetDef: function() {
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = this._args.type;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrGet({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': "application/json"
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs) {
					this._widgetDef = result;
					this._userDelta = null;
					this._sharedDelta = null;
					this.validateDefaults();		
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchWidgetDef", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_putDelta: function(/*Object*/ delta, /*String*/ deltaType) {
		// summary:
		//		Save delta information for the widget definition
		// delta:
		//		contains the delta to the definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;		
		var json = dojo.toJson(delta, false);
		
		dojo.xhrPut({
			url: this._getPutURL(deltaType),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': 'application/json',
				'Content-Length': json.length
			},
			handleAs: "json",
			putData: json,
			sync: true,
			load: dojo.hitch(this, function(result, ioArgs) {									
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_putDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_deleteDelta: function(/*String*/ deltaType) {
		// summary:
		//		Delete delta information for the widget definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = deltaType;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrDelete({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: function(result, ioArgs) {									
			},
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_deleteDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_content: function() {
		// summary:
		//		Construct a "content" object for sending an xdr request;
		//		IE will sometimes return http requests from cache without unique parms
		var content = {};
		
		// add date and random
		var now = new Date();		
		content.d = now.getTime();
		content.r = Math.random();
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			content.registryLocation = this._args.registryLocation;
		}
		
		return content;
	},

	_getDatasourceURL: function() {
		// summary:
		//		Returns url for fetching a datasource
		var url = null;
		
		if (this._widgetDef) {
			var providerId = this._args.provider;
			if (this.getProviderType() === "any") {
				providerId = "any";
			}
			
			var datasourceId = this._args.datasource;
			if (this.getDatasourceType() === "any") {
				datasourceId = "any";
			}
			
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(providerId);
			url += (datasourceId ? ("/datasources/" + encodeURIComponent(datasourceId)) : "");
		}
		else {
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(this._args.provider);
			url += (this._args.datasource ? ("/datasources/" + encodeURIComponent(this._args.datasource)) : "");
		}
		
		return url;	
	},

	_getWidgetDefinitionURL: function() {
		// summary:
		//		Returns url for fetching a widget definition
		var url = this._getDatasourceURL();
		
		if (this._widgetDef) {
			url+= "/widgets/"+ this.getWidgetType() + "/definitions/" + this.getWidgetId() + "-" + this.getWidgetVersion();
		}
		else {
			url+= "/widgets/"+ this._args.widget + "/definitions/" + this._args.definition;			
		}
		
		return url;
	},
	
	_getPutURL: function(deltaType) {
		// summary:
		//		Build url necessary to do a put or post operation;
		//		have to add on the "type" and "version" ourself
		//		since the dojo.xhr function doesn't include the content
		var url = this._getWidgetDefinitionURL();
		var questionMark = false;
		
		// delta type
		if (!questionMark) {
			questionMark = true;
			url += "?type=" + deltaType;
		}
		else {
			url += "&type=" + deltaType;
		}
		
		// profileID if specified
		var profileID = this.getProfileID();
		if (profileID) {
			if (!questionMark) {
				questionMark = true;
				url += "?profileID=" + profileID;
			}
			else {
				url += "&profileID=" + profileID;
			}
		}
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			if (!questionMark) {
				questionMark = true;
				url += "?registryLocation=" + this._args.registryLocation;
			}
			else {
				url += "&registryLocation=" + this._args.registryLocation;
			}
		}
						
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		return url;
	}
});
