/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */
 
dojo.provide("ibm.tivoli.navmodel.DatasetStore");

// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
	
ibm.tivoli.navmodel.DatasetStore._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.DatasetStore");
	
dojo.declare("ibm.tivoli.navmodel.DatasetStore", null, {
	// private members 
	_listening: false,
	_refresh: false,
	_sortIds: null,
	_conditionIds: null,	
	_trace: ibm.tivoli.navmodel.DatasetStore._trace,
	
	/*=====
	constructor_kwArgs: function(dataset, defaultProperties, returnItem) {
		// summary:
		//		An anonymous object used to configure the datastore
		// dataset: Object
		//		A ibm.tivoli.navmodel.Dataset instance
		// defaultProperties: String[]
		//		An array of property id values that are pre-fetched with the items
		// returnItem: Boolean
		//		A boolean indicating if the getValue() method should just return the actual
		//		item so that it can be used for custom formatting of the value
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs){
		// summary:
		//		Constructor for abstract base class dojo datastore;
		//		that receives ids data from an ibm.tivoli.navmodel.Dataset
		var args = {
			dataset: null,
			defaultProperties: ["all"],
			returnItem: false
		};
		dojo.mixin(args, kwArgs);
		dojo.mixin(this, args);
		
		// set up Dataset event handler
		this._setupEventHandler();			
    },
    
    CURIRefresh: function() {
		// summary:
		//		invoked by TED to indicate "Refresh" is occurring now
		this._refresh = true;
		
		if (this.dataset.isClearCacheOnRefresh()) {
			this.dataset._clearAllCaches();
		}
	},
	
	isRefresh: function() {
		// summary:
		//		returns true if TED refresh is occurring
		var refresh = this._refresh;
		this._refresh = false;
		return refresh;
	},
	

	//////////////////////////////////////////////////////////////
	// dojo.data.api.Identity
	//////////////////////////////////////////////////////////////
	
	getIdentityAttributes: function(/*Object*/ item){
		// summary:
		//		Returns an array of attribute names that are used to generate the identity.
		//		For most stores, this is a single attribute, but for some complex stores
		//		such as RDB backed stores that use compound (multi-attribute) identifiers
		//		it can be more than one. If the identity is not composed of attributes
		//		on the item, it will return null. This function is intended to identify
		//		the attributes that comprise the identity so that so that during a render
		//		of all attributes, the UI can hide the the identity information if it
		//		chooses.
		// item:
		//		The item from the store from which to obtain the array of public attributes that
		//		compose the identifier, if any.
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return null;
	},
    
    
	//////////////////////////////////////////////////////////////
	// dojo.data.api.Read
	//////////////////////////////////////////////////////////////
	
	getFeatures: function(){    	
		// summary:
		//		The getFeatures() method returns an simple keyword values object
		//		that specifies what interface features the datastore implements.
		return {
			'dojo.data.api.Read': true,
			'dojo.data.api.Identity': true,
			'dojo.data.api.Notification':true
		};
	},
		
	close: function(/*Object*/ request) {
		// summary:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.
		// description:
		//		The close() method is intended for instructing the store to 'close' out
		//		any information associated with a particular request.  In general, this API
		//		expects to receive as a parameter a request object returned from a fetch.
		//		It will then close out anything associated with that request, such as
		//		clearing any internal datastore caches and closing any 'open' connections.
		//		For some store implementations, this call may be a no-op.
		// request:
		//		An instance of a request for the store to use to identify what to close out.
		//		If no request is passed, then the store should clear all internal caches (if any)
		//		and close out all 'open' connections.  It does not render the store unusable from
		//		there on, it merely cleans out any current data and resets the store to initial
		//		state.	
		if (!request) {
			this._cleanupEventHandler();
		}		
	},
  
	isItem: function(/*Object*/ something){
		// summary:
		//		Returns true if *something* is an item and came from the store instance.
		//		Returns false if *something* is a literal, an item from another store instance,
		//		or is any object other than an item.
		// something:
		//		Can be anything.
		if (!something || (typeof something !== "object")) {
			return false;
		}
                
		try {
			return this._getObjectTypes()[something.getObjectType()];			
		}
		catch(err){
			return false;    		
		}	
	},
	
	isItemLoaded: function(/*Object*/ item) {
		// summary:
		//		Returns false if isItem(something) is false.  Returns false if
		//		if isItem(something) is true but the the item is not yet loaded
		//		in local memory (for example, if the item has not yet been read
		//		from the server).
		// item:
		//		Item previously handed out by datastore.
		//		ibm.tivoli.navmodel.Item object
		if (!item || !this.isItem(item)){
  			return false;
  		}
  		
  		return !item.isExpired();
	},	
	
	getLabel: function(/*Object*/ item) {
		// summary:
		//		Method to inspect the item and return a user-readable 'label' for the item
		//		that provides a general/adequate description of what the item is.
		// item:
		//		The item to return the label for.
		//		ibm.tivoli.navmodel.Item object
		// returns:
		// 		A user-readable string representing the item or undefined if no user-readable label can
		//		be generated.	
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return item.getLabel();
	},
  
	getLabelAttributes: function(/*Object*/ item) {
		// summary:
		//		Method to inspect the item and return an array of what attributes of the item were used
		//		to generate its label, if any.
		// item:
		//		The item to return the list of label attributes for.
		//		ibm.tivoli.navmodel.Item object
		// returns:
		//		An array of attribute names that were used to generate the label, or null if public attributes
		//		were not used to generate the label.  
		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
		return ["label"];
	},
  
 	getAttributes: function(/*Object*/ item) {
		// summary:
		//		Returns an array with all the attributes that this item has.  This
		//		method will always return an array; if the item has no attributes
		//		at all, getAttributes() will return an empty array: [].
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// exceptions:
		//		Throws an exception if *item* is not an item
  		if (!this.isItem(item)){
  			throw new Error("Specified item argument is not a valid item.");
  		}
  		
  		return item.getCachedPropertyIds(true);  		
    },
    
	hasAttribute: function(/*Object*/ item, /*String*/ attribute) {
		// summary:
		//		Returns true if the given *item* has a value for the given *attribute*.
		// item:
		//		The item to access attributes on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string		
		if (!item) {
			return false;
		}
		
		if (!this.isItem(item)){
			throw new Error("Specified item argument is not a valid item.");
		}
    	
		if (!dojo.isString(attribute)) {
			throw new Error("Specified attribute argument is not a string.");
		}
        
		var attributes = this.getAttributes(item);
		if (attributes) {
			var attrsLength = attributes.length;
			for (var attrIndex = 0;  attrIndex < attrsLength;  ++ attrIndex) {
				if (attribute === attributes[attrIndex]) {
					return true;
				}
			}
		}
    	
		return false;
	},
    
  getValues: function(/*Object*/ item, /*String*/ attribute, /*Object?*/ defaultValue, /*Object*/ useValue) {
		// summary:
		//		This getValues() method works just like the getValue() method, but getValues()
		//		always returns an array rather than a single attribute value.  The array
		//		may be empty, may contain a single attribute value, or may contain many
		//		attribute values.
		//		
		//		If the item does not have a value for the given attribute, then getValues()
		//		will return an empty array: [].  (So, if store.hasAttribute(item, attribute)
		//		returns false, then store.getValues(item, attribute) will return [].)
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string 	
    	return [this.getValue(item, attribute, defaultValue, useValue)];
    },
    
	containsValue: function(/*Object*/ item, /*String*/ attribute, /*Object*/ value) {
		// summary:
		//		Returns true if the given *value* is one of the values that getValues()
		//		would return.
		// item:
		//		The item to access values on.
		//		ibm.tivoli.navmodel.Item object
		// attribute:
		//		The attribute to access represented as a string.
		// value:
		//		The value to match as a value for the attribute.
		// exceptions:
		//		Throws an exception if *item* is not an item, or *attribute* is not a string
		var values = this.getValues(item, attribute);
		if (values) {
			var valuesLength = values.length;
				for (var valueIndex = 0;  valueIndex < valuesLength;  ++valueIndex) {
				if (value === values[valueIndex]) {
					return true;
				}
			}
		}
    	
		return false;
    },
	
	
    //////////////////////////////////////////////////////////////
	// dojo.data.api.Notification
	//////////////////////////////////////////////////////////////
	
	onSet: function(/*Object*/ item, 
					/*String*/ attribute, 
					/*Object[]*/ oldValue,
					/*Object[]*/ newValue) {
		// summary: 
		//		See dojo.data.api.Notification.onSet()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it.
	},

    onNew: function(/*Object*/ newItem, /*Object?*/ parentInfo) {
		// summary: 
		//		See dojo.data.api.Notification.onNew()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it. 
	},

    onDelete: function(/*Object*/ deletedItem) {
		// summary: 
		//		See See dojo.data.api.Notification.onDelete()
		
		// No need to do anything. This method is here just so that the 
		// client code can connect observers to it. 
	},
	
	onRefresh: function() {
		// summary: 
		//		This is NOT part of the dojo.data.api.Notification but can be attached to
		//		to indicate that the datastore items have changed dramatically and should
		//		be re-fetched.
		//
		// 		Refresh is coming as a result of async notification instead
		// 		of the actual Refresh button being clicked on
		this._refresh = true;		
	},
		
		
	//////////////////////////////////////////////////////////////
	// private methods
	///////////////////////////////////////////////////////////////
   
	_hasSort: function() {
		// summary: 
		//		Returns true if a sort is active
		return (this._sortIds ? true : false);
	},
	
	_affectsSort: function(/*String*/ propId) {
		// summary: 
		//		Returns true if propId is involved in sort
		return (this._sortIds && this._sortIds[propId]);
	},
	
	_hasCondition: function() {
		// summary: 
		//		Returns true if the filter condition(s) are active
		return (this._conditionIds ? true : false);
	},
	
	_affectsCondition: function(/*String*/ propId) {
		// summary: 
		//		Returns true if propId is involved in filter condition(s)
		return (this._conditionIds && this._conditionIds[propId]);
	},
	
	_getSort: function(/*Object[]*/ sort) {
		// summary: 
		//		Returns comma delimited sort string; indicating descending sort with a preceding "-"
 		this._sortIds = null;		
		var sortOrder = null;	 	
		if (sort && sort.length > 0) {			
			this._sortIds = {};						
			sortOrder = "";
			var sortLength = sort.length;
			for (var sortIndex = 0;  sortIndex < sortLength;  ++sortIndex){
				if (sortIndex > 0) {
					sortOrder += ",";
				}
				
				if (sort[sortIndex].descending) {
					sortOrder += "-";
				}
				
				var propId = sort[sortIndex].attribute;
				sortOrder += propId;
				
        		// remember sort ids
        		this._sortIds[propId] = true;				
        	}        	        	
        }        
        return sortOrder;
 	},
 		
 	_getCondition: function(/*Object*/ query) {
		// summary: 
		//		Returns array of condition objects involved in the current filter
		//			ibm.tivoli.navmodel.[String|Number|Date|Boolean|Enum|Map]Condition
		this._conditionIds = null;
		var condition = null;
 		 		
		if (query) {
			// new query format
			if (query.expression && query.ids) {
				this._conditionIds = query.ids;
				condition = query.expression;
			}
			// old "search" format compatible with original SAPM datastore
			else if (query.navModel && query.navModel.search && query.navModel.search.term) {
				this._conditionIds = {};
				condition = "(";
				
				var searchLength = query.navModel.search.attributes.length;
 				for (var searchIndex = 0;  searchIndex < searchLength;  ++searchIndex) {
 					if (searchIndex > 0) {
 						condition += " or ";
 					}
 					
 					var id = query.navModel.search.attributes[searchIndex];
 					condition += (id + " contains '" + query.navModel.search.term + "'");
 					this._conditionIds[id] = true;
 				}
 				
 				condition += ")";
			}
		}
 		
		return condition;
 	},
 	
 	_validateEventHandler: function() {
 		// summary: 
		//		Ensure that we are listening for events if we are supposed to be;
		//		a close() invocation could have caused us to stop listening
		if (!this._listening) {
			this._setupEventHandler();
		}
	},
    
 	_setupEventHandler: function() {
		// summary: 
		//		Add event listener to the dataset
 		if (!this._listening && this.dataset && this.dataset.isEventsEnabled()) {
			this.dataset.addEventListener(this, dojo.hitch(this, this._eventHandler));
			this._listening = true;
		}
 	},
 	
 	_cleanupEventHandler: function() {
		// summary: 
		//		Remove event listener from the dataset
 		if (this._listening && this.dataset && this.dataset.isEventsEnabled()) {
			this.dataset.removeEventListener(this);
			this._listening = false;			
		}
 	},
 	
	_eventHandler: function(/*Object*/ listenerObj, /*Object*/ event) {
		// summary: 
		//		Handle asynchronous events from the dataset
 		if (this._trace.isFine()) {
				this._trace.fine("datasetId=" + event.getDataset().getId() + " event=" + event.getType() +  " eventId=" + event.getId(), "_eventHandler");
		}
			
		switch(event.getType()) {
			case "changed":
				this._handleChangedEvent(event);
				break;
			case "labelChanged":
				this._handleLabelChangedEvent(event);
				break;
			case "statusChanged":
				this._handleStatusChangedEvent(event);
				break;					
			case "itemsAdded":
				this._handleItemsAddedEvent(event);
				break;
			case "itemsRemoved":
				this._handleItemsRemovedEvent(event);
				break;
			case "itemStatusChanged":
				this._handleItemStatusChangedEvent(event);
				break;
			case "itemPropertiesChanged":
				this._handleItemPropertiesChangedEvent(event);
				break;
			case "linksAdded":
				this._handleLinksAddedEvent(event);
				break;
			case "linksRemoved":
				this._handleLinksRemovedEvent(event);
				break;
			case "linkStatusChanged":
				this._handleLinkStatusChangedEvent(event);
				break;
			case "linkPropertiesChanged":
				this._handleLinkPropertiesChangedEvent(event);
				break;
			case "linkChanged":
				this._handleLinkChangedEvent(event);
				break;
			case "relationshipStatusChanged":
				this._handleRelationshipStatusChangedEvent(event);
				break;
			case "relationshipPropertiesChanged":
				this._handleRelationshipPropertiesChangedEvent(event);
				break;
			case "exception":
				this._handleExceptionEvent(event);
				break;
			case "eventRouter":
				this._handleEventRouterEvent(event);
				break;
			default:
				if (this._trace.isFine()) {
					this._trace.fine("datasetId=" + event.getDataset().getId() + " unrecognized event=" + event.getType(), "_eventHandler");
				}									
		}
	},
	
	_handleChangedEvent: function(/*Object*/ event) {
		// signal that data from store should be re-requested by consumer
		this.onRefresh();	
	},
	
	_handleLabelChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemsAddedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemsRemovedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleItemPropertiesChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinksAddedEvent: function(/*Object*/ event) {	
	},
	
	_handleLinksRemovedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinkStatusChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleLinkPropertiesChangedEvent: function(/*Object*/ event) {				
	},
	
	_handleLinkChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleRelationshipStatusChangedEvent: function(/*Object*/ event) {			
	},
	
	_handleRelationshipPropertiesChangedEvent: function(/*Object*/ event) {		
	},
	
	_handleExceptionEvent: function(/*Object*/ event) {		
	},
	
	_handleEventRouterEvent: function(/*Object*/ event) {		
	}
}); 
