/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Item"); 
 
// includes
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status");
dojo.require("ibm.tivoli.navmodel.LinkStyle");
dojo.require("ibm.tivoli.navmodel.Exception");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.Item._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Item");

////////////////////////////////////
// Item
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Item", null, {
	// private members       
	_item: null,
	_dataset: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Item._trace,
	
	/*=====
	constructor_kwArgs: function(item, dataset, propIds, allProperties, allMetadata, expiration) {
		// item: Object
		//		son item object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the item already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if item properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Item object.
		this._item = kwArgs.item;
		if (!this._item) {
			this._item = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};			
		this._allProps = kwArgs.allProperties;
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._item.properties) {
			var propsLength = this._item.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._item.properties[propIndex], dataset: this._dataset, objectType: "item"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
		
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	mixin: function(item, resetLazyLoad) {
		// summary:
		//		Mixin properties and status from this Item object	
		if (item && item !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			// reset lazy load indicators;  mixin will
			// provide the current values
			if (resetLazyLoad) {
				this.resetLazyLoad();
			}
			
			this.mixinAttributes(item.getAttributes());
			this.mixinProperties(item._props, item._nullPropIds);
			this._status = item.getStatuses({widgetDef: null});
			this._defaultTask = item.getDefaultTask();			
			this._allProps = (this._allProps || item._allProps);
			this._allMetadata = (item._allProps && item._allMetadata);				
		}	
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the item
		return this._dataset;
	},

	getObjectType: function() {
		// summary:
		//		Indicate that this object is an "item"
		return "item";
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._item[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._item[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._item[key];
	},

	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the item
		var item = dojo.clone(this._item);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			item.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return item;		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the item
		dojo.mixin(this._item, attr);		
	},

	getUri: function() {
		if (!this._item.uri) {
			this._item.uri = this._dataset.getAttribute("uri") + "/items/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._item.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns item id
		return this._item.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the item id
		this._item.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of item
		return this._item.type;
	},
	
	setType: function(type) {		
		// summary:
		//		Set the item type
		this._item.type = type;		
	},

	getTypeLabel: function() {		
		// summary:
		//		Returns globalized label for the item type
		var label = this._item.typeLabel;
		if (!label) {
			var style = this.getItemStyle({widgetDef: null});
			if (style) {
				label = style.getLabel();
			}
		}
		
		return label;
	},

	setTypeLabel: function(typeLabel) {		
		// summary:
		//		Set the globalized label for the item type
		this._item.typeLabel = typeLabel;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the item
		return this._item.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the item
		this._item.label = label;		
	},

	getDescription: function() {		
		// summary:
		//		Returns globalized description for the item
		return this._item.description;
	},

	setDescription: function(description) {		
		// summary:
		//		Set the globalized description for the item
		this._item.description = description;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized toolip for the item
		return this._item.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the item
		this._item.tooltip = tooltip;
	},
	
	isLazyLoad: function() {		
		// summary:
		//		Returns true if the item is elligible for "lazy load"
		return (this._item.lazyLoad !== undefined ? this._item.lazyLoad : false);
	},

	setLazyLoad: function(lazyLoad) {		
		// summary:
		//		Set the lazy load attribute
		this._item.lazyLoad = lazyLoad;
	},

	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._item.relationshipsKey;
	},

	resetLazyLoad: function() {
		// summary:
		//		Resets current attributes determing whether a lazyLoad
		//		should be performed to find related items
		delete this._item.lazyLoad;
		delete this._item.relationshipsKey;				
	},
	
	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._item.relationshipsKey = relationshipsKey;
	},

	isSubgraph: function() {
		// summary:
		//		Returns true if the item represents a nested subgraph parent
		return (this._item.subgraph !== undefined ? this._item.subgraph : false);
	},
	
	setSubgraph: function(subgraph) {		
		// summary:
		//		Set the attribute indicating if the item is a nested subgraph parent
		this._item.subgraph = subgraph;
	},
	
	getSubgraphId: function() {
		// summary:
		//		Returns id of nested subgraph parent; no value if item is not in a subgraph
		return this._item.subgraphId;
	},
	
	setSubgraphId: function(subgraphId) {
		// summary:
		//		Set the id of the sub-graph parent
		this._item.subgraphId = subgraphId;
	},
	
	isSubgraphOrigin: function() {
		// summary:
		//		Returns true if the item is an origin within a nested subgraph
		return (this._item.subgraphOrigin !== undefined ? this._item.subgraphOrigin : false);
	},
	
	setSubgraphOrigin: function(subgraphOrigin) {
		// summary:
		//		Set the attribute indicating if the item is an origin within a nested subgraph
		this._item.subgraphOrigin = subgraphOrigin;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the item status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
				
		if (!this._status && this._item.status) {
			this._status = [];
			var statusLength = this._item.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._item.status.label && this._item.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._item.status[statusIndex], version: this._version()}));				
				}
				else {
					var statusArgs = {
						type: "item",
						statusId: this._item.status[statusIndex].id,
						statusValue: this._item.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "item");
			}			
		}
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the item status values
		this._status = statuses;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the item
		return (this._item.supportsProperties !== undefined ? this._item.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether item  supports "Properties"
		this._item.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if item has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if item has all properties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration(kwArgs);
				
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}
	},

	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}					
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
						
		// check prop cache
		var prop = this._propsById[propId];
		
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on item	
			if (this._item.properties) {
				var propsLength = this._item.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._item.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "item"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);			
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var item = dojo.clone(this._item);
			item.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the item
		// property:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var item = dojo.clone(this._item);
			item.properties = [property.getAttributes()];
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnItemStyle: function() {		
		// summary:
		//		Returns true if the item has an ItemStyle that overrides the default
		//		item style for the item type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._item.style ? true: false);
	},
	
	/*=====
	getItemStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.ItemStyle for this item
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._item.style) {
				this._style = new ibm.tivoli.navmodel.ItemStyle({style: this._item.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getItemStyle({itemType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef);
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createItemStyle(args.widgetDef, widgetStyle);
				}
			}			
		}
		
		return style;
	},

	setItemStyle: function(style) {		
		// summary:
		//		Set item style for the item
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the item
		if (!this._defaultTask && this._item.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._item.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the item
		this._defaultTask = defaultTask;
	},

	isOrigin: function() {		
		// summary:
		//		Returns true if the item is an origin of a topology or tree
		return (this._item.origin !== undefined ? this._item.origin : false);
	},

	setOrigin: function(isOrigin) {		
		// summary:
		//		Set the value indicating if the Item is an origin of a topology or tree
		this._item.origin = isOrigin;		
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationships: String
		//		comma delimited list of relationships to determine related items
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		keywords: child, parent, all (if null; value from item will be used)
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: this.getId(),
			relationships: this.getLazyLoadRelationships(),
			start: 0,
			count: null,
			properties: null,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		if (!args.relationships) {
			args.relationships = this.getLazyLoadRelationships();
		}
		
		if (!args.properties) {
			args.properties = this.getCachedPropertyIds();
		}
		
		// call the dataset to get the related items
		this._dataset.getRelatedItems(args);
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
			
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var item = this._dataset.getItem({itemId: this.getId(), properties: properties, metadata: metadata});
				if (item) {
					this.mixin(item);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},
	
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "item"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
				
		var content = ibm.tivoli.navmodel.Utils.getContent();		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "item"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}		
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});
