/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Property");
dojo.require("ibm.tivoli.navmodel.Status"); 
dojo.require("dojo.date.stamp");

////////////////////////////////////
// Property
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Property", null, {
	// private members
	_prop: null,
	_dataset: null,
	_objectType: null,
	_value: null,
	_task: null,

	/*=====
	constructor_kwArgs: function(property, dataset) {
		// property: Object
		//		json property object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// objectType: String
		//		objectType of the owning object "item", "link", or "relationship"
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Property object.
		this._dataset = kwArgs.dataset;
		this._objectType = (kwArgs.objectType ? kwArgs.objectType : "item");
		this._prop = kwArgs.property;
		
		// optimization of json on the server can leave out:
		// "displayValue"
		if (!this._prop.displayValue) {
			this._prop.displayValue = this._prop.value;
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	 
	clone: function() {		
		// summary:
		//		Clones the property object and returns a new one with the same values
		var args = {
			property: dojo.clone(this._prop),
			dataset: this._dataset,
			objectType: this._objectType
		};		
		
		return new ibm.tivoli.navmodel.Property(args);
	},

	mixin: function(property) {		
		// summary:
		//		Mixin contents of property with ours
		if (property !== this && property.getId() === this.getId()) {
			this._prop = property.getAttributes();
			
			// re-cache values on next request
			this._value = null;
			this._task = null;		
		}
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._prop[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._prop[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the property
		return dojo.clone(this._prop);		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the property
		dojo.mixin(this._prop, attr);
		
		// re-cache values on next request
		this._value = null;
		this._task = null;
	},

	getId: function() {		
		// summary:
		//		Returns the property id
		return this._prop.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the property id
		this._prop.id = id;
	},

	getLabel: function() {		
		// summary:
		//		Returns the property label
		return this._prop.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the property label
		this._prop.label = label;
	},
	
	getTooltip: function() {
		// summary:
		//		Returns the property tooltip
		return this._prop.tooltip;
	},
	
	setTooltip: function(tooltip) {
		// summary:
		//		Set the property tooltip
		this._prop.tooltip = tooltip;
	},

	getDisplayValue: function() {		
		// summary:
		//		Returns the globalized display value for the property
		return this._prop.displayValue;
	},

	setDisplayValue: function(displayValue) {		
		// summary:
		//		Set the globalized display value for the property
		this._prop.displayValue = displayValue;
	},

	getValue: function() {		
		// summary:
		//		Returns the actual value object for the property
		if (!this._value && this._prop.value !== null && this._prop.value !== undefined) {
			var valueObj = this._prop.value;
			
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time": 
					if (typeof valueObj === "number") {
						this._value = new Date(valueObj);
					}
					else if (typeof valueObj === "string") {
						this._value = dojo.date.stamp.fromISOString(valueObj);										
					}
					break;
				
				case "date[]":
				case "datetime[]":
				case "time[]":
					var dates = [];
					var length = valueObj.length;
					for (var index = 0;  index < length;  ++index) {
						if (typeof valueObj[index] === "number") {
							dates[index] = new Date(valueObj[index]);
						}
						else if (typeof valueObj[index] === "string") {
							dates[index] = dojo.date.stamp.fromISOString(valueObj[index]);						
						}										
					}
					this._value = dates;
					break;
				
				case "status":
					// full status (i.e. was not optimize)
					if (valueObj.label && valueObj.image) {
						this._value = new ibm.tivoli.navmodel.Status({status: valueObj, version: this._dataset.getVersion()});
					}
					// optimized status; look up with dataset
					else {
						var statusArgs = {
							type: this._objectType,
							statusId: valueObj.id,
							statusValue: valueObj.value				
						};
						this._value = this._dataset.getStatus(statusArgs);
					}
					break;
					
				default:
					this._value = valueObj;
			}
		}
		
		return this._value;
	},
	
	/*=====
	getStatusValue_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		WidgetDefinition to use for the merge		
	},
	=====*/
	
	getStatusValue: function(/*getStatusValue_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the "status" value object for the property but
		//		also merges in information from widget definition
		var args = {
			widgetDef: null,
			objectType: this._objectType
		};
		dojo.mixin(args, kwArgs);
		
		var status = null;
		if (this.getValueType() === "status") {
			status = this.getValue();
			if (status && args.widgetDef) {
				status = status.clone(args.widgetDef, args.objectType);
			}
		}
		
		return status;
	},

	setValue: function(value) {		
		// summary:
		//		Modifies the value object for the property
		this._value = value;
				
		switch (this.getValueType()) {
			case "date":
			case "datetime":
			case "time": 
				this._prop.value = value.getTime();
				break;
				
			case "date[]":
			case "datetime[]":
			case "time[]":
				this._prop.value = [];
				var length = value.length;
				for (var index = 0;  index < length;  ++index) {
					this._prop.value[index] = value[index].getTime();
				}
				break;
				
			default:
				this._prop.value = value;
		}
	},

	getValueType: function() {		
		// summary:
		//		Returns the type associated with the property value
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType"
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map", "json"
		//		|		"status",
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "percentage", "float[]", "double[]"
		return this._prop.valueType;
	},

	setValueType: function(valueType) {		
		// summary:
		//		Set the type associated with the property value
		this._prop.valueType = valueType;
	},

	getValueState: function() {		
		// summary:
		//		Returns the state assoicated with the property value
		//		|	values:	"ok", "none", "pending", "error"
		return this._prop.valueState ? this._prop.valueState : "ok";
	},

	setValueState: function(valueState) {		
		// summary:
		//		Set the state associated with the property value
		this._prop.valueState = valueState;
	},

	hasMetadata: function() {		
		// summary:
		//		Returns true if the metadata is present in this property object
		return (this._prop.sortWeight !== undefined);
	},

	getValueUnits: function() {		
		// summary:
		//		Returns a units qualifier that can be assoicated with the property value
		//		example:  "seconds", "hours", "mph"
		return this._prop.units;
	},

	setValueUnits: function(units) {		
		// summary:
		//		Set units qualifier
		this._prop.units = units;
	},

	getValueMin: function() {		
		// summary:
		//		Returns minimum value;  for string properties this is minimum length
		return this._getMetadataObj(this._prop.valueMin);
	},

	setValueMin: function(valueMin) {		
		// summary:
		//		Set minimum value
		this._prop.valueMin = valueMin;
	},

	getValueMax: function() {		
		// summary:
		//		Returns maximum value;  for string properties this is maximum length
		return this._getMetadataObj(this._prop.valueMax);
	},

	setValueMax: function(valueMax) {		
		// summary:
		//		Set maximum value
		this._prop.valueMax = valueMax;
	},

	getValueDefault: function() {		
		// summary:
		//		Returns default value for property (i.e. if has no value and 
		//		a set defaults operation is chosen
		return this._getMetadataObj(this._prop.valueDefault);
	},

	setValueDefault: function(valueDefault) {		
		// summary:
		//		Set default value
		this._prop.valueDefault = valueDefault;
	},

	getSortWeight: function() {		
		// summary:
		//		Returns the sort weight for the property;  when sorting a diverse list of properties
		//		the sortWeight can be used to determine sort order.  Properties with equal sortWeight
		//		should be sorted alphabetically by the label.
		return this._prop.sortWeight;
	},

	setSortWeight: function(sortWeight) {		
		// summary:
		//		Set the sort weight
		this._prop.sortWeight = sortWeight;
	},

	isHidden: function() {		
		// summary:
		//		Returns true if the property should be hidden from the user
		return (this._prop.hidden !== undefined ? this._prop.hidden : false);
	},
	
	setHidden: function(hidden) {		
		this._prop.hidden = hidden;
	},

	isEditable: function() {		
		// summary:
		//		Returns true if the property is editable by the user
		return (this._prop.editable !== undefined ? this._prop.editable : true);
	},
	
	setEditable: function(editable) {
		this._prop.editable = editable;
	},

	isRequired: function() {		
		// summary:
		//		Returns true if the property is required (i.e. must have a value if edited)
		return (this._prop.required !== undefined ? this._prop.required : false);
	},
	
	setRequired: function(required) {
		this._prop.required = required;
	},

	isSensitive: function() {		
		// summary:
		//		Returns true if the property is sensitive and should not 
		//		be displayed in the clear
		return (this._prop.sensitive !== undefined ? this._prop.sensitive : false);
	},
	
	setSensitivie: function(sensitive) {
		this._prop.sensitive = sensitive;
	},
	
	getTask: function() {
		if (!this._task && this._prop.task) {
			this._task = new ibm.tivoli.navmodel.Task({task: this._prop.task, dataset: this._dataset});
		}
		
		return this._task;
	},
	
	setTask: function(task) {
		this._task = task;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getMetadataObj: function(obj) {		
		// summary:
		//		Makes necessary conversion on objects for
		//		metadata functions like min,max,default
		if (obj) {
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time":
				case "date[]":
				case "datetime[]":
				case "time[]":
					if (typeof obj === "number") {
						return new Date(obj);
					}
					else if (typeof obj === "string") {
						return dojo.date.stamp.fromISOString(obj);						
					}									
			}		
		}
		
		return obj;
	}
});
