/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ProviderUtils"); 

// includes
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");
dojo.require("ibm.tivoli.navmodel.Provider");
dojo.require("ibm.tivoli.navmodel.Datasource");
dojo.require("ibm.tivoli.navmodel.DatasetInfo");

ibm.tivoli.navmodel.ProviderUtils._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.ProviderUtils");

ibm.tivoli.navmodel.ProviderUtils.createRemoteProvider = function(kwArgs) {
	// summary: 
	//		Create a remote provider definition on the server.
	// baseUrl: String
	//		base url for identifying rest service
	// provider: ibm.tivoli.navmodel.Provider
	//		provider definition to create
	// remoteAttributes: Boolean
	//		Indicates whether a request should be sent to the remote
	//		provider to retrieve additional attributes.
	// version: Float

	var args = {
		baseUrl: null,
		provider: null,
		remoteAttributes: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var exception = null;		
	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider.getId());
	if (args.remoteAttributes) {
		url += "?remoteAttributes=true";
	}
	
	var json = dojo.toJson(args.provider.getAttributes(), false);
	
	dojo.xhrPut({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version),
			'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider"), 	
			'Content-Length': json.length		
		},
		handleAs: "json",
		putData: json,
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs){														
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "createRemoteProvider", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
		})
	});
	
	if (exception) {
		throw exception;
	}					
};


ibm.tivoli.navmodel.ProviderUtils.deleteRemoteProvider = function(kwArgs) {
	// summary: 
	//		Delete a remote provider definition on the server.
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// version: Float

	var args = {
		baseUrl: null,
		provider: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var exception = null;		
	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	
	dojo.xhrDelete({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version)	
		},
		handleAs: "json",
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs){														
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "deleteRemoteProvider", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
		})
	});
	
	if (exception) {
		throw exception;
	}					
};


ibm.tivoli.navmodel.ProviderUtils.getProviders = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.Provider instances
	// onComplete: function(providers, providerCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// provider: ibm.tivoli.navmodel.Provider
	//		Optional: if specified will be used as passthru connection information
	//		to fetch providers from remote CURI.  If NOT specified; the providers from
	//		the local CURI will be returned
	// baseUrl: String
	//		base url for identifying rest service
	// start: int
	//		start index to fetch
	// count: int
	//		number of providers to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(providers, providerCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		provider: null,
		baseUrl: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers";
	
	// returning providers from local CURI
	if (!args.provider) {
		var content = ibm.tivoli.navmodel.Utils.getContent();
		if (args.start) {
			content.start = args.start;
		}
		if (args.count) {
			content.count = args.count;
		}
		if (args.sort) {
			content.sort = args.sort;
		}
		if (args.condition) {
			content.condition = args.condition;
		}
	
		return dojo.xhrGet({
			url: url,
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "providerList")			
			},
			handleAs: "json",
			sync: args.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var providers = [];			
				if (result.items) {
					var length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						providers[index] = new ibm.tivoli.navmodel.Provider({provider: result.items[index], baseUrl: args.baseUrl, version: args.version});
					}
				}
					
				args.onComplete(providers, result.filteredRows, args);										
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProviders", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
				args.onError(error, exception, ioArgs);
			})
		});
	}
	
	// returning providers from remote CURI
	else {
		var qm = false;
		if (args.start) {
			url += (qm ? "&start=" : "?start=") + args.start;
			qm = true;
		}
		if (args.count) {
			url += (qm ? "&count=" : "?count=") + args.count;
			qm = true;			
		}
		if (args.sort) {
			url += (qm ? "&sort=" : "?sort=") + args.sort;
			qm = true;
		}
		
		if (args.condition) {
			url += (qm ? "&condition=" : "?condition=") + args.condition;			
		}
		
		var json = dojo.toJson(args.provider.getAttributes(), false);
			
		return dojo.xhrPost({
			url: url,
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "providerList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider"), 	
				'Content-Length': json.length		
			},
			handleAs: "json",
			postData: json,
			sync: args.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				var providers = [];			
				if (result.items) {
					var length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						providers[index] = new ibm.tivoli.navmodel.Provider({provider: result.items[index], baseUrl: args.baseUrl, version: args.version});
					}
				}
					
				args.onComplete(providers, result.filteredRows, args);										
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProviders", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
				args.onError(error, exception, ioArgs);
			})
		});
	}	
};

ibm.tivoli.navmodel.ProviderUtils.getProvider = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.Provider matching the given provider id.
	// onComplete: function(provider, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// sync: Boolean
	//		indicates if fetch is synchronous	
	// version: Float

	var args = {
		onComplete: function(provider, kwArgs){
		},
		onError: function(error, exception, ioArgs){
		},
		baseUrl: null,
		provider: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	var content = ibm.tivoli.navmodel.Utils.getContent();

	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){			
			var provider = new ibm.tivoli.navmodel.Provider({provider: result});
			args.onComplete(provider, args);											
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProvider", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
			args.onError(error, exception, ioArgs);
		})
	});
};

ibm.tivoli.navmodel.ProviderUtils.getProviderById = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.Provider matching the given provider id.
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id
	// version: Float

	var args = {
		baseUrl: null,
		provider: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var provider = null;
	var exception = null;		
	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	var content = ibm.tivoli.navmodel.Utils.getContent();

	dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "provider")			
		},
		handleAs: "json",
		sync: true,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){			
			provider = new ibm.tivoli.navmodel.Provider({provider: result});											
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getProviderById", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
		})
	});
	
	if (exception) {
		throw exception;
	}				

	return provider;
};


ibm.tivoli.navmodel.ProviderUtils.getDatasources = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.Datasource instances for a provider
	// onComplete: function(datasources, datasourceCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id 
	// start: int
	//		start index to fetch
	// count: int
	//		number of datasources to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(datasources, datasourceCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		provider: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider) + "/datasources";
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasourceList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var datasources = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					datasources[index] = new ibm.tivoli.navmodel.Datasource({datasource: result.items[index], baseUrl: args.baseUrl, version: args.version});
				}
			}
					
			args.onComplete(datasources, result.filteredRows, args);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getDatasources", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
			args.onError(error, exception, ioArgs);
		})
	});	
};


ibm.tivoli.navmodel.ProviderUtils.getDatasets = function(kwArgs) {
	// summary: 
	//		Returns the ibm.tivoli.navmodel.DatasetInfo instances for a provider/datasource
	// onComplete: function(datasets, datasetCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// provider: String
	//		provider id 
	// datasource: String
	//		datasourceId or null 
	// start: int
	//		start index to fetch
	// count: int
	//		number of datasources to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(datasets, datasetCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		provider: null,
		datasource: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
	url += "/datasets";
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "datasetList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var datasets = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					datasets[index] = new ibm.tivoli.navmodel.DatasetInfo({dataset: result.items[index], version: args.version});
				}
			}
					
			args.onComplete(datasets, result.filteredRows, args);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getDatasets", ibm.tivoli.navmodel.ProviderUtils._trace, args.version);
			args.onError(error, exception, ioArgs);
		})
	});	
};
