/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * Restricted Materials of IBM
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDefUtils"); 

// includes
dojo.require("ibm.tivoli.navmodel.WidgetTypeInfo");
dojo.require("ibm.tivoli.navmodel.WidgetDefInfo");
dojo.require("ibm.tivoli.navmodel.WidgetGroupInfo");
dojo.require("ibm.tivoli.navmodel.Utils");
dojo.require("ibm.tivoli.navmodel.Trace");

ibm.tivoli.navmodel.WidgetDefUtils._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.WidgetDefUtils");

ibm.tivoli.navmodel.WidgetDefUtils.createWidgetDef = function(/*createWidgetDef_kwArgs*/ kwArgs) {
	// summary: 
	//		Create a widget definition in the server database and then construct the
	//		requested widget definition object (i.e. sub-class of AbstractWidgetDef or WidgetDef)
	// widgetClass: String
	//		widget definition class to construct (ex: ibm.tivoli.navmodel.TopoWidgetDef)
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id
	// datasource: String
	//		datasource id
	// widgetType: String
	//		widget type in form <type>-<version>
	// widgetId: String
	//		widget definition id in form <id>-<version>
	// definitionObj: Object
	//		javascript object representing the widget definition contents
	// overwrite: Boolean
	//		indicates if widget definition should be overwritten if it already exists
	// profileID: String
	//		optional unique id to associate with reading and writing the
	//		"user" and "shared" deltas to the widget definition.  If not
	//		supplied; a uniqueId is created consisting of all the related
	//		information (i.e. providerType, datasourceType, widgetType, etc)
	// parentWidgetDef: ibm.tivoli.navmodel.AbstractWidgetDef  (or descendent)
	//		optional parent widget definition
	// version: Float

	var args = {
		widgetClass: null,
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		widgetType: null,
		widgetId: null,
		definitionObj: null,
		overwrite: false,
		profileID: null,
		parentWidgetDef: null,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	// url needed to create the widget definition
	var url = args.baseUrl;
	url += "/providers/" + encodeURIComponent(args.provider);
	url += args.datasource ? ("/datasources/" + encodeURIComponent(args.datasource)) : "";
	url += "/widgets/"+ args.widgetType; 
	url += "/definitions/" + args.widgetId;
	url += "?type=base";
	if (args.overwrite) {
		url += "&overwrite=true";
	}
	if (args.registryLocation && args.registryLocation === "local") {
		url += "&registryLocation=local";
	}

	var exception = null;		
	var json = dojo.toJson(args.definitionObj, false);

	dojo.xhrPut({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version),
			'Content-Type': 'application/json',
			'Content-Length': json.length
		},
		handleAs: "json",
		putData: json,
		sync: true,
		load: dojo.hitch(this, function(result, ioArgs) {									
		}),
		error: dojo.hitch (this, function(error, ioArgs){
			exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "createWidgetDef", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);				
		})
	});

	if (exception) {
		throw exception;
	}

	var widget = null;
	var objectClass = dojo.getObject(args.widgetClass);
	if (objectClass) {
		var widgetArgs = {
			baseUrl: args.baseUrl,
			provider: args.provider,
			datasource: args.datasource,
			widget: args.widgetType,
			definition: args.widgetId,
			type: "user",
			profileID: args.profileID,
			parentWidgetDef: args.parentWidgetDef,
			version: args.version
		};

		try {
			widget = new objectClass(widgetArgs);
		}
		catch(ex) {
			ibm.tivoli.navmodel.WidgetDefUtils._trace.error("Exception constructing widget definition: ex=" + ex, "createWidgetDef");					
		}											
	}
	else {
		ibm.tivoli.navmodel.WidgetDefUtils._trace.error("Cannot instantiate widget def class=" + args.widgetClass, "createWidgetDef");				
	}

	return widget;
};

ibm.tivoli.navmodel.WidgetDefUtils.getWidgetTypes = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.WidgetTypeInfo instances
	// onComplete: function(widgetTypes, widgetTypeCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id (can also be null or key words "any" and "all")
	// datasource: String
	//		datasource id (can also be null or key words "any" and "all")
	// start: int
	//		start index to fetch
	// count: int
	//		number of definitions to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(widgetTypes, widgetTypeCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = null;	
	if (args.provider) {
		url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
		url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
		url += "/widgets";
	}
	else {
		url = args.baseUrl + "/widgets";
	}
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.registryLocation && args.registryLocation === "local") {
		content.registryLocation = args.registryLocation;
	}
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "widgetTypeList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var widgetTypes = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					widgetTypes[index] = new ibm.tivoli.navmodel.WidgetTypeInfo({widgetType: result.items[index], version: args.version});
				}
			}
					
			kwArgs.onComplete(widgetTypes, result.filteredRows, kwArgs);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getWidgetTypes", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);
			kwArgs.onError(error, exception, ioArgs);
		})
	});	
};


ibm.tivoli.navmodel.WidgetDefUtils.getWidgetDefinitions = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.WidgetDefInfo instances
	// onComplete: function(widgetDefs, widgetDefCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id (can also use key words "any" and "all")
	// datasource: String
	//		datasource id (can also use key words "any" and "all")
	// widgetType: String
	//		widget type (i.e. topology-0.1.0;  can also use key word "all")
	// groupIds: String[]
	//		array of groupIds used to filter the returned definitions
	// start: int
	//		start index to fetch
	// count: int
	//		number of definitions to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(widgetDefs, widgetDefCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		widgetType: null,
		groupIds: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
	url += "/widgets/" + args.widgetType;
	url += "/definitions";
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.registryLocation && args.registryLocation === "local") {
		content.registryLocation = args.registryLocation;
	}
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	if (args.groupIds) {
		content.groupIds = args.groupIds.join(",");
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "widgetDefList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var widgetDefs = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					widgetDefs[index] = new ibm.tivoli.navmodel.WidgetDefInfo({widgetDef: result.items[index], version: args.version});
				}
			}
					
			kwArgs.onComplete(widgetDefs, result.filteredRows, kwArgs);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getWidgetDefinitions", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);
			kwArgs.onError(error, exception, ioArgs);
		})
	});	
};


ibm.tivoli.navmodel.WidgetDefUtils.getWidgetGroups = function(kwArgs) {
	// summary: 
	//		Returns list of ibm.tivoli.navmodel.WidgetGroupInfo instances
	// onComplete: function(widgetGroups, widgetGroupCount, kwArgs)
	//		callback for successful fetch
	// onError: function(error, exception, ioArgs)
	//		callback for error conditions
	// baseUrl: String
	//		base url for identifying rest service
	// registryLocation: String
	//		(optional) Indicates if "remote" provider definition is in local registry or is
	//		located at the remote rest service.  
	//		values are: "local" or "remote"  (Default for TIP is "local")
	// provider: String
	//		provider id (can also be key words "any" and "all")
	// datasource: String
	//		datasource id (can also be key words "any" and "all")
	// start: int
	//		start index to fetch
	// count: int
	//		number of groups to fetch; null means all
	// sort: String
	//		comma deliminated attribute string (preceded with optional +/- indicating ascending/descending)
	//		example:  label,type
	// condition: String
	//		condition expression string 
	//		example: description contains 'abcd'
	// sync: Boolean
	//		indicates if fetch is synchronous
	// version: Float

	var args = {
		onComplete: function(widgetGroups, widgetGroupCount, kwArgs){
		},
		onError: function(error, exception, ioArgs){				
		},
		baseUrl: null,
		registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
		provider: null,
		datasource: null,
		start: 0,
		count: null,
		sort: null,
		condition: null,
		sync: false,
		version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
	};
	dojo.mixin(args, kwArgs);

	var url = args.baseUrl + "/providers/" + encodeURIComponent(args.provider);
	url += (args.datasource ? "/datasources/" + encodeURIComponent(args.datasource) : "");
	url += "/widgets/groups";	
	
	var content = ibm.tivoli.navmodel.Utils.getContent();
	if (args.registryLocation && args.registryLocation === "local") {
		content.registryLocation = args.registryLocation;
	}
	if (args.start) {
		content.start = args.start;
	}
	if (args.count) {
		content.count = args.count;
	}
	if (args.sort) {
		content.sort = args.sort;
	}
	if (args.condition) {
		content.condition = args.condition;
	}
	
	return dojo.xhrGet({
		url: url,
		headers: {
			'Accept': ibm.tivoli.navmodel.Utils.getContentType(args.version, "widgetGroupList")			
		},
		handleAs: "json",
		sync: args.sync,
		content: content,
		load: dojo.hitch(this, function(result, ioArgs){
			var widgetGroups = [];			
			if (result.items) {
				var length = result.items.length;
				for (index = 0;  index < length;  ++index) {
					widgetGroups[index] = new ibm.tivoli.navmodel.WidgetGroupInfo({widgetGroup: result.items[index], version: args.version});
				}
			}
					
			kwArgs.onComplete(widgetGroups, result.filteredRows, kwArgs);										
		}),
		error: dojo.hitch(this, function(error, ioArgs){
			var exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getWidgetGroups", ibm.tivoli.navmodel.WidgetDefUtils._trace, args.version);
			kwArgs.onError(error, exception, ioArgs);
		})
	});	
};
