/**
 * ibm.tivoli.tip.dijit.ListBuilder is a pair of ibm.tivoli.tip.dijit.DndListboxes, with Add and Remove buttons between them
 * 
 * Creating a ListBuilder in markup:
 * <div dojoType="ibm.tivoli.tip.dijit.ListBuilder" showAllButtons="true" filtered="true" sorted="false" accept="role" 
 * 		type="role,text" name="myroles" leftLabel="Available roles" rightLabel="Selected roles"></div>
 * 
 * Attributes:
 * shoAllButtons: if true, show the "Add All" and "Remove All" buttons
 * sorted: if true, the right-hand list is kept sorted. the left-hand list is always sorted
 * filtered: if true, the lists include the filter textbox
 * accept: array of types that can be dropped on the 2 DndListboxes
 * type: array of types describing the items in the 2 DndListboxes
 * leftLabel: text displayed above the left-hand list
 * rightLabel: text displayed above the right-hand list
 * 
 * Events:
 * 	fires OnChange when the value of the list changes. (TODO: implement priorityChange to only fire when list looses focus)
 * 
 */
dojo.provide("ibm.tivoli.tip.dijit.ListBuilder");

dojo.require("dijit.form.Button");
dojo.require("ibm.tivoli.tip.dijit.DndListbox");
dojo.require("dojo.i18n");
dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources");

dojo.declare("ibm.tivoli.tip.dijit.ListBuilder", [dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templateString: dojo.cache("ibm.tivoli.tip.dijit", "templates/ListBuilder.html"),
	showAllButtons: true,	// do we show the Add All and Remove All buttons
	filtered: true,			// do we show the filter input text boxes
	sorted: false,			// is the right list kept sorted? (the left list is always sorted)
	accept: ["text"],		// array of types that can be dropped here
	type: ["text"],			// array of types objects in this box represent
	name: "",				// give the widget a name if you want its value submitted with a form
	leftLabel: "",			// label for the left list
	rightLabel: "",			// label for the right list
	required: false,		// is a value required for this list?
	// nls strings
	addLabel: "",
	removeLabel: "",
	addAllLabel: "",
	removeAllLabel: "",
	acceptBuffer:null,
	showReOrderButtons: false,	//if sorted, hide the "move up", "move down" buttons rather than just disable
	disabled:false,
	isWidthSet:false,

	postMixInProperties: function()
	{
		var string_table = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
		this._resources = string_table;
		this.addLabel = string_table.ADD;
		this.removeLabel = string_table.REMOVE;
		this.addAllLabel = string_table.ADDALL;
		this.removeAllLabel = string_table.REMOVEALL;
		this.inherited(arguments);
	},
	/**
	 * startup()
	 *
	 * Basic start-up rountine, connect event handlers and 
	 * add the first items to the list.
	 */
	startup: function() {
		if(!this.showAllButtons) {
			dojo.style(this._balAddAllBtn.domNode, "display", "none");
			dojo.style(this._balRemAllBtn.domNode, "display", "none");
		}

		// Connect click listeners to the buttons
		this.connect(this._balAddBtn.domNode, 'onclick', this.add);
		this.connect(this._balAddAllBtn.domNode, 'onclick', this.addAll);
		this.connect(this._balRemBtn.domNode, 'onclick', this.remove);
		this.connect(this._balRemAllBtn.domNode, 'onclick', this.removeAll);
		//dojo.subscribe("/dnd/drop", dojo.hitch(this, this.updateButtons));
		this.connect(this.domNode, 'onclick', this.updateButtons);
		
		this.connect(this.left, "OnChange", this.updateButtons);
		this.connect(this.right, "OnChange", this._onChange);
		
		this.updateButtons();
		this.inherited(arguments);
	},	
	destroy: function()
	{
		console.log("ListBuilder.destroy");
		this.inherited(arguments);
	},
	_getValueAttr: function()
	{
		var list = this.right.get("value");
		return list;
	},
	_setValueAttr: function(value)
	{
		var item;
		var items = [];
		var varr = dojo.isArray(value) ? value : [value];
		var from = this.left;
		var to = this.right;

		// move non-matching items from the right list back to the left
		to.getAllNodes().forEach(function(n){
			item = to.getItem(n.id);
			if(item && item.data && dojo.indexOf(varr, item.data.toString()) < 0) {
				items.push(item.data);
				to.delItem(n.id);
			}
		});
		from.insertNodes(items);
		
		// move matching items from the left list to the right
		items = [];
		from.getAllNodes().forEach(function(n){
			item = from.getItem(n.id);
			if(item && item.data && dojo.indexOf(varr, item.data.toString()) >= 0) {
				items.push(item.data);
				from.delItem(n.id);
			}
		});
		this.right.insertNodes(items);

		this.updateButtons();
	},
	
	/**
	* This function provides the ability to enable/disable a ListBuilder.  By default the ListBuilder is enabled.
	* Once user call to disable a listBuilder, all the button needs to be disabled, and Drag n Drop also should
	* be disabled.
	* This function will call DnDListBox.setDisabled(value) which in turn will disable drag n drop.
	*/
	_setDisabledAttr: function(/*Boolean*/ value){
        var i;
		console.log("ListBuilder.js : _setDisabledAttr");
		this.right.setDisabled(value); //target needs to be disabled for DnD
		this.left.setDisabled(value); // Source needs to be disabled for DnD
		if(value){
		this._balAddBtn.set('disabled', value);
		this._balRemBtn.set('disabled', value);
		
		}
		this._balRemAllBtn.set('disabled', value);
		this._balAddAllBtn.set('disabled', value);
		this.disabled=value;
	},
	
	/**
	 * addData()
	 * 
	 * Add data to the available list
	 * 
	 * Input:
	 * 	data: the data being added to the available list
	 * 
	 */
	addData: function(data)
	{
		this.left.addData(data);
		this.updateButtons();
		this._setButtonWidth();
	},
	
	addDataToRight: function(data){
		this.right.addData(data);
		this.updateButtons();
	},
	
	moveUp: function(e){
		var nodes = this.right.getSelectedNodes();
		var anchor = nodes[0].previousSibling;
		var items = [];
		for(var i=0;i<nodes.length;i++){
			items.push(this.right.getItem(nodes[i].id).data);
		}		
		this.right.deleteSelectedNodes();
		this.right._source.insertNodes(true, items, true, anchor);	
		this.updateButtons();
	},
	
	moveDown: function(e){
		var nodes = this.right.getSelectedNodes();	
		var anchor = nodes[nodes.length-1].nextSibling;
		var items = [];
		for(var i=0;i<nodes.length;i++){
			items.push(this.right.getItem(nodes[i].id).data);
		}		
		this.right.deleteSelectedNodes();
		this.right._source.insertNodes(true, items, false, anchor);	
		this.updateButtons();
	},
	
	/**
	 * addAll()
	 *
	 * Add All button handler.  Moves all items from the available list to 
	 * the reserve list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	addAll: function(e) {
		if(!this.disabled){
			this.left.selectAll();
			this._move(this.left, this.right);
			this.updateButtons();
		}
	},

	/**
	 * add()
	 *
	 * Add button handler.  Moves selected items from the left list to 
	 * the right list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	add: function(e) {
	if(!this.disabled){
		this._move(this.left, this.right);
		}
	},

	/**
	 * removeAll()
	 *
	 * Remove All button handler.  Moves all items reserve from the list to 
	 * the available list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	removeAll: function(e) {
	if(!this.disabled){
		 this.right.selectAll();
		 this._move(this.right, this.left);
		 this.updateButtons();
		 }
	},
	/**
	 * remove()
	 *
	 * Remove button handler.  Moves selected items from the right list
	 * to the left list.
	 * 
	 * Inputs:
	 *   e- DOM event object from Dojo (assumed fixed)
	 * Returns:
	 *   None.
	 */
	remove: function(e) {
	if(!this.disabled){
		this._move(this.right, this.left);
		}
	},
	_move: function(from, to)
	{
		var nodes = from.getSelectedNodes();
		// for some reason dojo.dnd can do it, but I can't insert nodes, 
		// but only items.  This also means I loose the items' type in 
		// the process
		var items = [];
		var item;
		nodes.forEach(function(n) {
			item = from.getItem(n.id);
			if(item && item.data)
			items.push(item.data);
		});
		to.insertNodes(items);
		from.deleteSelected();
		this.updateButtons();
	},
	/**
	 * updatesButtons()
	 *
	 * Updates all buttons disabled state based on the content
	 * of the list.  It disables button when the list is empty 
	 * and enable button when their is something to move.
	 * 
	 * Inputs:
	 *   None.
	 * Returns:
	 *   None.
	 */
	updateButtons: function() {

		if(!this.disabled)
		{
		// If any nodes are selected, enable the add button
		var b = this.left.getSelectedNodes().length < 1;
		this._balAddBtn.set('disabled', b);

		// If the available area contains any nodes, enable the add all button
		b = this.left.get("value").length < 1;
		this._balAddAllBtn.set('disabled', b);

		// If the reserve area contains any seelected items, enable the remove button
		b = this.right.getSelectedNodes().length < 1;
		this._balRemBtn.set('disabled', b);

		// If the available area contains any nodes, enable the remove all button
		b = this.right.get("value").length < 1;
		this._balRemAllBtn.set('disabled', b);
		
		var nodeCount = this.right.getSelectedNodes().length;		
		
		this.moveUpBtn.set('disabled',  nodeCount<1 || !this.right.getSelectedNodes()[0].previousSibling);
		this.moveDownBtn.set('disabled', nodeCount<1 || !this.right.getSelectedNodes()[nodeCount-1].nextSibling);
		
		dojo.style(this._reorderButtonContainer, "display", (this.sorted && !this.showReorderButtons) ? "none" : "");
		}
	},
	isValid: function()
	{
		return !this.required || this.get("value").length > 0;
	},
	_onChange: function()
	{
		this.updateButtons();
		this.OnChange();
	},
	OnChange: function()
	{
		// override or connect to me
	},
	
	/**
	*  Earlier the buttons were set to 7em by using CSS classes and for long strings(in different locales), the string was overflowing the span
	*  _setButtonWidth will calculate the width of all 4 strings(Add, AddAll etc) and the buttons would then be set for that width.
	*  This method uses spans from the dijit.form.button. 
	*/
	_setButtonWidth: function()
	{
		if(this.isWidthSet == true)
			return;
		var allButtons = new Array(this._balAddBtn, this._balAddAllBtn, this._balRemBtn, this._balRemAllBtn);
		var width = this._findMaxButtonWidth();
		if(width == 0)
			return;
		
		width = width + "px";
		for(button in allButtons)
		{
			var allSpans = dojo.query("span", allButtons[button].domNode);//.attr(style, {width:10em});
			for(var key in allSpans)
			{
				if(allSpans[key].className != undefined && allSpans[key].className.indexOf("dijitButtonNode") != -1)
				{
					dojo.style(allSpans[key], "width", width);
					break;
				}
			}
		}
		this.isWidthSet = true;
	},
	
	_findMaxButtonWidth : function()
	{	
		var maxWidth = 0;
		var allButtons = new Array(this._balAddBtn, this._balAddAllBtn, this._balRemBtn, this._balRemAllBtn);
		for(var button in allButtons)
		{
			var allSpans = dojo.query("span", allButtons[button].domNode);
			for(var key in allSpans)
			{
				if(allSpans[key].className != undefined && allSpans[key].className.indexOf("dijitButtonText") != -1)
				{
					if(allSpans[key].offsetWidth  > maxWidth)
						maxWidth = allSpans[key].offsetWidth;
						
					break;
				}
			}
		}
		return maxWidth;
	}
});
