/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.analysis.base._AnalysisPaneContainerMixin");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.analysis.AnalysisPane");
dojo.require("com.ibm.security.modeling.dijit.layout.ContentPane");

dojo.declare("com.ibm.security.modeling.analysis.base._AnalysisPaneContainerMixin", null,  {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.base._AnalysisPaneContainerMixin"),
	
	region: "bottom",
	splitter: true,
	
	postCreate:function() {
		// tags:
		//			protected
		this._trace.entering(this.id, "postCreate");	
		this.inherited(arguments);
		this._loadPreferences();
		this._createAnalysisPaneContainer();
		this._subscribeToPanelLaunch(); 
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	_loadPreferences: function() {
		// Method to set 'region', 'splitter', and _splitterClass per the preferences.
		
		// Preferences loading is not supported yet.
		// Hence default values.
		
		this.region = "bottom";
		this.splitter = true;
		this.container._splitterClass = "com.ibm.security.modeling.dijit.layout.ToggleSplitter";
	},
	
	_createAnalysisPaneContainer: function() {
		if (this.analysisPaneContainer) {
			return;
		}
		var containerInitialContents = "<div id='"+this.id+"_analysisPane'></div>";
		this.analysisPaneContainer = new com.ibm.security.modeling.dijit.layout.ContentPane({
			id: this.id+"_analysisPaneConainter",
			region: this.region, 
			splitter: this.splitter,
			"class": "analysisPaneContainer",
			content: containerInitialContents
		});
	},
	
	_subscribeToPanelLaunch: function() {
		// Summary:
		//		Register function to panelChanged event.
		//		- Useful to handle analysis pane show/hide.
		
		this.subscribe("/com/ibm/security/modeling/panelChanged", function() {
			// Extract data from arguments.
			var breadcrumbArgs = null;
			var panelId = null;
			var roleAnalysisData = null;
			if (arguments.length > 2) {
				breadcrumbArgs = arguments[0];
				panelId = arguments[1];
				roleAnalysisData = arguments[2];
			}
			
			this._postLaunchPanel(panelId, roleAnalysisData);
		});
	},
	
	_checkIfAnalysisPaneMaximized: function() {
		if (this._isAnalysisPaneContainerInDOM()) {
			var splitter = this.container.getSplitter(this.analysisPaneContainer.region);
			if(splitter && splitter.state &&  (splitter.state == "maximize") ) {
				return true;
			}
		}
		return false;
	},
	
	_minimizeAnalysisPane: function() {
		if (this._isAnalysisPaneContainerInDOM()) {
			var splitter = this.container.getSplitter(this.analysisPaneContainer.region);
			if( splitter && splitter._minimize && dojo.isFunction(splitter._minimize) ) {
				splitter._minimize();
			}
		}
	},
	
	_postLaunchPanel: function(panelId, roleAnalysisData) {
		// Summary:
		//		Logic to execute when a panel is launched.
		//		- show/hide the analysis 
		//
		// panelId: String
		//		Id representing a panel being launched/switched to.
		//		refer to dojo.config.panels
		// roleAnalysisData.modelId: String
		//		Address of the currently selected model.
		// roleAnalysisData.roleUrl.String
		// 		Address of the currently selected role.
		// roleAnalysisData.projectUrl.String
		// 		Address of the currently selected project.
		// tags:
		//		protected
		
		// Check if the analysis pane is maximized. If yes, first minimize it and then do whatever processing in terms of breadcrumb navigation.
		
		if (this._checkIfAnalysisPaneMaximized()) {
			this._minimizeAnalysisPane();
		}
		
		if (this._isAnalysisPanelAttached(panelId)) {
			// Retrieve current context data.
			var contextData = this._getAnalysisPanelContextData(roleAnalysisData);
			// Add analysis panel to GUI and populate it per current context data.
			this.addAnalysisPanel(contextData);
		} else {
			// Analysis panel is not associated with current panel.
			// Remove Analysis panel from GUI.
			this.removeAnalysisPanel();
		}
	},
	
	_showRoleAnalysis: function(/*Object*/ roleAnalysisData) {
		// Summary:
		//		If analysis pane is attached to DOM then, it shows the analysis for the role.
		//		Role information is provided in 'roleAnalysisData' argument.
		//
		// roleAnalysisData.modelId: String
		//		Address of the currently selected model.
		// roleAnalysisData.roleUrl.String
		// 		Address of the currently selected role.
		// roleAnalysisData.projectUrl.String
		// 		Address of the currently selected project.
		// tags:
		//		protected
		
		if (this._isAnalysisPaneContainerInDOM()) {
			// Open analysis pane in split mode. 
			var splitter = this.container.getSplitter(this.analysisPaneContainer.region);
			if (splitter && splitter._split && dojo.isFunction(splitter._split)) {
				splitter._split();
			}
			// Display role analysis details.
			var contextData = this._getAnalysisPanelContextData(roleAnalysisData);
			this._analysisPane.show(contextData, /*roleAnalysis*/ true);
		}
	},
	
	_getAnalysisPanelContextData: function(roleAnalysisData) {
		// Summary:
		//		Extracts data from 'breadcrumbArgs' per the 'panelId'
		//		Extracted data is considered as context data for the 
		//		analysis panel.
		// roleAnalysisData.modelId: String
		//		Address of the currently selected model.
		// roleAnalysisData.roleUrl.String
		// 		Address of the currently selected role.
		// roleAnalysisData.projectUrl.String
		// 		Address of the currently selected project.
		//
		// tags:
		//		private
		var contextData = {};
		
		var projectUrl = null;
		var modelId = null;
		var roleUri = null;
		
		if (roleAnalysisData) {
			//projectUrl, roleUrl, and modelId
			projectUrl = roleAnalysisData["projectUrl"];
			modelId = roleAnalysisData["modelId"];
			roleUri = roleAnalysisData["roleUrl"];
		}
		
		if (projectUrl) {
			contextData["projectUrl"] = projectUrl;
		}
		if (modelId) {
			contextData["modelId"] = modelId;
		}
		if (roleUri) {
			contextData["roleUri"] = roleUri;
		}
		
		return contextData;
	},
	
	_isAnalysisPanelAttached: function(panelId) {
		// Summary:
		//		Checks analysis panel is associated with the specified launcher.
		// 
		// panelId: 	String
		//		Id of the panel which has just been launched.
		//		refer dojo.config.panels
		//
		// returns: 	boolean
		//		true:	if launcher is instance of any of the following:
		//				-	dojo.config.panels.HIERARCHY.id
		//				- 	dojo.config.panels.ROLE.id
		//				- 	dojo.config.panels.FILTER.id
		//		false:	otherwise.
		// tags:
		//		protected
		
		if (panelId && 
				(panelId == dojo.config.panels.HIERARCHY.id
				|| panelId == dojo.config.panels.ROLE.id
						||panelId == dojo.config.panels.FILTER.id)) {
			return true;
		} 
		
		return false;
	},
	
	addAnalysisPanel: function(/*Object*/ contextData) {
		// summary:
		//		Displays the analysis panel.
		// tags:
		//		public
		// contextData: object
		//		Analysis panel's context data.
		
		this._trace.entering(this.id, "addAnalysisPanel");
		
		if (!this._isAnalysisPaneContainerInDOM()) {
			// Add analysis pane container to DOM
			this.container.addChild(this.analysisPaneContainer);
			this.analysisPaneContainer.domNode.setAttribute("region", this.region);
			this._establishResizeConnection();
		}
		
		if(this._analysisPane) { 
			this._analysisPane.show(contextData);
		} else {
			this._createAnalysisPane(contextData);
		}
		
		this._trace.exiting(this.id, "addAnalysisPanel");
	},
	
	removeAnalysisPanel: function() {
		// summary:
		//		Removes the analysis panel from DOM.
		//		Does not destroy the panel instance.
		// tags:
		//		public
		
		this._trace.entering(this.id, "removeAnalysisPanel");
		if (this._isAnalysisPaneContainerInDOM()) {
			 
			if (this._analysisPane) {
				this._analysisPane.resetPane();
				this._analysisPane.onHide();
			}
			this._disconnectResizeConnection();
			this.container.removeChild(this.analysisPaneContainer);
		}
		this._trace.exiting(this.id, "removeAnalysisPanel");
	},
	
	_createAnalysisPane: function(contextData) {
		// Summary:
		//		Creates a ContentPane and sets Analysis Pane as content of it.
		this._analysisPane = this.panelFactory.createAnalysisPanel(contextData);
		
		this.connect(this.analysisPaneContainer, "onShow", function() {
			if (this._analysisPane) {
				this._analysisPane.onShow();
			}
		});
		this.connect(this.analysisPaneContainer, "onHide", function() {
			if (this._analysisPane) {
				this._analysisPane.onHide();
			}
		});
		
		this._analysisPane.startup();
	},
	
	_isAnalysisPaneContainerInDOM: function() {
		// Summary:
		//		Checks if analysis pane container [this.analysisPaneContainer] is already attached to DOM.
		if (this.container 
				&& this.analysisPaneContainer 
				&& this.container.getIndexOfChild(this.analysisPaneContainer) != -1) {
			return true;
		}
		return false;
	},
	
	_establishResizeConnection: function() {
		if (this._isAnalysisPaneContainerInDOM()) {
			if (!this._analysisPaneResizeConnections) {
				this._analysisPaneResizeConnection = dojo.connect(this.container, "resize", this, this._onAnalysisPaneContainerResize);
			}
		}
	},
	
	_disconnectResizeConnection: function() {
		if (this._analysisPaneResizeConnection) {
			dojo.disconnect(this._analysisPaneResizeConnection);
			delete this._analysisPaneResizeConnection;
			this._analysisPaneResizeConnection = null;
		}
	},
	
	_onAnalysisPaneContainerResize: function() {
		if (this._isAnalysisPaneContainerInDOM()) {
			var splitter = this.container.getSplitter(this.analysisPaneContainer.region);
			if (splitter && splitter.layout && dojo.isFunction(splitter.layout)) {
				splitter.layout();
			}
		}
	},
	
	destroy: function() {
		
		this._disconnectResizeConnection();
		
		this.inherited(arguments);
	}

});
