/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.analysis.base._AnalysisTaskPanel");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("com.ibm.security.modeling.analysis.base._AnalysisPaneUtil");

dojo.require("com.ibm.security.modeling.widgets.ProgressBar");
dojo.requireLocalization("com.ibm.security.modeling.analysis", "AnalysisPaneText");

/**
 * A type that all analysis task panels must extend. 
 */
dojo.declare("com.ibm.security.modeling.analysis.base._AnalysisTaskPanel", null,  {
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.analysis.base._AnalysisTaskPanel"),
	
	postReset: false,
	needsRefresh: false,
	constructor: function() {
		// title: string
		//		Title to display when this panel is on display.
		// helpFile: string
		//		Help file name.
		// factory: _AnalysisTaskPanelFactory
		//		Factory which created this panel instance.
		// contextData: Object
		//		 Data needed to display this panel.
	
		var args = {};
		dojo.mixin(args, arguments[0]);
		
		this.setAnalysisTaskData(args.analysisTaskData);
	},
	
	getTitle: function() {
		// Summary:
		//		Gets the translated title of the panel.  
		//		The Analysis pane will use this method 
		//		to updated the title on the Analysis pane when the panel is displayed.
	
		// returns: string
		//		title
		return this.title;
	},
	
	getInheritanceLabel: function() {
		// Summary:
		//		Gets the label to be shown against the inheritance check-box of each task. 
		//
		// returns: string
		//		inheritanceLabel
		return this.inheritanceLabel;
	},	
	
	getHelpFile: function() {
		// Summary:
		//		Gets the help file name for the panel.
	
		// returns: string
		//		help file name
		return this.helpFile;
	},
	
	
	getFactory: function() {
		// Summary:
		//		Returns the factory used to create this panel.  
		//		(This will used in the future to populate 'Recent Views' stack). 
		//
		// returns: _AnalysisTaskPanelFactory
		//		Factory which created this panel instance.
		
		return this.factory;
	},
	
	getContextData: function() {
		// Summary:
		//		Returns the context data needed to display this panel.
		//		For example role uri,  selected user attribute etc..
		//		(This will used in the future to populate 'Recent Views' stack).
		//
		// returns:	Object
		//		Context data needed to display this panel
		
		return this.contextData;
	},
	
	setContextData: function(/*Object*/ contextData) {
		// Summary:
		//		This method is used to set new data in the task panel.  
		//		For example if the panel is current displayed 
		//		and a new role notebook is opened.  
		//		The new role uri would be passed in.
		//
		//		This panel should update / reload as necessary.
		// contextData: Object
		//		Context data needed to display this panel
		
		this.contextData = contextData;
	},
	
	getSupplementData: function() {
		return this.supplementData;
	},
	
	setSupplementData: function(/*Object*/ supplementData) {
		if (!supplementData) {
			supplementData = {};
		}
		
		this.supplementData = supplementData;
	},
	
	getAnalysisTaskData: function() {
		var data = {};
		var contextData = this.getContextData();
		var supplementData = this.getSupplementData();
		if (supplementData) {
			dojo.mixin(data, supplementData);
		}
		if (contextData) {
			dojo.mixin(data, contextData);
		}
		return data;
	},
	
	setAnalysisTaskData: function(/*Object*/ data) {
		var contextData = com.ibm.security.modeling.analysis.base._AnalysisPaneUtil.extractContextData(data);
		this.setContextData(contextData);
		
		var supplementData = com.ibm.security.modeling.analysis.base._AnalysisPaneUtil.extractSupplementData(data);
		this.setSupplementData(supplementData);
	},
	
	refresh: function() {
		// Summary:
		//		Refreshes the analysis panel.  
	},
	
	reset: function() {
		// Summary:
		//		Resets the task panel to its initial state.
		//		Initial state represents the internal data and DOM state when there was no context data.
		//		This method must be overridden to reset DOM to its initial state.
		
	},
	
	onLoadingStart: function() {
		// Summary:
		//		Performs logic which is needed when panel is loading data.
		
		this._showProgressBar();
	},
	
	onLoadingComplete: function() {
		// Summary:
		//		Performs logic which is needed when panel has finished loading data.
		this._hideProgressBar();
	},
	
	_showProgressBar: function() {
		// Summary:
		//		Creates [if not created earlier] progress bar and shows that.
		if (!this._progressBar) {
			this._progressBar = new com.ibm.security.modeling.widgets.ProgressBar({
				id: this.id+"_progressBar"
			});
		}
		
		var nlsResource = this.__getAnalysisResources();
		this._progressBar.show(nlsResource.LOADING_ANALYSIS_MESSAGE);
	},	
	
	_hideProgressBar: function() {
		// Summary:
		//		Hides the progress bar.
		if (this._progressBar) {
			this._progressBar.hide();
			
			/* 
			 * Sometimes, hide doesn't seem to destroy the dialog. 
			 * Hence, destroying dialog explicitly. 
			 */
			this._progressBar.destroyProgressDialog(); 
		}
	},
	
	__getAnalysisResources: function() {
		if (!this.__analysisResources) {
			this.__analysisResources = dojo.i18n.getLocalization("com.ibm.security.modeling.analysis", "AnalysisPaneText");
		}
		return this.__analysisResources;
	},
	
	onSupplementDataChange: function() {
	
	}
	
});
