/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.api.RestManager");

dojo.require("com.ibm.security.modeling.util.ErrorUtilities");
dojo.require("com.ibm.security.modeling.util.Trace");

dojo.declare("com.ibm.security.modeling.api.RestManager", null, {
	
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.api.RestManager"),
	//	_errorUtil:	com.ibm.security.modeling.util.ErrorUtilities
	//		Provides utilities for handling errors.
	_errorUtil: null,
	// resourceUri: string
	//		uri for REST resource.
	resourceUri: null, 
	// syncMode: boolean
	//		true:	specifies that REST calls will be synchronous 
	//		false:	specifies that REST calls will be asynchronous and call-back handlers are required for processing result.
	syncMode: false,
	// messageArea: ibm.tivoli.tip.dijit.TIPMessageArea
	//		This is to display messages on GUI when any error occurs.
	messageArea: null,
	
	//	_resourceSchema:	object
	//		Object representing JSON schema of resource type like: Role, User 
	_resourceSchema: null,
	
	// _identifier:	string
	//		String representing identifier property in JSON representation of resource type
	idAttribute: "id",
	
	// _ResourceMO:	object
	//		An object representing resource type (class). It is obtained from JSON rest store.
	//		To create a new resource item:
	//			'new' operator can be applied to it.
	//			And newly created resource item is known to JSON rest store. 
	//			hence calling save on JSON rest store, results into HTTP POST call for this resource item.
	_ResourceMO: null,
	
	// _resourceJsonRestStore:	dojox.data.JsonRestStore
	//		An instance of dojox.data.JsonRestStore.
	_resourceJsonRestStore: null,
	
	// _resourceRestService: dojox.rpc.Rest	
	//		An instance of dojox.rpc.Rest.
	_resourceRestService: null,
	allowNoTrailingSlash:false,
	
	
	
	constructor: function(args) {
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		this.ignoreMatchQuery=null;
		this._errorUtil = new com.ibm.security.modeling.util.ErrorUtilities();
		dojo.mixin(this, args);
		
		this._trace.exiting("", "constructor");
	},
	
	_setupJsonRestStore: function () {
		this._trace.entering(this.id, "_setupJsonRestStore");
		dojo.require("dojox.data.JsonRestStore");

		this._resourceJsonRestStore = dojox.data.JsonRestStore.getStore({
			// Note: Passing resource uri. Resource Id will be supplied to it when fetch() is invocation. 
			// Actual REST call over HTTP also takes place at fetch() and not here. 
			target: this.resourceUri, 
			schema: this._resourceSchema,
			syncMode: this.syncMode,
			idAttribute: this.idAttribute
		});
		this._trace.exiting(this.id, "_setupJsonRestStore");
	},
	
	getJsonRestStore: function () {
		// summary:
		//		provides instance of dojox.data.JsonRestStore store. 
		//		If instance is not available then creates one.
		this._trace.entering(this.id, "getJsonRestStore");
		try {
			if (this._resourceJsonRestStore == null) {
				this._setupJsonRestStore();
			}
			
			return this._resourceJsonRestStore;
		} finally {
			this._trace.exiting(this.id, "getJsonRestStore");
		}
	}, 
	
	getIsrmTableRestStore: function () {
		// summary:
		//		provides instance of com.ibm.security.modeling.tables.JsonRestStore store.
		this._trace.entering(this.id, "getIsrmTableRestStore");
		dojo.require("com.ibm.security.modeling.tables.JsonRestStore");
		try {
			return new com.ibm.security.modeling.tables.JsonRestStore({
					target: this.resourceUri,
					schema: this._resourceSchema,
					syncMode: this.syncMode,
					idAttribute: this.idAttribute,
					allowNoTrailingSlash:this.allowNoTrailingSlash,
					ignoreMatchQuery: this.ignoreMatchQuery
				});
		} finally {
			this._trace.exiting(this.id, "getIsrmTableRestStore");
		}
	},
	
	_setupRestService: function () {
		this._trace.entering(this.id, "_setupRestService");
		// second boolean parameter 'true' indicates that data is in JSON format.
		this._resourceRestService = new dojox.rpc.Rest(this.resourceUri,true);
		this._trace.exiting(this.id, "_setupRestService");
	},
	
	getRestService: function () {
		// summary:
		//		provides instance of dojox.rpc.Rest api. If instance is not available then creates one.
		this._trace.entering(this.id, "getRestService");
		try {
			if (this._resourceRestService == null) {
				this._setupRestService();
			}
			
			return this._resourceRestService;
		} finally {
			this._trace.exiting(this.id, "getRestService");
		}
	},
	
	loadResource: function (
			/*string*/ resourceId, 
			/*function (item, request)*/successCallBack, 
			/*function (error, request)*/errorCallBack) {
		
		// summary:
		//		Loads resources by using instance of com.ibm.security.modeling.tables.JsonRestStore store. 
		// resourceId: string
		//		Id of Resource that is to be loaded.
		// successCallBack:	function (item, request)
		//		callback function, used when syncMode is false, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used when syncMode is false, used to handle fetch error result		
		this._trace.entering(this.id, "loadResource");
		this._trace.object(arguments, "loadResource", "function args");
		try {
			// Fetch Resource.
			if (this.syncMode == true) {
				// as synchMode is true for this store; no need to add callback for the query results.
				// .results can directly be invoked to retrieve result.
				// not results[0]; as resource object(item) is returned, and not an array that warps it.
				var resourceItem = this.getJsonRestStore().fetch({
						query: (resourceId == null) ? "" : resourceId.toString(),
						onError: dojo.hitch(this, this._onError),
						syncMode: true
					}).results; 
				
				if (resourceItem == undefined) {
					resourceItem = null;
				}
				
				return resourceItem;
			} else {
				// callback method attached to process response result data.
				if (errorCallBack) {
					this.getJsonRestStore().fetch({
							query: (resourceId == null) ? "" : resourceId.toString(), 
							onComplete: successCallBack, 
							onError: errorCallBack 
						});
				} else {
					this.getJsonRestStore().fetch({
							query: (resourceId == null) ? "" : resourceId.toString(), 
							onComplete: successCallBack, 
							onError: dojo.hitch(this, this._onError) 
						});
				}
			}
			
			return null;
		} finally {
			this._trace.exiting(this.id, "loadResource");
		}
	},
	
	requestResource: function(
			/*string*/resourceId, 
			/*function (result)*/successCallBack, 
			/*function (error, request)*/errorCallBack) {
		
		// summary:
		//		Loads resource by using instance of dojox.rpc.Rest api. 
		// resourceId: string
		//	 	id of the resource that is to be loaded.
		// successCallBack:	function (result)
		//		callback function, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used to handle fetch error result
		//
		this._trace.entering(this.id, "requestResource");
		this._trace.object(arguments, "requestResource", "function args");
		var Service = this.getRestService();
		// Make request for resource object(item) from Service.
		var resourceRequest = Service((resourceId == null) ? "" : resourceId.toString()); 	// HTTP GET request
		
		resourceRequest.addCallback(successCallBack);
		
		if (errorCallBack) {
			resourceRequest.addErrback(errorCallBack);
		} else {
			resourceRequest.addErrback(dojo.hitch(this, this._onError));
		}
		
		// This log helps in debugging when any error occurs in call back method. Else it is absorbed silently.
		// this._trace.object(resourceRequest, "requestResource", "request object");
		this._trace.exiting(this.id, "requestResource");
	},
	
	doHttpPut: function (
			/*string*/ resourceId, 
			/*json object*/ data, 
			/*function (result)*/ successCallBack, 
			/*function (error, request)*/ errorCallBack) {
		
		// summary:
		//		Makes an HTTP PUT request, using instance of dojox.rpc.Rest api. 
		// resourceId: string
		//	 	id of the resource that is updated. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// successCallBack:	function (result)
		//		callback function, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used to handle fetch error result
		//		if null, then default is used.
		this._trace.entering(this.id, "doHttpPut");
		this._trace.object(arguments, "doHttpPut", "function args");
		
		var putRequest = this.getRestService().put(
				(resourceId == null) ? "" : resourceId.toString(), 
				data.toString());
		putRequest.addCallback(successCallBack);
		if (errorCallBack) {
			putRequest.addErrback(errorCallBack);
		} else {
			putRequest.addErrback(dojo.hitch(this, this._onError));
		}
		this._trace.exiting(this.id, "doHttpPut");
	},
	
	// Used for creation
	doHttpPost: function (
			/*json object*/ data, 
			/*function (result)*/ successCallBack, 
			/*function (error, request)*/ errorCallBack) {
		
		// summary:
		//		Makes an HTTP POST request, using instance of dojox.rpc.Rest api. 
		// successCallBack:	function (result)
		//		callback function, used to handle fetch success result
		// errorCallBack: function (error, request)
		//		callback function, used to handle fetch error result
		//		if null, then default is used.
		var postRequest = this.getRestService().post("", // no id
				                                     data.toString());
		postRequest.addCallback(successCallBack);
		if (errorCallBack) {
			postRequest.addErrback(errorCallBack);
		} else {
			postRequest.addErrback(dojo.hitch(this, this._onError));
		}
	},
	
	// Used for deletion
	doHttpDelete: function (
			/*string*/ resourceId, 
			/*function (result)*/ successCallBack, 
			/*function (error, request)*/ errorCallBack) {
		
		// summary:
		//		Makes an HTTP DELETE request, using instance of dojox.rpc.Rest api. 
		// resourceId: string
		//	 	id of the resource that is to be deleted. 
		//		If it is null then it is considered that rest api is pointing to that resource.
		// successCallBack:	function (result)
		//		callback function, used to handle delete success result
		// errorCallBack: function (error, request)
		//		callback function, used to handle delete error result
		//		if null, then default is used.
		this._trace.entering(this.id, "doHttpDelete");
		this._trace.object(arguments, "doHttpDelete", "function args");
		
		var deleteRequest = this.getRestService()['delete'](
				(resourceId == null) ? "" : resourceId.toString());
		deleteRequest.addCallback(successCallBack);
		if (errorCallBack) {
			deleteRequest.addErrback(errorCallBack);
		} else {
			deleteRequest.addErrback(dojo.hitch(this, this._onError));
		}
		this._trace.exiting(this.id, "doHttpDelete");
	},
	
	_onError: function (error, request) {
		this._trace.entering(this.id, "_onError");
		this._trace.exception(error, "_onError");
		this._trace.object(arguments, "_onError", "Error function args");
		this._errorUtil.handleRESTError(error, this.messageArea);
		this._trace.exiting(this.id, "_onError");
	}
});