/*
Licensed Materials - Property of IBM

Role and Policy Modeler

 Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.indicator.SaveIndicator");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.security.modeling.util.Trace");
// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");

/* 
 * At minimum the save indicator will be displayed for 3 seconds.
 * If multiple saves occur once, then the save indicator will be displayed for at least 3 seconds
 * OR longer if it takes longer for the save to be processed.
 * Once the save has occurred, the progress indicator will be hidden.
 */ 
dojo.declare (
	"com.ibm.security.modeling.indicator.SaveIndicator", 
    [ dijit._Widget, dijit._Templated ],
{
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.indicator.SaveIndicator"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.indicator","templates/SaveIndicator.html"),
	
	widgetsInTemplate: true,
	
	// globalResources: Object
	//			Reference to the global shared resource bundle
	globalResources: null,
	
	// _savingCount: integer
	//		A counter specifying number of save operations that are is taking place
	//		It is to control Close button behavior. 
	_savingCount: 0,
	
	// _hasTimedOut: boolean
	//		Indicates whether hide() method timeout has occurred or not.
	_hasTimedOut: true,
	
	// _timeOutInterval: integer
	//		Time interval in milliseconds; at minimum the save indicator will be displayed for.
	_timeOutInterval: 3000,
	
	saveIndicatorText: null,
	
	// saveIndicatorStack: Array
	//		Holds the list of indicator texts corresponding to the operations in progress currently.
	saveIndicatorStack: null,
	
	constructor: function() {
		this._trace.entering("", "constructor");
		this.inherited (arguments);		
		this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		this.saveIndicatorText = this.globalResources.SAVING;
		this.saveIndicatorStack = new Array();
		this._trace.exiting("", "constructor");
	},

	destroy: function() {
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			this.inherited(arguments);
		} catch (e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	// Function to push the indicator text corresponding to the current operation
	// 		in the stack.
	pushIndicatorText: function(saveIndicatorText) {
		this._trace.entering(this.id, "pushIndicatorText");
		stackLen = this.saveIndicatorStack?this.saveIndicatorStack.length:0;
		this.saveIndicatorStack[stackLen] = saveIndicatorText;
		this._trace.exiting(this.id, "pushIndicatorText");
	},
	
	// Function to get the indicator text of the most recent operation.
	getMostRecentIndicatorText: function() {
		this._trace.entering(this.id, "getMostRecentIndicatorText");
		stackLen = this.saveIndicatorStack?(this.saveIndicatorStack.length-1):-1;
		this._trace.exiting(this.id, "getMostRecentIndicatorText");
		return (stackLen >= 0)?this.saveIndicatorStack[stackLen]:null;
	},
	
	// Function to show the indicator text of the most recent operation.
	showMostRecentIndicatorText: function() {
		this._trace.entering(this.id, "showMostRecentIndicatorText");	
		this.saveIndicatorText = this.getMostRecentIndicatorText();
		
		if(!this.saveIndicatorText) {
			this._trace.exiting(this.id, "showMostRecentIndicatorText");
			return;
		}
		
		this.indicatorTextDiv.innerHTML = this.indicatorTextDiv.innerHTML.replace("SAVE_INDICATOR_TEXT", this.saveIndicatorText);
		
		// Show save indicator
		dojo.removeClass(this.id,"hideDiv");
		dojo.addClass(this.id,"showDivInline");
		if (this._hasTimedOut) {
			this._hasTimedOut = false;
			// set timer to invoke hide() after timeout interval.
			setTimeout (
					dojo.hitch (this, function() {
						this._hide(this.saveIndicatorText, true);	// true: indicates that _hide call is due to time out.
					}), 
					this._timeOutInterval);	
		}
		
		this._trace.exiting(this.id, "showMostRecentIndicatorText");
	},

	// Function to remove the indicator text of a completed operation, from the stack.
	deleteElementFromStack: function(indexToDelete) {
		this._trace.entering(this.id, "deleteElementFromStack");

		var stackLen = this.saveIndicatorStack.length;

		// Split the indicator stack into two portions: Slice it
		// at the index passed in.		
		array1 = this.saveIndicatorStack.slice(0, indexToDelete);
		array2 = this.saveIndicatorStack.slice(indexToDelete);
		
		// Remove the last element (basically the element that corresponds to the index to delete.
		array2.shift();
		
		this.saveIndicatorStack = array1.concat(array2);

		this._trace.exiting(this.id, "deleteElementFromStack");		
	},
	
	// Function to hide the most recent entry of the stack indicator text, corresponding
	// to the operation that just completed. There may be multiple values of "Saving..."
	// in the stack. We would be deleting the latest entry of "Saving..." from the list,
	// if there's an operation that uses "Saving..." as its indicator text and it just
	// finished.
	deleteMostRecentInstanceofIndicatorText: function(currentOperationText) {
		this._trace.entering(this.id, "deleteMostRecentInstanceofIndicatorText");
		var stacklen = this.saveIndicatorStack.length;
		
		for(var loopCtr = stacklen - 1; loopCtr >= 0; loopCtr--) {
			if(this.saveIndicatorStack[loopCtr] == currentOperationText) {
				this.deleteElementFromStack(loopCtr);
				this._trace.exiting(this.id, "deleteMostRecentInstanceofIndicatorText");
				return;
			}
		}
		this._trace.exiting(this.id, "deleteMostRecentInstanceofIndicatorText");
	},
	
	// Function to reset the indicator text to "SAVE_INDICATOR_TEXT" so that further replacements
	// work fine on this text.
	resetIndicatorText: function(currentStackTop) {
		this._trace.entering(this.id, "resetIndicatorText");
		indicatorText = currentStackTop?currentStackTop:this.getMostRecentIndicatorText();
		if (indicatorText) {
			this.indicatorTextDiv.innerHTML = this.indicatorTextDiv.innerHTML.replace(indicatorText, "SAVE_INDICATOR_TEXT");
		}
		this._trace.exiting(this.id, "resetIndicatorText");
	},

	// Function to display the save indicator with the text thats passed in.
	show: function(currentOperationText) {
		// summary:
		//		Increases saving counter and shows Save indicator.
		//		also sets a timer to hide save indicator after timeout interval.
		
		// Increase the save count
		this._trace.entering(this.id, "show");
		
		if(currentOperationText) {
			this.saveIndicatorText = currentOperationText;
		} else {
			this.saveIndicatorText = this.globalResources.SAVING;
		}
		
		if(this.saveIndicatorStack.length > 0) {
			this.resetIndicatorText();
		}
		
		this.pushIndicatorText(this.saveIndicatorText);
		
		this._savingCount++;
		
		this.showMostRecentIndicatorText();
		
		this._trace.exiting(this.id, "show");
	},
	
	// Function to hide the indicator that was "shown" with the text as passed in.
	hide: function(currentOperationText) {
		// summary:
		//		Decreases saving counter, and 
		//		invokes _hide() indicating that 
		//		it is not invoked due to timeout rather 
		//		it is due to a call from client.
		
		this._trace.entering(this.id, "hide");

		if(currentOperationText) {
			this.saveIndicatorText = currentOperationText;
		} else {
			this.saveIndicatorText = this.globalResources.SAVING;
		}
		
		// Decrease the save count 
		this._savingCount--;
		// resetting, if client code accidently called hide(), more than show() calls.
		if (this._savingCount < 0) {
			this._savingCount = 0;
		}
		
		var currentIndicatorStackTop = this.getMostRecentIndicatorText();
		
		if (this._savingCount >= 0) {
			this.deleteMostRecentInstanceofIndicatorText(this.saveIndicatorText);
		}

		if (this._savingCount == 0) {
			setTimeout (
					dojo.hitch (this, function() {
						this._hide(currentIndicatorStackTop, true);	// true: indicates that _hide call is due to time out.
					}), 
					this._timeOutInterval);	
		} else {
			this.resetIndicatorText(currentIndicatorStackTop);
			this.showMostRecentIndicatorText();
		}
		
		this._trace.exiting(this.id, "hide");
	},
	
	_hide: function(indicatorText, isTimedOut) {
		
		this._trace.entering(this.id, "_hide");
		if (isTimedOut == true) {
			this._hasTimedOut = true;
		}
		
		if (!indicatorText) {
			indicatorText = this.globalResources.SAVING;
		}
			
		// Hide save indicator only if no save is pending and no timer is running.
		if (this._hasTimedOut == true && this._savingCount == 0) {
			dojo.removeClass(this.id,"showDivInline");
			dojo.addClass(this.id,"hideDiv");
			this.indicatorTextDiv.innerHTML = this.indicatorTextDiv.innerHTML.replace(indicatorText, "SAVE_INDICATOR_TEXT");
		}

		this._trace.exiting(this.id, "_hide");
	},
	
	isSaving: function() {
		this._trace.entering(this.id, "isSaving");
		try {
			if (this._savingCount == 0 &&
					dojo.hasClass(this.id,"hideDiv")) {	// No save is in progress. And indicator is hidden.
				return false;
			}
			return true;
		} finally {
			this._trace.exiting(this.id, "isSaving");
		}
	}

});
