/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.membership.FilterData");

dojo.require("dojox.rpc.JsonRest");
dojo.require("com.ibm.security.modeling.util.Trace");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo["requireLocalization"]("customizations", "AttributeCustomization");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// API to map filter data to filter display text for properties like: phrase attribute, phrase values
(function() {
	// 'var' should be used to declare local variables.
	var trace= new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterData");
	
	var _attributeOperatorI18N = null;	// A map of phrase attribute operator value and corresponding i18n value to display.
	var _localResources = null;	// i18n resource
	var _attributeResources = null;
	var _globalResources = null;
	
	var _setupI18NResources = function() {
		_globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		_attributeResources = dojo.i18n.getLocalization("customizations", "AttributeCustomization");
		_localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
	};
	
	var _setupAttributeOperatorI18N = function () {
		if(!_localResources) {
			_setupI18NResources();
		}
		_attributeOperatorI18N = {
			"containedIn": _localResources.CONDITION_ATTRIBUTE_OPERATOR_CONTAINED,
			"contains": _localResources.CONDITION_ATTRIBUTE_OPERATOR_CONTAINS,
			"endsWith": _localResources.CONDITION_ATTRIBUTE_OPERATOR_ENDSWITH,
			"equals": _localResources.CONDITION_ATTRIBUTE_OPERATOR_EQUALS,
			"greaterThan": _localResources.CONDITION_ATTRIBUTE_OPERATOR_GREATERTHAN,
			"greaterThanOrEqual": _localResources.CONDITION_ATTRIBUTE_OPERATOR_GREATERTHAN_OREQUAL,
			"lessThan": _localResources.CONDITION_ATTRIBUTE_OPERATOR_LESSTHAN,
			"lessThanOrEqual": _localResources.CONDITION_ATTRIBUTE_OPERATOR_LESSTHAN_OREQUAL,
			"notContainedIn": _localResources.CONDITION_ATTRIBUTE_OPERATOR_NOTCONTAINED,
			"notEquals": _localResources.CONDITION_ATTRIBUTE_OPERATOR_NOTEQUALS,
			"startsWith": _localResources.CONDITION_ATTRIBUTE_OPERATOR_STARTSWITH,
			"substring": _localResources.CONDITION_ATTRIBUTE_OPERATOR_SUBSTRING
		};
	};

	var _getAttributeOperatorI18NValue = function(/*string*/ operator) {
		// Summary:
		//		Provides i18n value for the attribute operator
		// operator:	string
		//		Operator value
		
		var value = operator;
		if (!_attributeOperatorI18N) {
			_setupAttributeOperatorI18N();
		}
		
		if (operator && (operator in _attributeOperatorI18N)) {
			value = _attributeOperatorI18N[operator];
		}
		return value;
	};
	
	// Declaration in global scope.
	var _fd = com.ibm.security.modeling.membership.FilterData = {
		getFilterGroups: function(/*json object*/ filter) {
			// Summary:
			// 		Extracts filter groups array from filter json.
			//
			// filter:	json object
			//		Json object representing a filter.
			//
			// returns: Array
			//		groups [if any]
			//		null otherwise.
			var groups = null;
			if (filter) {
				var fragments = filter["fragments"];
				if (fragments && dojo.isArray(fragments) && fragments.length > 0) {
					var fragment = fragments[0];
					if (fragment) {
						
						var children = fragment["children"];
						if (children && dojo.isArray(children) && children.length > 0) {
							groups = children;
						}
					}
				}
			}
			return groups;
		},
		
		getFilterGroupOperator: function(/*json object*/ filter) {
			// Summary:
			// 		Extracts filter groups operator from filter json.
			//
			// filter:	json object
			//		Json object representing a filter.
			//
			// returns: string
			
			var operator = null;
			if (filter) {
				var fragments = filter["fragments"];
				if (dojo.isArray(fragments) && fragments.length > 0) {
					var fragment = fragments[0];
					if (fragment && fragment["operator"]) {
						operator = fragment["operator"];
					}
				}
			}
			return operator;
		},
		
		getFilterGroupName: function(/*json object*/ filterGroup) {
			// Summary:
			// 		Extracts filter group name from filter group json.
			//
			// filterGroup:	json object
			//		Json object representing a filter group.
			//
			// returns: string
			
			var name = null;
			if (filterGroup && filterGroup["name"]) {
				name = filterGroup["name"]; 
			}
			return name;
		},
		getFilterGroupPhrases: function(/*json object*/ filterGroup) {
			// Summary:
			// 		Extracts filter group phrases array from filter group json.
			//
			// filter:	json object
			//		Json object representing a filter group.
			//
			// returns: Array
			//		phrases [if any]
			//		null otherwise.
			
			var phrases = null;
			if (filterGroup) {
				var rules = filterGroup["rules"];
				if (dojo.isArray(rules) && rules.length > 0) {
					phrases = rules;
				}
			}
			return phrases;
		},
		
		getFilterGroupPhraseOperator: function(/*json object*/ filterGroup) {
			// Summary:
			// 		Extracts filter group phrases operator from filter group json.
			//
			// filterGroup:	json object
			//		Json object representing a filter group.
			//
			// returns: string
			
			var operator = null;
			if (filterGroup && filterGroup["operator"]) {
				operator = filterGroup["operator"];
			}
			return operator;
		},
		
		getAttributeNameIdentifier: function (/*json object*/ phrase) {
			// Summary:
			//		Extracts attribute name identifier (attribute URI) from phrase.
			//
			// phrase:	json object
			//		Json object representing a phrase
			
			var attributeUri = null;
			if (phrase && ("attributeUri" in phrase)) {
				attributeUri = phrase["attributeUri"];
			}
			return attributeUri;
		},
		
		getAttributeOperatorIdentifier: function (/*json object*/ phrase) {
			// Summary:
			//		Extracts attribute operator identifier (operator value from REST) from phrase.
			//
			// phrase:	json object
			//		Json object representing a phrase
			
			var operator = null;
			if (phrase && ("operator" in phrase)) {
				operator = phrase["operator"];
			}
			return operator;
		},
		
		getAttributeValuesIdentifiers: function (/*json object*/ phrase) {
			// Summary:
			//		Extracts attribute values identifier (values from REST) from phrase.
			//
			// phrase:	json object
			//		Json object representing a phrase
			
			var attributeValues = null;
			if (phrase && ("attributeValues" in phrase)) {
				var values = phrase["attributeValues"];
				if (values && dojo.isArray(values) && values.length > 0) {
					attributeValues = values;
				}
			}
			return attributeValues;
		},
		
		
		
		getAttributeName: function (/*json object*/ phrase) {
			// Summary:
			//		Extracts attribute name from phrase.
			//
			// phrase:	json object
			//		Json object representing a phrase
			
			var name = null;
			
			if(!_attributeResources) {
				_setupI18NResources();
			}			
			
			if (phrase && ("attributeName" in phrase)) {
				name = phrase["attributeName"];
				if (name) {					
					name = (name.toUpperCase() in _attributeResources) ? 
						_attributeResources[name.toUpperCase()] : name;
				}
			}
			return name;
		},
		
		getAttributeOperator: function (/*json object*/ phrase) {
			// Summary:
			//		Extracts attribute operator from phrase.
			//
			// phrase:	json object
			//		Json object representing a phrase
			
			var operatorText = null;
			if (phrase && ("operator" in phrase)) {
				operatorText = _getAttributeOperatorI18NValue(phrase["operator"]);
			}
			return operatorText;
		},
		
		getAttributeValues: function(/*json object*/ phrase) {
			// Summary:
			//		Extracts attribute values array from phrase.
			//
			// phrase:	json object
			//		Json object representing a phrase
			
			var values = null;
			if (phrase && ("attributeValues" in phrase)) {
				var data = phrase["attributeValues"];
				if (dojo.isArray(data) && data.length > 0) {
					values = data;
				}
			}
			return values;
		},
		
		getAttributeValuesText: function(/*phrase json object | array*/ obj) {
			// Summary:
			//		Constructs the display text from attribute values
			//
			// obj
			//		This can be 1) Phrase json object 2) An array containing values.
			var text = "";
			if (!obj) {
				return text;
			}
			
			var values = obj;
			if (!dojo.isArray(obj)) { // not an array. 
				// considering it a phrase json; retrieve values from it.
				values = _fd.getAttributeValues(obj);
			}
			
			// construct the text.
			if (values && dojo.isArray(values) && values.length > 0) {
				if(!_globalResources) {
					_setupI18NResources();
				}
				for (var i=0; i<values.length; i++) {
					if (i != 0) {
						text = text + _globalResources.COMMA;
					}
					text = text + values[i]["value"];
				}
			}
			return text;
		},
		
		getFilterJsonForREST: function(/*json*/ uiFilter, /*json*/ roleItem) {
			// Summary:
			//		Reads the input JSON and provides the json per the REST interface.
			//
			// uiFilter: json object
			//		json representing a filter from UI elements.
			// roleItem: json object
			//		json represnting a role
			//
			// returns: json
			//		Filter json per the REST interface.
			var restFilter = null;
			var restGroups = [];
			
			if (!uiFilter || !roleItem) {
				return restFilter;
			}
			
			// Extract Groups and Operator information from UI object.
			var uiGroups = uiFilter["elements"];	// filter groups
			if (!uiGroups || !dojo.isArray(uiGroups) || !(uiGroups.length > 0)) {
				// no filter
				return restFilter;
			}
			
			var restGroupOperator = _fd._getItemAttributeValue(/*item container*/uiFilter["operator"], /*attribute*/"idAttribute"); // filter group level operator
			var restGroup = null;
			for (var i=0; i<uiGroups.length; i++) {
				restGroup = _fd._getGroupJsonForREST(uiGroups[i]);	// extract group
				if (restGroup) {
					restGroups.push(restGroup);
				}
			}
			
			// Construct Filter JSON Object
			if ( restGroups.length == 0) { // No groups: no filter.
				return restFilter;
			}
			
			restFilter = {
				"fragments": [
					{
						"name": "",
						"operator": restGroupOperator? restGroupOperator: "or",
						"isRoot": true,
						"children": restGroups
				    }
				],
				"role": roleItem["uri"]
			};
			
			return restFilter;
		},
		
		_getGroupJsonForREST: function(/*json*/ uiGroup) {
			// Summary:
			//		Reads the input group JSON and provides the group json per the REST interface.
			//
			// uiGroup: json object
			//		json representing a group from UI elements.
			//
			// returns: json
			//		Group json per the REST interface.
			var restGroup = null;
			var restPhrases = [];
			
			if (!uiGroup) {
				return restGroup;
			}
			
			var restGroupName = uiGroup["name"];
			var restPhraseOperator = _fd._getItemAttributeValue(/*item container*/uiGroup["operator"], /*attribute*/"idAttribute"); // phrase level operator
			
			var uiPhrases = uiGroup["elements"];	// phrases
			var restPhrase = null;
			if (uiPhrases && dojo.isArray(uiPhrases) && uiPhrases.length > 0) {
				for (var i=0; i<uiPhrases.length; i++) {
					restPhrase = _fd._getPhraseJsonForREST(uiPhrases[i]);	// extract phrase
					if (restPhrase) {
						restPhrases.push(restPhrase);
					}
				}
			}
			
			// Construct Group JSON Object
			restGroup = {
					"children": null,
                    "name": restGroupName? restGroupName:"",
                    "operator": restPhraseOperator? restPhraseOperator: "and",
                    "isRoot": false,
                    "rules": restPhrases
			};
			
			return restGroup;
		},
		
		_getPhraseJsonForREST: function(/*json*/ uiPhrase) {
			// Summary:
			//		Reads the input phrase JSON and provides the phrase json per the REST interface.
			//
			// uiPhrase: json object
			//		json representing a phrase from UI elements.
			//
			// returns: json
			//		Phrase json per the REST interface.
			
			var restPhrase = null;
			if (!uiPhrase) {
				return restPhrase;
			}
			
			var restAttributeUri = null;
			var restAttributeOperator = null;
			var restAttributeValues = [];
			var restAttributeValue = null;
			
			restAttributeUri = _fd._getItemAttributeValue(/*item container*/uiPhrase["attribute"], /*attribute*/"idAttribute"); // phrase attribute
			restAttributeOperator = _fd._getItemAttributeValue(/*item container*/uiPhrase["operator"], /*attribute*/"idAttribute"); // phrase attribute operator
			
			var uiAttributeValues = uiPhrase["values"];
			var uiAttributeValue = null;
			var uiAttributeValueUri = null;
			if (uiAttributeValues && dojo.isArray(uiAttributeValues) && uiAttributeValues.length > 0) {
				for (var i=0; i< uiAttributeValues.length; i++) {
					restAttributeValue = null;
					
					uiAttributeValue = _fd._getItemAttributeValue(/*item container*/uiAttributeValues[i], /*attribute*/"valueAttribute"); // phrase value
					uiAttributeValueUri = _fd._getItemAttributeValue(/*item container*/uiAttributeValues[i], /*attribute*/"idAttribute"); // phrase value uri
					
					if (uiAttributeValue || uiAttributeValueUri) {
						restAttributeValue = {};
						// send either 'uri' or 'value.
						// if both are availabe then 'uri' get the preference.
						if (uiAttributeValueUri) {
							restAttributeValue["uri"]= uiAttributeValueUri;
						} else if (uiAttributeValue) {
							restAttributeValue["value"]= uiAttributeValue;
						}
						restAttributeValues.push(restAttributeValue);
					}
				}
			}
			
			restPhrase = {
                "attributeUri": restAttributeUri,
                "attributeValues": restAttributeValues,
                "operator": restAttributeOperator
			};
			return restPhrase;
		},
		// Util
		_getItemAttributeValue: function(/*json*/ container, /*attribute*/ attribute) {
			// Summary:
			//		Reads the input operator JSON and provides the operator json per the REST interface.
			//
			// container: json object
			//		json object conating an "item"
			// attribute: string
			//		attribute to be read from the "item"
			//
			// returns: json [string]
			//		"item" attribute value.
			
			var value = null;
			if (!container || !attribute) {
				return value;
			}
			var item = container.item;
			var valueAttribute = container[attribute];
			if (!item || !valueAttribute) {
				return value;
			}
			
			value = item[valueAttribute];
			if (value && isNaN(value)) {
				value = value.toString();
			}
			return value;
			
		},
		
		areFiltersEqual: function(/*json*/ filter1, /*json*/ filter2) {
			// Summary:
			//		Validates, if both filter objects are equal.
			//
			//		A simple implementation, limited to compare filter json.
			
			if (!filter1 && !filter2) { // both are null or undefined.
				return true;
			} 
			if ((!filter1 && filter2) || (filter1 && !filter2)) { // one of the filter is null or undefined.
				return false;
			}
			
			var filter1Operator = _fd.getFilterGroupOperator(filter1);
			var filter2Operator = _fd.getFilterGroupOperator(filter2);
			if (filter1Operator != filter2Operator) {
				return false;
			}
			
			var groups1 = _fd.getFilterGroups(filter1);
			var groups2 = _fd.getFilterGroups(filter2);
			if (!groups1 && !groups2) { // both are null or undefined.
				return true;
			} 
			
			if (groups1.length == groups2.length) {
				for (var i=0; i<groups1.length; i++) {
					if (! _fd.areFilterGroupsEqual(groups1[i], groups2[i])) {
						return false;
					}
				}
				return true;
			} 
			return false;
		},
		
		areFilterGroupsEqual: function(/*json*/ group1, /*json*/ group2) {
			// Summary:
			//		Validates, if both filter group objects are equal.
			//
			//		A simple implementation, limited to compare filter group json.
			if (!group1 && !group2) { // both are null or undefined.
				return true;
			} 
			if ((!group1 && group2) || (group1 && !group2)) { // one of the group is null or undefined.
				return false;
			}
			
			if ((group1["operator"] != group2["operator"]) ||
					(group1["name"] != group2["name"])) {
				return false;
			}
			
			var phrases1 = _fd.getFilterGroupPhrases(group1);
			var phrases2 = _fd.getFilterGroupPhrases(group2);
			if (!phrases1 && !phrases2) { // both are null or undefined.
				return true;
			}
			if ((!phrases1 && phrases2) || (phrases1 && !phrases2)) { // one of the phrases is null or undefined.
				return false;
			}
			
			if (phrases1.length == phrases2.length) {
				for (var i=0; i<phrases1.length; i++) {
					if (! _fd.areFilterGroupPhrasesEqual(phrases1[i], phrases2[i])) {
						return false;
					}
				}
				return true;
			} 
			return false;
		},
		
		areFilterGroupPhrasesEqual: function(/*json*/ phrase1, /*json*/ phrase2) {
			// Summary:
			//		Validates, if both filter group phrase objects are equal.
			//
			//		A simple implementation, limited to compare filter group phrase json.
			if (!phrase1 && !phrase2) { // both are null or undefined.
				return true;
			} 
			if ((!phrase1 && phrase2) || (phrase1 && !phrase2)) { // one of the phrase is null or undefined.
				return false;
			}
			
			if ((phrase1["attributeUri"] != phrase2["attributeUri"]) ||
					(phrase1["operator"] != phrase2["operator"])) {
				return false;
			}
			
			var phrase1Values = phrase1["attributeValues"];
			var phrase2Values = phrase2["attributeValues"];
			if (!phrase1Values && !phrase2Values) { // both are null or undefined.
				return true;
			} 
			if ((!phrase1Values && phrase2Values) || (phrase1Values && !phrase2Values)) { // one of the phrase values is null or undefined.
				return false;
			}
			
			if (phrase1Values.length == phrase2Values.length) {
				for (var i=0; i<phrase1Values.length; i++) {
					var valueAttr = "uri" in phrase1Values[i]? "uri": "value";
					if (phrase1Values[i][valueAttr] != phrase2Values[i][valueAttr]) {
						return false;
					}
				}
				return true;
			} 
			return false;
		}
	};
	
})();