/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.membership.FilterEvaluationUtil");

dojo.require("dojox.rpc.JsonRest");
dojo.require("com.ibm.security.modeling.util.Trace");

// NLS
dojo.requireLocalization("com.ibm.security.modeling", "PluginMessages");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// API to perform filter evaluation, and display response.

(function() {
	// 'var' should be used to declare local variables.
	var trace = new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.FilterEvaluationUtil");
	
	// i18n resources
	var _pluginResources = null; 
	var _filterResources = null;	
	
	var _setupI18NResources = function() {
		_pluginResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "PluginMessages");
		_filterResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
	};
	
	var _getPluginResources = function() {
		if (!_pluginResources) {
			_setupI18NResources();
		}
		return _pluginResources;
	};
	
	var _getFilterResources = function() {
		if (!_filterResources) {
			_setupI18NResources();
		}
		return _filterResources;
	};
	
	// Declaration in global scope.
	var _feu = com.ibm.security.modeling.membership.FilterEvaluationUtil = {
		handleFilterEvaluationRequest: function(args) {
			// Summary:
			//		Adds success, error call-backs to the filterEvaluationRequest object.
			//		Displays the save indicator.
			// args:	argument object
			// 		It should contain following attributes:
			// 		-	filterEvaluationRequest: xhr request
			//				xhr request corresponding to the filter evaluation HTTP GET request.
			// 		-	filterUri: uri string
			//				Filter Uri
			//		-	caller: invoker object
			//		- 	saveIndicator: save indicator
			//		- 	onFilterEvaluationComplete: funation
			//				function to invoke when evaluation completes
			//		- 	messageArea: message area
			
			var filterEvaluationRequest = args["filterEvaluationRequest"];
			
			
			if (!filterEvaluationRequest) {
				trace.finest(" - no xhr.", "handleFilterEvaluationRequest");
				return;
			}
			
			var caller = args["caller"];
			
			trace.entering(caller.id, "handleFilterEvaluationRequest");
			
			var filterUri = args["filterUri"];
			var saveIndicator = args["saveIndicator"];
			var saveIndicatorText = args["saveIndicatorText"];
			var onFilterEvaluationStart = args["onFilterEvaluationStart"];
			var onFilterEvaluationComplete = args["onFilterEvaluationComplete"];
			var handleRESTError = args["handleRESTError"];
			
			// show save indicator
			onFilterEvaluationStart();
			saveIndicator.show(saveIndicatorText);
			
			var onResponse = function(/*boolean*/ isError) {
				// hide save indicator
				saveIndicator.hide(saveIndicatorText);
				onFilterEvaluationComplete();
			};
			
			// Success handler
			var successCallBack = function( result ) {
				try {
					trace.entering(caller.id+" - evaluation.", "successCallBack");
					return result;
				} finally {
					onResponse(/*isError*/ false);
					trace.exiting(caller.id+" - evaluation.", "successCallBack");
				}
			};
			
			// Failure handler
			var errorCallBack = function( error, request ) {
				try {
					trace.entering(caller.id+" - evaluation.", "errorCallBack");
					// Display HTTP Error message
					handleRESTError(error, request);
					
					// Diaplay evaluation failed Message and link to re-evaluate.
					_feu.renderFilterEvaluationMessage(args);
					return error;
					
				} finally {
					onResponse(/*isError*/ true);
					trace.exiting(caller.id+" - evaluation.", "errorCallBack");
				}
			};
			
			// attach callbacks with the request 
			filterEvaluationRequest.addCallback(successCallBack);
			filterEvaluationRequest.addErrback(errorCallBack);
		
			trace.exiting(caller.id, "handleFilterEvaluationRequest");
		},
		
		renderFilterEvaluationMessage: function(args) {
			// Summary:
			//		If role is out of sync with the statistics, [filter status is "FAILED"]
			//		then display an error message with a link to run an evaluation should be displayed.
			//		
			//		This method constructs, and displays the message.
			//
			// filterUri: uri string
			//		URI of the filter.
			
			var filterUri = args["filterUri"];
			var caller = args["caller"];
			var messageArea = args["messageArea"];
			
			trace.entering(caller.id, "renderFilterEvaluationMessage");
			
			if (!filterUri) {
				return;
			}
			
			// show message, and provide link to re-evaluate.
			// Get message text for evaluation failure.
			var evaluationFailedMessageText = _getPluginResources().PLUGIN_REST_MEMBERSHIP_FILTER_EVALUATE_MEMBERSHIP_FILTER_EVALUATION_EXCEPTION;
			var reEvaluateText = _getFilterResources().FILTER_RE_EVALUATE_TEXT;
			var anchorId = caller.id+"_reEvaluate";
			var anchorHtmlText = "<div class='reEvaluateFilterLinkContainer'><a href='javascript:void(0);' id='"+anchorId+"' class='reEvaluateFilterLink'>" +
							reEvaluateText +
						"</a></div>";
			
			var messageHtmlText = evaluationFailedMessageText+anchorHtmlText;
			
			var messageWidgetId = messageArea.showErrorMessage(messageHtmlText);
			var onClickHandler = function() {
				_feu._onReEvaluateFilter(args, messageWidgetId);
			};
			
			var anchorQuery = "#"+messageWidgetId+" a.reEvaluateFilterLink";
			var anchorNodeList = dojo.query(anchorQuery); // NodeList
			anchorNodeList.onclick(onClickHandler);
			
			trace.exiting(caller.id, "renderFilterEvaluationMessage");
		},
		
		_onReEvaluateFilter: function(args, /*string*/ messageWidgetId) {
			// Summary:
			//		Starts re-evaluation of the filter.
			
			var filterUri = args["filterUri"];
			var caller = args["caller"];
			var messageArea = args["messageArea"];
			
			trace.entering(caller.id, "_onReEvaluateFilter");
			if (!filterUri) {
				trace.entering(caller.id+" - no filter uri.", "_onReEvaluateFilter");
				return;
			}
			
			// HTTP GET for re-evaluation
			var filterEvaluationRequest = _feu.startFilterEvaluation(filterUri);
			args["filterEvaluationRequest"] = filterEvaluationRequest;
			_feu.handleFilterEvaluationRequest(args);
			
			// re-evaluation started.
			// hide message now.
			messageArea.hideMessage(messageWidgetId);
			trace.exiting(caller.id, "_onReEvaluateFilter");
		},
		
		startFilterEvaluation: function(/*uri string*/filterUri) {
			// Summary:
			//		Makes an HTTP get request to start the filter evaluation.
			// filterUri: uri string
			//		URI of the filter. appending "?evaluate=true" makes it URI to evaluate filter.
			
			
			var filterEvaluationRequest = null;
			if (filterUri) {
				// "?evaluate=true" can be appended to it to create the URI to initiate the filter evaluation.
				var  evaluationUri = filterUri+"?evaluate=true";
				var filterEvaluationRestService = new dojox.rpc.Rest(evaluationUri, true);
				filterEvaluationRequest = filterEvaluationRestService("");
			}
			return filterEvaluationRequest;
		},
		
		isFilterEvaluationFailed: function(/*json object*/ filterData) {
			// Summary:
			//		Returns true if the evaluation of the filter failed.
			//
			// filterData: json object
			//		JSON object from REST, representing a filter.
			
			return filterData["status"] && filterData["status"] == "FAILED";
		}
	};
})();