/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

(c) Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.membership.StackContainer");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.security.modeling.membership.StackItem");
dojo.require("com.ibm.security.modeling.util.Trace");

//NLS
dojo.requireLocalization("com.ibm.security.modeling", "GlobalText");
dojo.requireLocalization("com.ibm.security.modeling.membership", "FilterText");

// define the widget class
dojo.declare (
	"com.ibm.security.modeling.membership.StackContainer", 
    [ dijit._Widget, dijit._Templated ], {
		
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.membership.StackContainer"),
	templatePath: dojo.moduleUrl("com.ibm.security.modeling.membership","templates/StackContainer.html"),
	widgetsInTemplate: true,
	
	// _items: Array
	//			Holds references of the stacked items.
	//			Last item in the array is the stack top.
	// itemsIdAttribute: string
	//			Identifier property of the items.
	
	// _allowDuplicateItems: boolean
	//			Indicates if items with same value attribute is allowed
	_allowDuplicateItems: false,
	
	
	// _focusItem: StackItem
	//		Item to focus when this widget is focused via focus() call.
	
	constructor: function() {
		// globalResources: Object
		//			Reference to the global shared resource bundle
		// localResources: Object
		//			Reference to the resource bundle unique to this widget
		
		this._trace.entering("", "constructor");
		this._trace.object(arguments, "constructor", "Constructor args");
		this._items = [];
		this._focusItem = null;
		this._trace.exiting("", "constructor");
	},
	
	postMixInProperties: function() {
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postMixInProperties");
		
		this.inherited(arguments);
		if (!this.globalResources) {
			this.globalResources = dojo.i18n.getLocalization("com.ibm.security.modeling", "GlobalText");
		}
		if (!this.localResources) {
			this.localResources = dojo.i18n.getLocalization("com.ibm.security.modeling.membership", "FilterText");
		}
		
		if (!this.itemsIdAttribute) {
			this.itemsIdAttribute = "uri";
		}
		this._trace.exiting(this.id, "postMixInProperties");
	},
	
	postCreate: function() { 
		// Summary:
		//		Widget life-cycle method. 
		
		this._trace.entering(this.id, "postCreate");
		
		this.inherited(arguments);
		
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function() {
		// Summary:
		//		Widget life-cycle method.
		//		Starts up the widget, and its child widget.
		this._trace.entering(this.id, "startup");
		
		this.inherited(arguments);
		this._trace.exiting(this.id, "startup");
	},
	
	destroy: function() {
		//	Summary:
		//		Widget life-cycle method.
		//		Destroys this widget and is contents recursively.
		
		this._trace.enteringWithStack(this.id, "destroy", true);
		try {
			if (this._items) {
				for (var i=0; i<this._items.length; i++) {
					this._items[i].destroyRecursive();
					this._items[i] = null;
				}
				delete this._items;
			}
			
			this.inherited(arguments);
			
			this._trace.exiting(this.id, "destroy");
		} catch(e) {
			this._trace.exception(e, "destroy");
			throw e;
		}
	},
	
	onBlur: function() {
		// Set the focusItem reference to the first item.
		//		So that if focus() is called, then focus goes to the first item.
		//		If stack is not empty, then set the first item as the item to be focused, when focus() is invoked.
		
		this._trace.entering(this.id, "onBlur");
		this._updateFocusItem();	
		this._trace.exiting(this.id, "onBlur");
	},
	
	focus: function() {
		// Summary:
		//		Method to set focus to this widget's focus item, via code.
		if (this._focusItem) {
			this._focusItem.focus();
		}
	},
	
	getJson: function() {
		// Summary:
		//		Constructs a JSON representation from UI elements representing phrase values.
		// returns:
		//		null - if no value is selected
		//		Array - containing values items.
		try {
			this._trace.entering(this.id, "getJson");
			
			var jsonObj = null;
			
			if (!this.isEmpty()) {
				jsonObj = []; 
				
				for (var i=0; i<this._items.length; i++) {
					
					jsonObj.push({
						item: this._items[i].data,		// data item
						valueAttribute: this._items[i].valueAttribute,	// data item value attribute
						idAttribute: "uri"
					});
				}
			}
			
			return jsonObj;
		} finally {
			this._trace.exiting(this.id, "getJson");
		}
	},
	
	addItem: function(/*json object*/ itemData, /*string*/ valueAttribute) {
		// Summary:
		//		Adds item to stack.
		//
		// itemData: json object
		//		Item to be added to stack
		//		Contains json object reference, for which stack item is to be shown on UI.
		// valueAttribute: string
		//		attribute, which should be used to display on UI.
		//
		// returns: boolean
		//		true: if item is added.
		//		false: otherwise.
		
		try {
			this._trace.entering(this.id, "addItem");
			
			if (!itemData) {
				this._trace.entering(this.id+" - no data.", "addItem");
				return false;
			}
			
			if (this._allowDuplicateItems === false) {
				// no duplicates allowed.
				if (this._isDuplicateItem(itemData, valueAttribute) === true ) {
					return false;
				}
			}
			
			var item = this._createStackItem(itemData, valueAttribute);
			this._push(item);
			dojo.window.scrollIntoView(item.domNode);
			
			return true;
		} finally {
			this._trace.exiting(this.id, "addItem");
		}
	},
	
	removeItem: function(item) {
		// Summary:
		//		Removes item from stack. 
		//
		// item: Value Item Widget [StackItem]
		
		this._trace.entering(this.id, "removeItem");
		var itemData = item.data;
		var itemWidget = this._splice(item);
		
		if (itemWidget) {
			// destroy the item widget
			itemWidget.destroyRecursive();
		}
		
		// adjust focus
		this.focus();
		
		this.onRemoveItem(itemData);
		this._trace.exiting(this.id, "removeItem");
	},
	
	onRemoveItem: function(/*json object*/ itemData) {
		// Summary:
		//		Method to connect to, to get the data of the removed item. 
		//
		// itemData: json object
		//		Data of removed Value Item Widget [StackItem]
		
		this._trace.entering(this.id, "onRemoveItem");
		
		this._trace.exiting(this.id, "onRemoveItem");
	},
	
	empty: function() {
		// Summary:
		//		Empties the stack.
		
		// Summary:
		//		Empties the stack.
		this._trace.entering(this.id, "empty");
		while (!this.isEmpty()) {
			var itemWidget = this._pop();
			itemWidget.destroyRecursive();
		}
		
		this._updateFocusItem();
		
		this._trace.exiting(this.id, "empty");
	},
	
	getValueText: function() {
		// Summary:
		//		Constructs the text by iterating over its items 
		//		and by invoking getValueText function on them
		try {
			this._trace.entering(this.id, "getValueText");
			if (this.isEmpty()) {
				return null;
			}
			
			var text = "";
			var topItemIndex = this._items.length-1;
			for (var i = topItemIndex; i >= 0; i--) {
				// top to bottom of the stack.
				if(i < topItemIndex) {// items should be separated by Comma 
					text = text + this.globalResources.COMMA;
				}
				text = text + this._items[i].getValueText();
			}
			return text;
		} finally {
			this._trace.exiting(this.id, "getValueText");
		}
	},
	
	isEmpty: function() {
		// Summary:
		//		Checks if stack is empty [contains no item]
		// returns:
		//		true - if stack is empty
		//		false - if stack is not empty
		
		try {
			this._trace.entering(this.id, "isEmpty");
			return this._items.length == 0;
		} finally {
			this._trace.exiting(this.id, "isEmpty");
		}
	},
	
	size: function() {
		// Summary:
		//		returns the number of stack items in the stack
		return this._items.length;
	},
	
	_push: function(item) {
		// Summary:
		//		Provides Stack push behavior.
		// item: attribute value widget [StackItem]
		
		this._trace.entering(this.id, "_push");
		this._items.push(item);
		
		// item to focus when focus() is invoked.
		if (!this._focusItem) {
			this._focusItem = this._items[0];
		}
		
		this._trace.exiting(this.id, "_push");
	},
	
	_pop: function() {
		// Summary:
		//		Provides Stack pop behavior.
		// Returns null, if stack is empty.
		//		else returns StackItem instance from stack top.
		
		try {
			this._trace.entering(this.id, "_pop");
		
			if (!this.isEmpty()) {
				var topItem = this._items[this._items.length-1];
				this._items.splice(this._items.length-1, 1);
				return topItem;
			}
			return null;
		} finally {
			this._trace.exiting(this.id, "_pop");
		}
	},
	
	_splice: function(/*string*/ item) {
		// Summary:
		//		Provides splice behavior, to remove a particular item from stack [breaks LIFO though]
		//		Returns null, if item was not found in stack.
		//
		// item: Value Item Widget [StackItem]
		
		try {
			this._trace.entering(this.id, "_splice");
			for (var i=0; i<this._items.length; i++) {
				if (this._items[i].id == item.id) {
					
					var itemWidget = this._items[i];
					this._items.splice(i, 1);
					
					// update item to focus.
					this._updateFocusItem(i-1);
					return itemWidget;
				}
			}
			return null;
		} finally {
			this._trace.exiting(this.id, "_splice");
		}
	},
	
	_itemCounter: 0,
	_createStackItem: function(/*json object*/ itemData, /*string*/ valueAttribute) {
		// Summary:
		//		Creates instance of com.ibm.security.modeling.membership.StackItem
		// returns 
		//		Created instance of com.ibm.security.modeling.membership.StackItem
		//
		// itemData: json object
		//		Contains json object reference, for which stack item is to be shown on UI.
		// valueAttribute: string
		//		attribute, which should be used to display on UI.
		
		var stackItem = new com.ibm.security.modeling.membership.StackItem({
			id: this.id+"_item"+(this._itemCounter++),
			data: itemData,
			valueAttribute: valueAttribute,
			container: this,
			globalResources: this.globalResources,
			localResources: this.localResources,
			"class": "mandatoryFullWidth"
		});
		
		// Attach the value item widget to the DOM.
		dojo.place(stackItem.domNode, this._container, "first");	// TOP of the container on UI.
		
		stackItem.startup();
		
		return stackItem;
	},
	
	_isDuplicateItem: function(/*json object*/ itemData, /*string*/ valueAttribute) {
		// Summary:
		//		checks if item is already available in stack.
		// 
		// itemData: json object
		//		Item to be checked.
		// valueAttribute: string
		//		attribute, which should be used to display on UI.
		//
		// returns: boolean
		//		true: if item is already in stack.
		//		false: otherwise.
		
		var data = null;
		for (var i=0; i<this._items.length; i++) {
			data = this._items[i].data;
			
			if (data == itemData ||
					(this.itemsIdAttribute && data[this.itemsIdAttribute] && itemData[this.itemsIdAttribute] && (data[this.itemsIdAttribute] == itemData[this.itemsIdAttribute])) ||
					(valueAttribute && data[valueAttribute] && itemData[valueAttribute] && (data[valueAttribute] == itemData[valueAttribute]))) {
				
				return true;
			}
		}
		return false;
	},
	
	_updateFocusItem: function(/*int?*/ itemIndex) {
		// Summary:
		//		Updates the _focusItem reference to the item (if stack is not empty) or sets it null.
		// itemIndex: int
		//		Optional.
		//		If invalid input, but stack is not empty, then sets the first item as _focusItem
		//		If valid input [>=0 and <  items length-1] then sets the item at the supplied index, as the _focusItem
		
		if (isNaN(itemIndex)) {
			itemIndex = 0;
		}
		// valid index.
		if (!this.isEmpty()) {
			if (itemIndex < 0 || this._items.length <= itemIndex) {
				itemIndex = 0;
			}
			this._focusItem = this._items[itemIndex];
		} else {
			// no item; nothing to focus.
			this._focusItem = null;
		}
		
	}
});