/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
dojo.provide("com.ibm.security.modeling.tables.JsonBatchRest");

dojo.require("dojox.rpc.JsonRest");
dojo.require("com.ibm.security.modeling.util.Trace");

// Adds batch update capability to dojox.rpc.JsonRest
(function() {
	// 'var' should be used to declare local variables.
	var trace= new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.tables.JsonBatchRest");
	
	// Declaration in global scope.
	com.ibm.security.modeling.tables.JsonBatchRest = {
			
		/*
		 * JsonRest has resolveJson() method, which should be used ideally.
		 * But this function is within the function scope in which JsonRest is defined, hence outside accessibility is restricted.
		 * 
		 * Hence, I had to have this method defined here.
		 */
		// resolves the Json object from parameter 'value' and creates/updates the object in Rest._index map.
		resolveJson : function (service, deferred, value, defaultId){
			var timeStamp = deferred.ioArgs && deferred.ioArgs.xhr && deferred.ioArgs.xhr.getResponseHeader("Last-Modified");
			if(timeStamp && dojox.rpc.Rest._timeStamps){
				dojox.rpc.Rest._timeStamps[defaultId] = timeStamp;
			}
			var hrefProperty = service._schema && service._schema.hrefProperty;
			if(hrefProperty){
				dojox.json.ref.refAttribute = hrefProperty;
			}
			value = value && dojox.json.ref.resolveJson(value, {
				defaultId: defaultId, 
				index: dojox.rpc.Rest._index,
				timeStamps: timeStamp && dojox.rpc.Rest._timeStamps,
				time: timeStamp,
				idPrefix: service.servicePath.replace(/[^\/]*$/,''),
				idAttribute: dojox.rpc.JsonRest.getIdAttribute(service),
				schemas: dojox.rpc.JsonRest.schemas,
				loader:	dojox.rpc.JsonRest._loader,
				idAsRef: service.idAsRef, 
				assignAbsoluteIds: true
			});
			dojox.json.ref.refAttribute  = "$ref";
			return value;
		}	
	};
	
	
	// local reference to dirtyObjects scoped variable from dojox.rpc.JsonRest.
	var dirtyObjects; // reference to dirtyObjects array from dojox.rpc.JsonRest
	
	
	/**
	 * Update dojox.rpc.JsonRest.commit() such that 
	 * if 'kwArgs' has 'handleAsBatch' true, 
	 * 	then do HTTP post in batched way.
	 * else run default behavior.
	 */
	var oldCommit = dojox.rpc.JsonRest.commit;
	dojox.rpc.JsonRest.commit = function (kwArgs) {
		if (kwArgs && kwArgs.handleAsBatch === true) {
			
			/*
			 * As batch request may result into partial success, failure hence update default revert logic.
			 * All candidate items (added/deleted) are available in 'dirtyObjects' variable.
			 * These items required later in the response handlers for revert.
			 * But while, save is on, there can be further updates in the store, 
			 * 
			 * hence keeping the copy of present dirtyObjects in savingObjects, and making it available later via 'kwArgs'.
			 */ 
			var savingObjects = [];
			kwArgs.revertOnError = false;
			kwArgs.savingObjects = savingObjects;
			
			dirtyObjects = dojox.rpc.JsonRest.getDirtyObjects();
			
			for(var i = 0; i < dirtyObjects.length; i++){
				var dirty = dirtyObjects[i];
				var object = dirty.object;
				var old = dirty.old;
				if(!(kwArgs.service && (object || old) && 
						(object || old).__id.indexOf(kwArgs.service.servicePath)) && dirty.save){
					savingObjects.push(dirty);
				}
			}
		}
		return oldCommit(kwArgs);	
	};
	
	/**
	 * Update dojox.rpc.JsonRest.sendToServer() such that 
	 * if 'kwArgs' has 'handleAsBatch' true,
	 * 	then do HTTP post in batched way.
	 * else run default behavior.
	 */
	var oldSendToServer = dojox.rpc.JsonRest.sendToServer;
	dojox.rpc.JsonRest.sendToServer = function (actions, kwArgs) {
		if (actions && actions.length > 0 && 
				kwArgs.handleAsBatch === true) {
			return sendBatchToServer(actions, kwArgs);	// ?:
		}
		return oldSendToServer(actions, kwArgs);
	};
	
	/**
	 * Performs batch HTTP POST and handles response.
	 * This method takes care of executing revert logic for partial failure of batch request items.
	 * 	
	 * This method also takes care of following scenarios where:
	 * 	A) An HTTP DELETE results into an UPDATE on REST server, and response contents have updated payload.
	 * 		- corresponding item in Rest._index is updated and item re-appears in GUI table (if any)
	 * 	B) An HTTP POST (Create) results into an UPDATE on REST server, and response contents have updated payload.
	 * 		- This is identified by checking if identifier in response payload already has an item in Rest._index map.
	 * 		- corresponding item in Rest._index is updated and duplicate item in GUI table (if any) is removed.
	 */
	var sendBatchToServer = function(actions, kwArgs) {
		// kwArgs: arguments, that are supplied to store.save() function.
		// kwArgs.targetUri: If specified, will override service.servicePath in deferred request.
		// kwArgs.uriAttribute: If specified, will be used as attribute to get value for 'uri' field of every request item.
		//						Please note: 	This is used only while creating request body and not for parsing response items.
		//										Response content item is always have 'uri' as idAttribute.
		// kwArgs.responsePayloadAttribute: attribute to pick response payload from within responseItem.content
		
		kwArgs.uriAttribute = kwArgs.uriAttribute || 'uri';
		 
		/*
		 * Update dojo.xhr (function) to include 'X-HTTP-Method-Override' request header.
		 */
		var dojoXhr = dojo.xhr;
		// A value of "BATCH" must be specified for the "X-HTTP-Method-Override" or the "X-Method-Override" request header parameter
		dojo.xhr = function(method, args) {
			args.headers = args.headers || {};
			args.headers['X-HTTP-Method-Override'] = "BATCH";
			return dojoXhr.apply(dojo, arguments);
		};
		
		/*
		 * Create batch request body content from actions.
		 * This logic depends upon "actions" that are created in JsonRest.commit().
		 * An action does not contain 'content' attribute for delete item action. 
		 */
		var contents = [];	// holds request items data for batch request
		var targetActions = {};		// keeps item identifier to 'action' mapping. This is required later when response arrives.
		for (var i=0; i<actions.length; i++) {
			var action = actions[i];
			var content = {};
			if(action.method.toUpperCase()!='DELETE'){
				//This means that we have to submit a new POST request to the server where in we need to include the 	
				//contents of the POST data in the request.
				//Need to look into how we can make this 
				var serviceAndId = dojox.rpc.JsonRest.getServiceAndId(actions[i].target.__id);	// all actions have same service hence using the first action to reftieve service details.
				var service = serviceAndId.service; // service attached to the store.
				action.content.method=action.method.toUpperCase();
				if(kwArgs.noContent){
					content.method=action.content.method;
					content.uri=action.content.__id;
				}else{
					for (var property in action.content) {
						if (action.content[property] != undefined) {
							content[property] = action.content[property];
						}
					}
					content=dojox.json.ref.toJson(content, false, service.servicePath, true);
					content=dojox.json.ref.fromJson(content);
					content.uri=action.content.__id;
				}
				targetActions[action.content.__id]=action;
			}else{
				targetActions[(action.target[kwArgs.uriAttribute] || action.content[kwArgs.uriAttribute])] = action;
				content = {
						"method": action.method.toUpperCase(),
						"uri": (action.target[kwArgs.uriAttribute] || action.content[kwArgs.uriAttribute])
				};
			}
			contents[contents.length] = content;
		}

		var store = dojox.data._getStoreForItem((actions[0].content || actions[0].target));	// store on which newItem() or deleteItem() was performed.
		var serviceAndId = dojox.rpc.JsonRest.getServiceAndId(actions[0].target.__id);	// all actions have same service hence using the first action to reftieve service details.
		var service = serviceAndId.service; // service attached to the store.
		
		/**
		 * Abstracts the logic of making HTTP POST.
		 * Uses store's service POST method to send HTTP request.
		 * 
		 * The URL to which HTTP POST request is sent:
		 * 	- If kwArgs have 'targetUri' specified, 
		 * 		then request is sent to this URL
		 * 	- Else default target of store's service is used as URL.
		 */
		var doHttpPost = function() {
			// Update Service to send requests to URL specified in kwArgs.targetUri
			
			/*
			 * Old service.getRequest() provides default Request.
			 * This Request needs to be updated for kwArgs.targetUri
			 */
			var oldGetRequest = service._getRequest; 
			if (kwArgs.targetUri && kwArgs.targetUri != '') {
				 
				service._getRequest = function(id, args) {
					
					var request = oldGetRequest(id, args);
					
					/*
					 * Logic to merge additional parameters with the URL.
					 */
					if (request.url.indexOf("?") == -1) { // URL has no additional parameters.
						if (kwArgs.targetUri.lastIndexOf("/") != (kwArgs.targetUri.length-1)) { // and no trailing slash.
							// Add trailing slash.
							kwArgs.targetUri = kwArgs.targetUri +"/";
						}
					} else { // there are additional parameters.
						if (kwArgs.targetUri.lastIndexOf("/") == (kwArgs.targetUri.length-1)) { // trailing slash is present.
							// remove trailing slash
							kwArgs.targetUri = kwArgs.targetUri.substring(0, kwArgs.targetUri.length-1);
						}
						// Keep the additional parameters.
						kwArgs.targetUri = kwArgs.targetUri + request.url.substring(request.url.indexOf("?"));
					}
					
					// Update request URL.
					request.url = kwArgs.targetUri;
					return request;
				};
			}
			
			// Invoke HTTP POST and supply contents.
			var deferred = actions.deferred = service['post'](
				'', 
				dojox.json.ref.toJson(contents, false, service.servicePath, true)
			);
			
			// Retain the old behavior.
			if (kwArgs.targetUri && kwArgs.targetUri != '') { 
				service._getRequest = oldGetRequest;
			}
			
			// return deferred request
			return deferred;
		};
		var dfd = doHttpPost();	// scoped variable, used in callbacks.
		
		/**
		 * Abstracts logic of performing revert().
		 * This logic updates JsonRest:: dirtyObjects variable.
		 * 
		 * And it relies on JsonRest:: revert() method.
		 */
		var revertOnError = function() {
			var postCommitDirtyObjects = [];
			for (var i=0; i<dirtyObjects.length; i++) {
				postCommitDirtyObjects.push(dirtyObjects[i]);
			}
			
			// working on reference from JsonRest
			dirtyObjects.splice(0,dirtyObjects.length);
			var savingObjects = kwArgs.savingObjects;
			for (var i=0; i<savingObjects.length; i++) {
				dirtyObjects.push(savingObjects[i]);
			}
			// revert logic
			dojox.rpc.JsonRest.revert(); 
			
			for (var i=0; i<postCommitDirtyObjects.length; i++) {
				dirtyObjects.push(postCommitDirtyObjects[i]);
			}
		};
		
		/**
		 * Abstracts logic of constructing item Id (key for the Rest._index map)
		 */
		var getItemId = function(newId) {
			// if the path starts in the middle of an absolute URL for Location, we will use the just the path part 
			var startIndex = newId.match(/(^\w+:\/\/)/) && newId.indexOf(service.servicePath);
			return  startIndex > 0 ? newId.substring(startIndex) : (service.servicePath + newId).
					// now do simple relative URL resolution in case of a relative URL. 
					replace(/^(.*\/)?(\w+:\/\/)|[^\/\.]+\/\.\.\/|^.*\/(\/)/,'$2$3');
		};
		
		/**
		 * Abstracts logic of processing response for one response item at a time.
		 * This response item is of a request which is either resulted into success or has been accepted at Rest server.
		 * It takes care of:
		 * 	- Successful Create
		 * 	- Successful Delete
		 * 	- Delete resulting into Update on Rest Server, updating GUI table (if any)
		 * 	- Create resulting into Update on Rest Server, updating GUI table (if any)
		 */
		var processResponseItem = function(action, responseItem) {
			var object = action.content || action.target;	// Update || Delete
			
			// Id update and populate latest data: Invoke resolveJson to update action.content object with latest data.
			if (responseItem.content &&
					(kwArgs.responsePayloadAttribute? responseItem.content[kwArgs.responsePayloadAttribute]: true)) { // action has 'content' except for successful Delete.
				var newId = kwArgs.responsePayloadAttribute ? 
						responseItem.content[kwArgs.responsePayloadAttribute].uri : responseItem.content.uri;
				//TODO review
				delete responseItem.content.uri;
				if(newId){ 
					newId = getItemId(newId);
					
					if(object.__id != newId){	// Old Id and new Id do not match: a) New item or b) New Item resulted in to Update
						if (action.content && dojox.rpc.Rest._index[newId] && 
								(!dojox.rpc.Rest._index[newId].__isDirty)) {	// An ADD was requested and an item already exists with this newId.
							// Hence ADD resulted into an Update.
							// So now table has two records. Remove the old one.
							store.onDelete(dojox.rpc.Rest._index[newId]); // Grid (if any) listens to it, and removes the item from view.
						} 
						
						// Update object id and object reference in index.
						object.__id = newId;
						dojox.rpc.Rest._index[newId] = object;
					}
					
					if (!action.content) { // Delete was requested. And it turned into update.
						delete dojox.rpc.Rest._index[object.__id].__isDirty;
						store.onNew(dojox.rpc.Rest._index[object.__id]);	// Grid (if any) listens to it, and displays the item in view.
					}
				}
				
				// Update the object with response content
				var indexedData = com.ibm.security.modeling.tables.JsonBatchRest.resolveJson(
						service, 
						dfd, 
						(kwArgs.responsePayloadAttribute ? responseItem.content[kwArgs.responsePayloadAttribute] : responseItem.content), 
						object && object.__id);
				//TODO Review
				indexedData.uri = newId ;
				if (kwArgs.responsePayloadAttribute) {
					responseItem.content[kwArgs.responsePayloadAttribute] = indexedData;
				} else {
					responseItem.content = indexedData;
				}
			}
			
		};
		
		/**
		 * Abstracts logic of updating 'dirtyObjects' and 'savingObjects' variables so that revert logic can take place.
		 * As revertOnError() depends upon JsonRest:: revert(), it is required to have only those actions reverted, which failed.
		 * 
		 * As by default JsonRest::revert reverts all dirtyItems.
		 */
		var handleFailedActions = function(succeededActions, failedActions, value) {
			try {
				kwArgs.onError.call(kwArgs.scope, [failedActions, value]);
			} catch (e) {// line number 121 from JsonRest throws exception.
				trace.exception(e, "batchOnCompleteCallback: for failed action.");
			}
			
			// JsonRest explicitly puts a dojo connect on kwArgs "onError". And adds all savingObjects to dirtyObjects 
			// hence removing.
			var savingObjects = kwArgs.savingObjects;
			for (var k=0; k<savingObjects.length; k++) {
				for (var a=0; a<dirtyObjects.length; a++) {
					if ((savingObjects[k].object || savingObjects[k].old) == (dirtyObjects[a].object || dirtyObjects[a].old)) {
						dirtyObjects.splice(a, 1);
						break;
					}
				}
			}
			// keep only those objects as savingObjects which failed.
			for (var l=0; l<succeededActions.length; l++) {
				for (var b=0; b<savingObjects.length; b++) {
					if ((savingObjects[b].object || savingObjects[b].old).uri == (succeededActions[l].target.uri || succeededActions[l].content.uri)) {
						savingObjects.splice(b, 1);
						break;
					}
				}
			}
			// do revert: failed items
			revertOnError();
		};
		
		/**
		 * Abstracts deferred request's onError callback.
		 * 
		 * Invokes error callbacks before reverting.
		 */
		var batchErrorCallbak = function(value) {
			
			try {
				// Batch request failed. Revert batch update.
				kwArgs.onError.call(kwArgs.scope, value);
			} catch (e) {// line number 121 from JsonRest throws exception.
				trace.exception(e, "batchErrorCallbak");
			}
			// do revert: all items
			revertOnError();
			
		};
		
		/**
		 * Abstracts deferred request's onComplete callback.
		 * 
		 * Per the response status (individual item within response) it separates successful actions and failed actions.
		 * And processes successful actions:
		 * 	- Updates Rest._index with new/updated items.
		 * 	- Updates GUI table (if any)
		 * And processes failed actions:
		 * 	- Performs revert
		 * 	- Updates GUI table (if any)
		 * And invokes success, failure (if some actions failed) callbacks.
		 * 
		 */
		var batchOnCompleteCallback = function(value) {
			try {
				// separate actions: succeeded, failed
				var succeededActions = [];
				var failedActions = [];
				succeededActions.deferred = failedActions.deferred = dfd;
				// 1. Process response.
				if (value.content) {
					for (var j=0; j<value.content.length; j++) {
						var responseItem = value.content[j];
						var action = targetActions[responseItem.requestUri];
						
						if (responseItem.statusCode == 201 || responseItem.statusCode == 204 || responseItem.statusCode == 200) {	
							// 	201: add success, 
							// 	204: remove success, 
							//	200: success [delete turned update]
							succeededActions.push(action);
							try {
								processResponseItem(action, responseItem);
							} catch (e) {
								trace.exception(e, "batchOnCompleteCallback: for request: "+ (responseItem.requestUri));
							}
						} else {
							failedActions.push(action);
						}
					}
				} else {	// No content indicates that a all actions failed.
					return batchErrorCallbak(value);
				}
				
				// 2. Invoke call backs
				if (kwArgs.onComplete) { // invoke success callback for succeeded actions
					// Use 'value' to retrieve information (like info/warn/error messages) from response.
					kwArgs.onComplete.call(kwArgs.scope, [succeededActions, value]);	// actions contain 'deferred'
				}
				
				if (failedActions.length > 0) { // invoke error callback and revert, only for failed actions (partial).
					handleFailedActions(succeededActions, failedActions, value);
				}
			} catch (e) {// JavaScript error, would cause store's onError and hence revert. Preventing it.
				trace.exception(e, "batchOnCompleteCallback.");
			}
			return value;
			
		};
		
		
		/**
		 * Attach success, failure callback with deferred request.
		 */
		dfd.addCallback(batchOnCompleteCallback);
		dfd.addErrback(batchErrorCallbak);
		
		// revert dojo xhr to original functionality.
		dojo.xhr = dojoXhr;
	};
	
	
})();