/* 
Licensed Materials - Property of IBM

Role and Policy Modeler

� Copyright IBM Corp. 2011 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or 
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.ibm.security.modeling.widgets.UploadTracker");

dojo.require("com.ibm.security.modeling.util.Trace");
dojo.require("dojo.io.iframe");
dojo.require("com.ibm.security.modeling.util.ConsoleUtilities");

dojo.declare ("com.ibm.security.modeling.widgets.UploadTracker", [dijit._Widget], {
	/**
	 * Summary:
	 * 		In memory store used to process uploading files.  This works inconcert 
	 * 		with the MultiFileInput widget and uses dojo.io.iframe to send files
	 * 		asynchronously in the background.   This class is responsible for
	 * 		queuing files for upload, tracking completion / error, and providing
	 * 		query services / notification support for listeners.  	 
	 * 
	 *  	This is an in-memory store, if it is destroyed with an upload in progress
	 *  	tracking will be lost and potentially the upload itself.  
	 *  
	 *  	Callers can connect to onUpload, onComplete, onError for notification of
	 *  	changes in the tracking queue.  
	 * 
	 */
	_nextUploadId : 0, /*counter used to give a unique id for the tracking store*/
	_uploadCount : 0, /*counter tracking number of files in the queue*/
	
	constructor : function (params) {
		this._trace.entering(this.id, "constructor");
		this.inherited(arguments);
		
		this._trace.exiting(this.id, "constructor");
	},

	postCreate: function() { 
		this._trace.entering(this.id, "postCreate");
		this.inherited(arguments);		
		this._trace.exiting(this.id, "postCreate");
	},
	
	startup: function () {
		this._trace.entering(this.id, "startup");
		this.inherited(arguments);
		this._trace.exiting(this.id, "startup");
	},
	_getNextId : function () {return this._nextUploadId++;},
	_onUpload : function(id, fileName, uploadUri, dfd, callbackData) {
		/**
		 * Summary:
		 * 		Invoked just after upload is queued 
		 * 		storing an item representation of the upload
		 * 		in our tracking store.  Note 'callbackData'
		 * 		is mixed into the item so that when we perform
		 * 		callbacks listners can see custom data
		 * 		passed in by the submitter.  This method is also
		 *      responsible for notifying listers that the upload
		 *      has been queued.  
		 * 
		 * 	@param id int used to track the upload.  Should be unique
		 * 				to this upload tracker
		 * @param fileName 'string' the name of the file being uploaded
		 * @param uploadURI 'string' the uri the file was being sent to
		 * @param dfd 'dojo.deferred' the deferred object for this request
		 * 				this could be used in the future to support 'cancel upload'
		 * @param callbackData Object passed in by submitter to add custom data
		 * 		to the callback item		 
		 * 
		 */
		
		//create the tracking item
		var item = {id : id, fileName : fileName, deferred : dfd, uploadUri: uploadUri};
		//mix in custom callback data
		var mixedItem = dojo.mixin(item, callbackData);
		
		//add it to the tracking store
		this.trackingStorage.addItem(mixedItem);
		//notify listenres
		this.onUpload(mixedItem);
	},
	onUpload : function(item) {
		/**
		 * Listener notification hook
		 */
		
	},
	onComplete : function (item) {
		/**
		 * Listener notification hook
		 */
	},
	onError : function (item, error) {
		/**
		 * Listener notification hook
		 */
	},
	_onComplete : function(data, ioArgs) {
		/**
		 * Summary:
		 * 	Handles the completion of a file upload including book keeping and 
		 *  notification 
		 * 
		 * Note io.iframe has a bug that is preventing it from handling 
		 * error conditions correctly a workaround has been put in that 
		 * sends bad 'Ok' on server responses but sets the 'type' field
		 * of the response to error if an error occurred.  Check this and
		 * notify callers based on the response type
		 * this workaround should be fixed once iframe is fixed
		 * 
		 */
		//decrement count
		this._uploadCount--;
		//remove the form that was used to send the 
		//request.  These are hanging around in the dom
		//so they are available for the iframe on send
		//See MultifileInput for more details
		this._removeTempForm(ioArgs.form);	
		//fetch the item and remove it from the tracking store
		var item = this.trackingStorage.getItemById(ioArgs.args.uploadTrackerId);
		this.trackingStorage.deleteItemById(item.id);
		
		//Notify listeners
		if (data instanceof Error ||data.type && data.type == 'Error') 	{
			this.onError(item, data); 
		} else {
			this.onComplete(item);
		}
	},
	_onError : function(error, dfd) {
		/**
		 * A bug in io.iframe is preventing this from being called
		 * when it is resolved this will invoke the error callback.
		 * 
		 * finish implementing once fixed for now leave a stub
		 */
		alert("Error");
	},
	startUpload : function(filesURI, uploadData, callbackData) {
		/**
		 * Summary:
		 * 		Starts the upload for each of the passed in forms
		 * 		and starts the _onUpload chain
		 */
		
		if (uploadData) {
			for (var i = 0; i < uploadData.length; i++) {	
				var uploadId = this._getNextId();			
				var dfd = this._queueUpload(filesURI, uploadData[i].form, uploadData[i].fileName, uploadId);
				this._uploadCount++;
				
				this._onUpload(uploadId,  uploadData[i].fileName, filesURI, dfd, callbackData);
			}	
		}
	},
	_removeTempForm: function(form) {
		/**
		 * Removes the temporary form left in the dom by MultiFileInput
		 * for use by io.iframe in sending the request
		 * 
		 * @param form the form node to remove
		 */
		dojo.destroy(form);	
	},
	_queueUpload : function(url, form, fileName, uploadTrackerId) {
		/**
		 * Summary:
		 * 	Queues the file for upload using dojo.io.iframe
		 *  connecting handlers for completion and error.
		 *  The 'uploadTrackerId' will be available to our
		 *  callbacks and is used as the 'key' to look up
		 *  the item associated with a given upload.   
		 * 
		 */
		
		var sessionName=ConsoleUtilities.getSessionCookieName();
		var seesionVal=ConsoleUtilities.getSessionCookieVal();
		
		 var dfd = dojo.io.iframe.send({
				method : "post",
				url : this.getEncodeURI(url, sessionName, seesionVal),
				form : form,
				content : {fileName : fileName},
				handle : dojo.hitch(this, this._onComplete),
				error : dojo.hitch(this, this._onError),
				handleAs : "json",
				uploadTrackerId : uploadTrackerId
			});
		return dfd;
	},
	
	getEncodeURI: function(url, name, value){
		return encodeURI(url + "?" + name + "=" + value);
	},
	
	getUploadCount:function() {
		/**
		 * Summary:
		 * 		Gets the current number of active uploads
		 * 
		 * @ruturn int num of uploads in the queue
		 * 
		 * 
		 */
		return this._uploadCount;
	},
	trackingStorage : {
		/**
		 * In memory data structure used to track pending uploads
		 * 
		 * This is a double map tracking by id and file url
		 * fileurl map was intended to support the question 
		 * of 'what files' are queued for url 'x'
		 */
		
		idToItemMap : {}, /*id to item map*/
		uploadUriToItemsMap : {}, /* uri to [item, item, item] map]*/
		addItem : function(item) {
			/**
			 * Adds an item to both maps
			 *  
			 */
			this.idToItemMap[item.id] = item;	
			
			var items = this.uploadUriToItemsMap[item.uploadUri];
			if (!items) {
				this.uploadUriToItemsMap[item.uploadUri] = [item];
			} else {
				items.push(item);
			}
		},
		getItemById : function(id) {
			/**
			 * Summary:
			 * 	Gets an item by id
			 * 
			 * 
			 */
			return this.idToItemMap[id];
		},
		getItemsByUploadUri : function(uploadUri) {
			/**
			 * Summary:
			 * 	Gets the list of items associated
			 *   with the uri
			 * 
			 * 
			 */
			return this.uploadUriToItemsMap[uploadUri];
		},
		deleteItemById : function(id) {
			/**
			 * deletes the items from both maps
			 * 
			 * 
			 */
			var item = this.getItemById(id);
			delete this.idToItemMap[id];
			delete this.uploadUriToItemsMap[item.uploadUri];
		}
	},
	_trace: new com.ibm.security.modeling.util.Trace("com.ibm.security.modeling.widgets.UploadTracker")
	
	
	
});